@@ -4,7 +4,7 @@
 // Description:
 //
 //
-// Author: Christian Hubinger <chubinger@gmail.com>, (C) 2004
+// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2004
 //
 // Copyright: See COPYING file that comes with this distribution
 //
@@ -24,6 +24,7 @@
 #include "../../core/kmfnetzone.h"
 #include "../../core/kmfnethost.h"
 #include "../../core/kmfprotocol.h"
+#include "../../core/kmfprotocolusage.h"
 #include "../../core/iptable.h"
 #include "../../core/iptchain.h"
 #include "../../core/iptrule.h"
@@ -31,14 +32,15 @@
 #include "../../core/kmferror.h"
 #include "../../core/kmferrorhandler.h"
 #include "../../core/kmfconfig.h"
+#include "../../core/xmlnames.h"
 
 
-
-
+namespace KMF {
 KMFIPTablesDocumentConverter::KMFIPTablesDocumentConverter() {
 
 	m_errorHandler = new KMFErrorHandler( "KMFIPTablesDocumentConverter" );
 	m_err = new KMFError();
+	m_iptdoc = 0;
 
 }
 
@@ -53,52 +55,56 @@ KMFIPTDoc* KMFIPTablesDocumentConverter::compileToIPTDoc( KMFGenericDoc* doc ) {
 		return 0;
 	}
 	//	kdDebug() << "Doc XLM:\n" << doc->getXMLSniplet() << endl;
-	KMFIPTDoc *iptdoc = new KMFIPTDoc( 0, "iptdoc" );
+	
+	m_iptdoc = new KMFIPTDoc( 0, "iptdoc", doc->target() );
 	KMFNetZone *zone = 0;
 	IPTable *table = 0;
 	IPTChain *chain = 0;
 
-	setupInAndOutHosts( iptdoc, doc->trustedHostsZone(), "ACCEPT" );
-	setupInAndOutHosts( iptdoc, doc->maliciousHostsZone(), "DROP" );
-	setupForbiddenHosts( iptdoc, doc->badClientsHostsZone(), "in" );
-	setupForbiddenHosts( iptdoc, doc->badServersHostsZone(), "out" );
+	setupInAndOutHosts( m_iptdoc, doc->trustedHostsZone(), "ACCEPT" );
+	setupInAndOutHosts( m_iptdoc, doc->maliciousHostsZone(), "DROP" );
+	setupForbiddenHosts( m_iptdoc, doc->badClientsHostsZone(), "in" );
+	setupForbiddenHosts( m_iptdoc, doc->badServersHostsZone(), "out" );
+	
+	setupICMPRules( doc, m_iptdoc );
+	setupLocalhostRules( doc, m_iptdoc );
 	
-	setupICMPRules( doc, iptdoc );
-
 	if ( doc->allowIncomingConnections() ) {
 		zone = doc->incomingZone();
-		table = iptdoc->table( "filter" );
-		chain  = table->chainForName( *(new QString("INPUT") ) );
-		addToChains(  zone, iptdoc, chain, "INPUT");
+		table = m_iptdoc->table( Constants::FilterTable_Name );
+		chain  = table->chainForName( Constants::InputChain_Name );
+		addToChains(  zone, m_iptdoc, chain, Constants::InputChain_Name);
 	}
 
 	if ( doc->restrictOutgoingConnections() ) {
 		zone = doc->outgoingZone();
-		chain  = table->chainForName( *(new QString("OUTPUT") ) );
-		addToChains(  zone, iptdoc, chain, "OUTPUT" );
+		table = m_iptdoc->table( Constants::FilterTable_Name );
+		chain  = table->chainForName( Constants::OutputChain_Name );
+		addToChains(  zone, m_iptdoc, chain, Constants::OutputChain_Name);
 	}
+	
+	
+	setupConnectionTracking( m_iptdoc );
+	setupPolicies( doc, m_iptdoc );
 
-	setupConnectionTracking( iptdoc );
-	setupPolicies( doc, iptdoc );
+	setupNatRules( doc, m_iptdoc );
+	setupLogging( doc, m_iptdoc );
 
-	setupNatRules( doc, iptdoc );
-	setupLogging( doc, iptdoc );
-
-	return iptdoc;
+	return m_iptdoc;
 }
 
 void KMFIPTablesDocumentConverter::setupConnectionTracking( KMFIPTDoc* doc ) {
 	kdDebug() << "void KMFIPTablesCompiler::setupConnectionTracking( KMFIPTDoc* doc )" << endl;
 
-	IPTable *table = doc->table( "filter" );
-	IPTChain *chain  = table->chainForName( *(new QString("INPUT") ) );
+	IPTable *table = doc->table( Constants::FilterTable_Name );
+	IPTChain *chain  = table->chainForName( Constants::InputChain_Name );
 
 	IPTRule *rule = chain->addRule( "CONNTRACK", m_err );
 	if ( ! m_errorHandler->showError( m_err ) ) {
 		return;
 	}
 	QPtrList<QString> args;
-	args.append( new QString("bool:on") );
+	args.append( new QString(XML::BoolOn_Value) );
 	args.append( new QString("RELATED,ESTABLISHED") );
 	QString  opt = "state_opt";
 	rule->addRuleOption( opt, args );
@@ -111,12 +117,55 @@ void KMFIPTablesDocumentConverter::setupConnectionTracking( KMFIPTDoc* doc ) {
 	                            "others send you to your requests.") );
 }
 
+void  KMFIPTablesDocumentConverter::setupLocalhostRules( KMFGenericDoc* gendoc,  KMFIPTDoc* doc ){
+	kdDebug() << "void KMFIPTablesCompiler::setupConnectionTracking( KMFGenericDoc* gendoc,  KMFIPTDoc* doc )" << endl;
+
+	IPTable *table = doc->table( Constants::FilterTable_Name );
+	IPTChain *chain  = table->chainForName( Constants::InputChain_Name );
+
+	IPTRule *rule = chain->addRule( "LOCALHOST", m_err );
+	if ( ! m_errorHandler->showError( m_err ) ) {
+		return;
+	}
+	QPtrList<QString> args;
+	args.append( new QString( Constants::Localhost_IP ) );
+	args.append( new QString( XML::BoolOff_Value ) );
+	QString  opt = "ip_opt";
+	rule->addRuleOption( opt, args );
+	rule->setTarget( "ACCEPT" );
+	
+	args.clear();
+	opt = "interface_opt";
+	args.append( new QString( "lo" ) );
+	args.append( new QString( XML::BoolOff_Value ) );
+	rule->addRuleOption( opt, args );
+	rule->setDescription( i18n( "Allows all localhost traffic" ) );
+	
+	
+	if ( gendoc->restrictOutgoingConnections() ) {
+		chain = table->chainForName( Constants::OutputChain_Name );
+		rule = chain->addRule( "LOCALHOST", m_err );
+		if ( ! m_errorHandler->showError( m_err ) ) {
+			return;
+		}
+	
+		args.clear();
+		opt = "interface_opt";
+		args.append( new QString( XML::BoolOff_Value ) );
+		args.append( new QString( "lo" ) );
+		rule->addRuleOption( opt, args );
+		rule->setTarget( "ACCEPT" );
+		rule->setDescription( i18n( "Allows all localhost traffic" ) );
+	}
+	
+}
+
 void KMFIPTablesDocumentConverter::setupPolicies( KMFGenericDoc* gendoc, KMFIPTDoc* iptdoc ) {
 	kdDebug() << "void KMFIPTablesCompiler::setupPolicies( KMFGenericDoc* gendoc, KMFIPTDoc* iptdoc )" << endl;
-	IPTable *table = iptdoc->table( "filter" );
-	IPTChain *chain  = table->chainForName( *(new QString("INPUT") ) );
+	IPTable *table = iptdoc->table( Constants::FilterTable_Name );
+	IPTChain *chain  = table->chainForName( Constants::InputChain_Name );
 	chain->setDefaultTarget( "DROP" );
-	chain  = table->chainForName( *(new QString("OUTPUT") ) );
+	chain  = table->chainForName( Constants::OutputChain_Name );
 
 	if ( gendoc->restrictOutgoingConnections() ) {
 		chain->setDefaultTarget( "DROP" );
@@ -133,14 +182,14 @@ void KMFIPTablesDocumentConverter::addToChains( KMFNetZone* zone, KMFIPTDoc* doc
 		addToChains(  it.current(), doc, chain, root_chain );
 		++it;
 	}
-	IPTable *table = doc->table( "filter" );
+	IPTable *table = doc->table( Constants::FilterTable_Name );
 	QString num = "";
 	num.setNum( i );
 	QString name = "";
 
-	if ( root_chain == "INPUT" ) {
+	if ( root_chain == Constants::InputChain_Name ) {
 		name = "IZ_" + num;
-	} else if ( root_chain == "OUTPUT" ) {
+	} else if ( root_chain == Constants::OutputChain_Name ) {
 		name = "OZ_" + num;
 	}
 
@@ -175,11 +224,11 @@ void KMFIPTablesDocumentConverter::addToChains( KMFNetZone* zone, KMFIPTDoc* doc
 		ch->setDescription( i18n("The Chain created to handle\nrules defined in zone %1.").arg( zone->guiName() ) );
 		
 		QPtrList<QString> args;
-		if ( root_chain == "INPUT" ) {
+		if ( root_chain == Constants::InputChain_Name ) {
 			args.append( new QString( zone->address()->toString()+"/"+zone->mask()->toString() ) );
-			args.append( new QString( "bool:off" ) );
-		} else if ( root_chain == "OUTPUT" ) {
-			args.append( new QString( "bool:off" ) );
+			args.append( new QString( XML::BoolOff_Value ) );
+		} else if ( root_chain == Constants::OutputChain_Name ) {
+			args.append( new QString( XML::BoolOff_Value ) );
 			args.append( new QString( zone->address()->toString()+"/"+zone->mask()->toString() ) );
 		}
 		QString s ="ip_opt";
@@ -192,32 +241,34 @@ void KMFIPTablesDocumentConverter::addToChains( KMFNetZone* zone, KMFIPTDoc* doc
 }
 
 void KMFIPTablesDocumentConverter::createRules( KMFNetZone* zone, IPTChain* chain, const QString& root_chain ) {
-	QPtrList<KMFProtocol>& prots = zone->protocols();
-	QPtrListIterator<KMFProtocol> it ( prots );
+	QPtrList<KMFProtocolUsage>& prots = zone->protocols();
+	QPtrListIterator<KMFProtocolUsage> it ( prots );
 	while ( it.current() ) {
-		KMFProtocol* prot = it.current();
-		if ( ! zone->protocolInherited( prot->name() ) ) {
+		KMFProtocolUsage* prot = it.current();
+		if ( ! zone->protocolInherited( prot->protocol()->uuid() ) ) {
 			createZoneProtocolRules( chain, prot );
 		} else {
-			kdDebug() << "Skipping inherited Portocol: " << prot->name() << " in zone: " << zone->guiName() << endl;
+			kdDebug() << "Skipping inherited Portocol: " << prot->protocol()->name() << " in zone: " << zone->guiName() << endl;
 		}
 		++it;
 	}
 
-	QPtrList<KMFNetHost>& hosts = zone->hosts();
-	QPtrListIterator<KMFNetHost> it2 ( hosts );
+	QPtrList<KMFTarget>& hosts = zone->hosts();
+	QPtrListIterator<KMFTarget> it2 ( hosts );
 	while ( it2.current() ) {
-		KMFNetHost* host = it2.current();
+		KMFNetHost* host = dynamic_cast<KMFNetHost*> ( it2.current() );
 		kdDebug() << "Will create rules for host: " << host->guiName() << " in zone:" << zone->guiName() << endl;
-		QPtrList<KMFProtocol>& prots = host->protocols();
-		QPtrListIterator<KMFProtocol> it3 ( prots );
+		QPtrList<KMFProtocolUsage>& prots = host->protocols();
+		QPtrListIterator<KMFProtocolUsage> it3 ( prots );
 		while ( it3.current() ) {
-			KMFProtocol* prot = it3.current();
-			if ( ! host->protocolInherited( prot->name() ) ) {
-				kdDebug() << "Found Protocol: " << prot->name() << endl;
-				createHostProtocolRules( chain, host, prot, root_chain );
+			KMFProtocolUsage* protUsage = it3.current();
+			kdDebug() << "Found Stored Usage" << endl;
+			kdDebug() << "  Name: " << protUsage->name() << endl;
+			if ( ! host->protocolInherited( protUsage->protocol()->uuid() ) ) {
+				kdDebug() << "Found Protocol: " << protUsage->protocol()->name() << endl;
+				createHostProtocolRules( chain, host, protUsage, root_chain );
 			} else {
-				kdDebug() << "Skipping inherited Portocol: " << prot->name() << " in host: " << host->guiName() << endl;
+				kdDebug() << "Skipping inherited Portocol: " << protUsage->protocol()->name() << " in host: " << host->guiName() << endl;
 			}
 			++it3;
 		}
@@ -225,29 +276,29 @@ void KMFIPTablesDocumentConverter::createRules( KMFNetZone* zone, IPTChain* chai
 	}
 }
 
-void KMFIPTablesDocumentConverter::createZoneProtocolRules( IPTChain* chain, KMFProtocol* prot ) {
+void KMFIPTablesDocumentConverter::createZoneProtocolRules( IPTChain* chain, KMFProtocolUsage* prot ) {
 	kdDebug() << "void KMFIPTablesCompiler::createProtocolRules( ITPChain* chain, KMFProtocol* protocol )" << endl;
-	QStringList tcpPorts = prot->tcpPorts();
-	QStringList udpPorts = prot->udpPorts();
-	if ( ! tcpPorts.empty() ) {
+	const QString& tcpPorts = prot->protocol()->tcpPortsList();
+	const QString& udpPorts = prot->protocol()->udpPortsList();
+	if ( ! tcpPorts.isEmpty() ) {
 		createZoneProtocol( chain, prot, "tcp", tcpPorts );
 	}
-	if ( ! udpPorts.empty() ) {
+	if ( ! udpPorts.isEmpty() ) {
 		createZoneProtocol( chain, prot, "udp", udpPorts );
 	}
 }
 
-void KMFIPTablesDocumentConverter::createZoneProtocol( IPTChain* chain, KMFProtocol* prot, const QString& option, QStringList ports ) {
+void KMFIPTablesDocumentConverter::createZoneProtocol( IPTChain* chain, KMFProtocolUsage* prot, const QString& option, const QString& ports ) {
 	kdDebug() << "void KMFIPTablesCompiler::createProtocol( IPTChain*, const QString& option, QStringList ports )" << endl;
 	QString	s;
 	QPtrList<QString> args;
 	args.clear();
-	args.append( new QString( "bool:on" ) );
-	args.append( new QString( "bool:off" ) );
+	args.append( new QString( XML::BoolOn_Value ) );
+	args.append( new QString( XML::BoolOff_Value ) );
 
 	IPTRule* rule;
-	rule = chain->addRule( prot->name()+ "_" + option , m_err );
-	if ( ports.count() > 1 ) {
+	rule = chain->addRule( prot->protocol()->name()+ "_" + option , m_err );
+	if ( ports.contains( "," ) > 0 ) {
 		s = option + "_multiport_opt";
 	} else {
 		s = option + "_opt";
@@ -257,16 +308,12 @@ void KMFIPTablesDocumentConverter::createZoneProtocol( IPTChain* chain, KMFProto
 		return;
 
 	rule->addRuleOption( s , args );
-	rule->setDescription( prot->description() );
+	rule->setDescription( i18n(	"Allow Protocol: %1\n"
+	                            "Protocol Description: %2" ).arg( prot->protocol()->name( ) ).arg( prot->protocol()->description() ) );
 
-	QString arg = "";
-	for ( QStringList::Iterator it = ports.begin(); it != ports.end(); ++it ) {
-		arg += *it +",";
-	}
-	if ( arg.endsWith( "," ) ) {
-		arg = arg.left( arg.length() -1 );
-	}
-	args.append( new QString( arg ) );
+	
+	rule->setDescription( prot->protocol()->description() );
+	args.append( new QString( ports ) );
 	rule->addRuleOption( s, args );
 
 	if ( prot->logging() ) {
@@ -276,7 +323,7 @@ void KMFIPTablesDocumentConverter::createZoneProtocol( IPTChain* chain, KMFProto
 	if ( prot->limit() > 0 ) {
 		s = "limit_opt";
 		args.clear();
-		args.append( new QString("bool:on") );
+		args.append( new QString(XML::BoolOn_Value) );
 		QString limit;
 		limit.setNum( prot->limit() );
 		limit += "/" + prot->limitInterval();
@@ -288,25 +335,25 @@ void KMFIPTablesDocumentConverter::createZoneProtocol( IPTChain* chain, KMFProto
 }
 
 
-void KMFIPTablesDocumentConverter::createHostProtocolRules( IPTChain* chain, KMFNetHost* host,  KMFProtocol* prot, const QString& root_chain ) {
+void KMFIPTablesDocumentConverter::createHostProtocolRules( IPTChain* chain, KMFNetHost* host,  KMFProtocolUsage* prot, const QString& root_chain ) {
 	kdDebug() << "void KMFIPTablesCompiler::createProtocolRules( ITPChain* chain, KMFProtocol* protocol )" << endl;
-	QStringList tcpPorts = prot->tcpPorts();
-	QStringList udpPorts = prot->udpPorts();
-	if ( ! tcpPorts.empty() ) {
+	const QString& tcpPorts = prot->protocol()->tcpPortsList();
+	const QString& udpPorts = prot->protocol()->udpPortsList();
+	if ( ! tcpPorts.isEmpty() ) {
 		createHostProtocol( chain, host, prot, "tcp", tcpPorts, root_chain );
 	}
-	if ( ! udpPorts.empty() ) {
+	if ( ! udpPorts.isEmpty() ) {
 		createHostProtocol( chain, host, prot, "udp", udpPorts, root_chain );
 	}
 }
 
-void KMFIPTablesDocumentConverter::createHostProtocol( IPTChain* chain, KMFNetHost* host, KMFProtocol* prot, const QString& option, QStringList ports, const QString& root_chain  ) {
+void KMFIPTablesDocumentConverter::createHostProtocol( IPTChain* chain, KMFNetHost* host, KMFProtocolUsage* prot, const QString& option, const QString& ports, const QString& root_chain  ) {
 	kdDebug() << "void KMFIPTablesCompiler::createProtocol( IPTChain*, const QString& option, QStringList ports )" << endl;
 	QString	s;
 	QPtrList<QString> args;
 	args.clear();
-	args.append( new QString( "bool:on" ) );
-	args.append( new QString( "bool:off" ) );
+	args.append( new QString( XML::BoolOn_Value ) );
+	args.append( new QString( XML::BoolOff_Value ) );
 	static int i = 0;
 
 	IPTRule* rule;
@@ -314,31 +361,21 @@ void KMFIPTablesDocumentConverter::createHostProtocol( IPTChain* chain, KMFNetHo
 	hn = hn.setNum( i );
 	i++;
 	hn = "H" + hn;
-	rule = chain->addRule( hn + "_" + prot->name() + "_" + option , m_err );
-	if ( ports.count() > 1 ) {
+	rule = chain->addRule( hn + "_" + prot->protocol()->name() + "_" + option , m_err );
+	if ( ports.contains( "," ) > 0 ) {
 		s = option + "_multiport_opt";
 	} else {
 		s = option + "_opt";
 	}
 	rule->setDescription( i18n(	"Rule created to apply filters for host: %1\n"
 	                            "Allow Protocol: %2\n"
-	                            "Protocol Description: %3" ).arg( host->guiName() ).arg( prot->name( ) ).arg( prot->description() ) );
+	                            "Protocol Description: %3" ).arg( host->guiName() ).arg( prot->protocol()->name( ) ).arg( prot->protocol()->description() ) );
 	if ( ! m_errorHandler->showError( m_err ) )
 		return;
 
 	rule->addRuleOption( s , args );
-	//rule->setDescription( prot->description() );
 
-	QString arg = "";
-	for ( QStringList::Iterator it = ports.begin(); it != ports.end(); ++it ) {
-		kdDebug() << "Found " << option << " port "  <<  *it << endl;
-		arg += *it +",";
-	}
-	if ( arg.endsWith( "," ) ) {
-		arg = arg.left( arg.length() -1 );
-	}
-	kdDebug() << "Adding " << option << " port string: "  <<  arg << endl;
-	args.append( new QString( arg ) );
+	args.append( new QString( ports ) );
 	rule->addRuleOption( s, args );
 
 	if ( prot->logging() ) {
@@ -348,7 +385,7 @@ void KMFIPTablesDocumentConverter::createHostProtocol( IPTChain* chain, KMFNetHo
 	if ( prot->limit() > 0 ) {
 		s = "limit_opt";
 		args.clear();
-		args.append( new QString("bool:on") );
+		args.append( new QString(XML::BoolOn_Value) );
 		QString limit;
 		limit.setNum( prot->limit() );
 		limit += "/" + prot->limitInterval();
@@ -358,8 +395,8 @@ void KMFIPTablesDocumentConverter::createHostProtocol( IPTChain* chain, KMFNetHo
 	}
 
 	args.clear();
-	if ( root_chain == "OUTPUT" ) {
-		args.append( new QString( "bool:off" ) );
+	if ( root_chain == Constants::OutputChain_Name ) {
+		args.append( new QString( XML::BoolOff_Value ) );
 	}
 	s = "ip_opt";
 	args.append( new QString( host->address()->toString() ) );
@@ -369,11 +406,11 @@ void KMFIPTablesDocumentConverter::createHostProtocol( IPTChain* chain, KMFNetHo
 
 void KMFIPTablesDocumentConverter::setupInAndOutHosts( KMFIPTDoc* iptdoc, KMFNetZone* zone, const QString& target ) {
 	kdDebug() << "KMFIPTablesCompiler::setupTrustedHosts( KMFNetZone* )" << endl;
-	QPtrListIterator<KMFNetHost> it ( zone->hosts() );
+	QPtrListIterator<KMFTarget> it ( zone->hosts() );
 	int i = 0;
 	while ( it.current() ) {
-		KMFNetHost *host = *it;
-		IPTable *table = iptdoc->table( "filter" );
+		KMFNetHost *host = dynamic_cast<KMFNetHost*> ( *it );
+		IPTable *table = iptdoc->table( Constants::FilterTable_Name );
 		IPTChain *chain;
 		IPTRule *rule;
 		QString ruleName = "";
@@ -386,7 +423,7 @@ void KMFIPTablesDocumentConverter::setupInAndOutHosts( KMFIPTDoc* iptdoc, KMFNet
 		QString opt  = "ip_opt";
 		QPtrList<QString> args;
 
-		chain = table->chainForName( *(new QString("INPUT") ) );
+		chain = table->chainForName( Constants::InputChain_Name );
 		rule = chain->addRule( ruleName, m_err );
 		if ( ! m_errorHandler->showError( m_err ) )
 			return;
@@ -405,10 +442,10 @@ void KMFIPTablesDocumentConverter::setupInAndOutHosts( KMFIPTDoc* iptdoc, KMFNet
 		}
 
 
-		chain  = table->chainForName( *(new QString("OUTPUT") ) );
+		chain  = table->chainForName( Constants::OutputChain_Name );
 		rule = chain->addRule( ruleName, m_err );
 		args.clear();
-		args.append( new QString( "bool:off" ) );
+		args.append( new QString( XML::BoolOff_Value ) );
 		args.append( new QString( host->address()->toString() ) );
 		rule->addRuleOption( opt, args );
 		if ( ! m_errorHandler->showError( m_err ) )
@@ -430,11 +467,11 @@ void KMFIPTablesDocumentConverter::setupInAndOutHosts( KMFIPTDoc* iptdoc, KMFNet
 }
 
 void KMFIPTablesDocumentConverter::setupForbiddenHosts( KMFIPTDoc* iptdoc , KMFNetZone* zone, const QString& inOut ) {
-	QPtrListIterator<KMFNetHost> it ( zone->hosts() );
+	QPtrListIterator<KMFTarget> it ( zone->hosts() );
 	int i = 0;
 	while ( it.current() ) {
-		KMFNetHost *host = *it;
-		IPTable *table = iptdoc->table( "filter" );
+		KMFNetHost *host = dynamic_cast<KMFNetHost*> ( *it );
+		IPTable *table = iptdoc->table( Constants::FilterTable_Name );
 		IPTChain *chain;
 		IPTRule *rule;
 		QString ruleName = "";
@@ -448,16 +485,16 @@ void KMFIPTablesDocumentConverter::setupForbiddenHosts( KMFIPTDoc* iptdoc , KMFN
 		QPtrList<QString> args;
 
 		if ( inOut == "in" ) {
-			chain = table->chainForName( *(new QString("INPUT") ) );
+			chain = table->chainForName( Constants::InputChain_Name );
 		} else {
-			chain = table->chainForName( *(new QString("OUTPUT") ) );
+			chain = table->chainForName( Constants::OutputChain_Name );
 		}
 		rule = chain->addRule( ruleName, m_err );
 		if ( ! m_errorHandler->showError( m_err ) )
 			return;
 
 		if ( inOut == "out" ) {
-			args.append( new QString("bool:off") );
+			args.append( new QString(XML::BoolOff_Value) );
 		}
 		args.append( new QString( host->address()->toString() ) );
 		rule->addRuleOption( opt, args );
@@ -486,13 +523,13 @@ void KMFIPTablesDocumentConverter::setupNatRules( KMFGenericDoc* doc, KMFIPTDoc*
 		return;
 	}
 
-	IPTable* table = iptdoc->table( "nat" );
+	IPTable* table = iptdoc->table( Constants::NatTable_Name );
 	if ( ! table ) {
 		kdDebug() << "ERROR: Couldn't find table nat!!!" << endl;
 		return;
 	}
 
-	IPTChain* chain = table->chainForName( *(new QString("POSTROUTING") ) );
+	IPTChain* chain = table->chainForName( Constants::PostRoutingChain_Name );
 	if ( ! chain ) {
 		kdDebug() << "ERROR: Couldn't find chain POSTROUTING!!!" << endl;
 		return;
@@ -508,7 +545,7 @@ void KMFIPTablesDocumentConverter::setupNatRules( KMFGenericDoc* doc, KMFIPTDoc*
 
 	QString opt  = "interface_opt";
 	QPtrList<QString> args;
-	args.append( new QString( "bool:off" ) );
+	args.append( new QString( XML::BoolOff_Value ) );
 	args.append( new QString( doc->outgoingInterface() ) );
 	rule->addRuleOption( opt, args );
 	setupNatTarget( doc, rule );
@@ -530,13 +567,13 @@ void KMFIPTablesDocumentConverter::setupLogging( KMFGenericDoc* doc, KMFIPTDoc*
 	if ( ! doc->logDropped() ) {
 		return;
 	}
-	IPTable* table = iptdoc->table( "filter" );
+	IPTable* table = iptdoc->table( Constants::FilterTable_Name );
 	if ( ! table ) {
 		kdDebug() << "ERROR: Couldn't find table filter!!!" << endl;
 		return;
 	}
 
-	IPTChain* chain = table->chainForName( *(new QString("INPUT") ) );
+	IPTChain* chain = table->chainForName( Constants::InputChain_Name );
 	if ( ! chain ) {
 		kdDebug() << "ERROR: Couldn't find chain INPUT!!!" << endl;
 		return;
@@ -545,7 +582,7 @@ void KMFIPTablesDocumentConverter::setupLogging( KMFGenericDoc* doc, KMFIPTDoc*
 	setupLoggingRules( doc, chain );
 	
 	if ( doc->restrictOutgoingConnections() ) {
-		chain = table->chainForName( *(new QString("OUTPUT") ) );
+		chain = table->chainForName( Constants::OutputChain_Name );
 		if ( ! chain ) {
 			kdDebug() << "ERROR: Couldn't find chain INPUT!!!" << endl;
 			return;
@@ -571,13 +608,13 @@ void KMFIPTablesDocumentConverter::setupICMPRules( KMFGenericDoc* doc, KMFIPTDoc
 	if ( ! doc->allowPingReply() ) {
 		return;
 	}
-	IPTable* table = iptdoc->table( "filter" );
+	IPTable* table = iptdoc->table( Constants::FilterTable_Name );
 	if ( ! table ) {
 		kdDebug() << "ERROR: Couldn't find table filter!!!" << endl;
 		return;
 	}
 
-	IPTChain* chain = table->chainForName( *(new QString("INPUT") ) );
+	IPTChain* chain = table->chainForName( Constants::InputChain_Name );
 	if ( ! chain ) {
 		kdDebug() << "ERROR: Couldn't find chain INPUT!!!" << endl;
 		return;
@@ -590,7 +627,7 @@ void KMFIPTablesDocumentConverter::setupICMPRules( KMFGenericDoc* doc, KMFIPTDoc
 	rule->setDescription( i18n("Rule to setup the ICMP Ping policy.") );
 	QString opt  = "icmp_opt";
 	QPtrList<QString> args;
-	args.append( new QString( "bool:on" ) );
+	args.append( new QString( XML::BoolOn_Value ) );
 	args.append( new QString( "echo-request" ) );
 	rule->addRuleOption( opt, args );
 	rule->setTarget( "ACCEPT" );
@@ -598,7 +635,7 @@ void KMFIPTablesDocumentConverter::setupICMPRules( KMFGenericDoc* doc, KMFIPTDoc
 	if ( doc->limitPingReply() ) {
 		args.clear();
 		QString opt  = "limit_opt";
-		args.append( new QString( "bool:on" ) );
+		args.append( new QString( XML::BoolOn_Value ) );
 		args.append( new QString( "5/second" ) );
 		args.append( new QString( "5" ) );
 		rule->addRuleOption( opt, args );
@@ -606,7 +643,7 @@ void KMFIPTablesDocumentConverter::setupICMPRules( KMFGenericDoc* doc, KMFIPTDoc
 	
 	
 	if ( doc->restrictOutgoingConnections() ) {
-		chain = table->chainForName( *(new QString("OUTPUT") ) );
+		chain = table->chainForName( Constants::OutputChain_Name );
 		if ( ! chain ) {
 			kdDebug() << "ERROR: Couldn't find chain INPUT!!!" << endl;
 			return;
@@ -619,16 +656,16 @@ void KMFIPTablesDocumentConverter::setupICMPRules( KMFGenericDoc* doc, KMFIPTDoc
 		rule->setDescription( i18n("Rule to setup the ICMP Ping policy.") );
 		QString opt  = "icmp_opt";
 		args.clear();
-		args.append( new QString( "bool:on" ) );
+		args.append( new QString( XML::BoolOn_Value ) );
 		args.append( new QString( "echo-request" ) );
 		rule->addRuleOption( opt, args );
 		rule->setTarget( "ACCEPT" );
 	}
 }
+}
 
 
 
 
 
 
-
