@@ -4,7 +4,7 @@
 // Description: 
 //
 //
-// Author: Christian Hubinger <chubinger@gmail.com>, (C) 2004
+// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2004
 //
 // Copyright: See COPYING file that comes with this distribution
 // License: GPL
@@ -18,6 +18,7 @@
 // QT includes
 #include <qstring.h>
 #include <qobject.h>
+#include <qtextstream.h>
 
 // KDE includes
 #include <kparts/part.h>
@@ -25,6 +26,7 @@
 #include <kparts/factory.h>
 #include <kxmlgui.h>
 
+namespace KMF {
 class IPTable;
 class KMFIPTDoc;
 
@@ -38,18 +40,21 @@ public:
 	
 	
 private:
+	// Script Generating methods, acting on m_stream
+	void printScriptTableRules( IPTable *tbl );
+	void printScriptTableChainDefinition( IPTable *tbl );
+	void printScriptStartFunction();
+	void printScriptStopFunction();
+	void printScriptModuleLoad();
+	void printScriptHeader(); 
+	void printScriptExecLogic(); 
+	
 	const QString& printScriptDebug( const QString&, bool newLine = true);
-	const QString& printScriptTableRules( IPTable *tbl );
-	const QString& printScriptTableChainDefinition( IPTable *tbl );
-	const QString& printScriptStartFunction();
-	const QString& printScriptStopFunction();
-	const QString& printScriptModuleLoad();
-	const QString& printScriptHeader(); 
-	const QString& printScriptExecLogic(); 
+
 	
 private: // DATA
 	KMFIPTDoc *m_iptDoc;
-
+	QTextOStream *m_stream;
 };
-
+}
 #endif
