@@ -1,7 +1,7 @@
 /***************************************************************************
 begin                : Tue Aug 20 2002
 copyright            : (C) 2002 by Christian Hubinger
-email                : chubinger@gmail.com
+email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -25,22 +25,26 @@ email                : chubinger@gmail.com
 
 // qt includes
 #include <qptrlist.h>
-
+#include <quuid.h>
 // project includes
 #include "iptchain.h"
-
+// qt classes
+class QString;
+namespace KMF {
 // project classes
 class KMFError;
 class KMFIPTDoc;
-// qt classes
-class QString;
+
 
 class KDE_EXPORT IPTable : public NetfilterObject {
 public:
 	/** initialise a IPTable with the given name */
-	IPTable( KMFIPTDoc*, const QString& name );
+	IPTable( KMFIPTDoc*, const char* name, const QString& tableName );
 	~IPTable();
 
+	virtual int type();
+	virtual void clear();
+	
 	/** Set the used flag */
 	void setUsed( bool used ) {
 		m_used = used;
@@ -50,14 +54,14 @@ public:
 	void reset();
 
 	/** Return the IPTChain with the name provided */
-	IPTChain* chainForName( QString& );
+	IPTChain* chainForName( const QString& );
 	
 	/** Return the IPTChain for the given ID */
-	IPTChain* chainForID( int );
+	IPTChain* chainForUuid( const QUuid& );
 
 	/**  Add a new IPTChain object to this table with the given prperties */
-	IPTChain* addChain( QString& chain_name,
-	                    QString& chain_target,
+	IPTChain* addChain( const QString& chain_name,
+	                    const QString& chain_target,
 	                    bool builtin,
 	                    KMFError* );
 
@@ -87,10 +91,13 @@ public:
 	QPtrList<IPTChain>& chains() const;
 
 	/** Return a QDomDocument representation of this Table */
-	const QDomDocument& getDOMTree();
+	virtual const QDomDocument& getDOMTree();
 	
 	/** Load the Table from the QDomDocument */
-	void loadXML( const QDomDocument& );
+	virtual void loadXML( const QDomDocument&, QStringList& errors );
+	
+	/** Load Table From DomNode */
+	virtual void loadXML( QDomNode, QStringList& errors );
 	
 	/** Stup the built-in chains */
 	void settupDefaultChains();
@@ -106,7 +113,7 @@ private:
 	private: KMFError* m_err;
 	private: KMFIPTDoc* kmfdoc;
 };
-
+}
 #endif
 
 
