@@ -1,7 +1,7 @@
 /***************************************************************************
  begin                : Mon Jan 28 2002
  copyright            : (C) 2002 by Christian Hubinger
- email                : chubinger@gmail.com
+ email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -27,36 +27,57 @@
 #include "iptrule.h"
 #include "iptable.h"
 #include "kmferror.h"
+#include "xmlnames.h"
 
-IPTChain::IPTChain( IPTable* table, const QString& name,  bool buildin ) : NetfilterObject( table ) {
+namespace KMF {
+
+IPTChain::IPTChain( IPTable* table, const char* name, const QString& chainName,  bool buildin ) : NetfilterObject( table, name ) {
 	// 	kdDebug() << "IPTChain::IPTChain(const QString& name,IPTable* table,bool buildin)" << endl;
 	m_err = new KMFError();
-	m_name = "UNDEFINED";
-	m_object_type = NetfilterObject::CHAIN;
-	setName( name );
+	
+	// m_object_type = NetfilterObject::CHAIN;
+	setName( chainName );
 	setTable( table );
 	is_build_in_chain = false;
 	has_default_target = false;
 	setBuildIn( buildin );
 	enable_log = false;
-	m_ruleset.setAutoDelete( true );
-	m_cmd_default_target = "UNDEFINED";
-	m_cmd_chain_definition = "UNDEFINED";
-	m_log_limit = "UNDEFINED";
-	m_log_prefix = "UNDEFINED";
-	m_log_burst = "UNDEFINED";
+	
+	
+	m_ruleset.setAutoDelete( false );
+	
+	m_cmd_default_target = XML::Undefined_Value;
+	m_cmd_chain_definition = XML::Undefined_Value;
+	m_log_limit = XML::Undefined_Value;
+	m_log_prefix = XML::Undefined_Value;
+	m_log_burst = XML::Undefined_Value;
 	setUsed( true );
-	// 	kdDebug() << "Setup Chain:" << name << " with Object ID: " << m_object_id << endl;
 }
 
 IPTChain::~IPTChain() {
-	kdDebug() << "\n\nIPChain::~IPTChain()" << endl;
+	kdDebug() << "\n\nIPChain::~IPTChain() : " << name() << endl;
+	m_ruleset.setAutoDelete( true );
 	m_ruleset.clear();
+	m_ruleset.setAutoDelete( false );
 	delete m_err;
 }
 
-void IPTChain::reset() {
+int IPTChain::type() {
+// 	kdDebug() << "IPTChain::type()" << endl;
+	return NetfilterObject::CHAIN;
+}
+
+void IPTChain::clear() {
+	m_ruleset.setAutoDelete( true );
 	m_ruleset.clear();
+	m_ruleset.setAutoDelete( false );
+}
+
+void IPTChain::reset() {
+	m_ruleset.setAutoDelete( true );
+	m_ruleset.clear();
+	m_ruleset.setAutoDelete( false );
+	changed();
 }
 
 void IPTChain::setDefaultTarget( const QString& target ) {
@@ -103,8 +124,12 @@ void IPTChain::setUsed( bool valid ) {
 	changed();
 }
 
+int IPTChain::indexOf( const IPTRule& rule ) {
+//	kdDebug() << "int IPTChain::indexOf( IPTRule* " << rule.name() << " ) " << endl;
+	return  m_ruleset.find( &rule );
+}
 
-IPTRule* IPTChain::addRule( const QString& rule_name, KMFError *err ) {
+IPTRule* IPTChain::addRule( const QString& rule_name, KMFError *err, int index ) {
 	kdDebug() << "IPTRule* IPTChain::addRule(QString& " << rule_name << ", KMFError *err) " << endl;
 
 	QString target = "DROP";
@@ -115,20 +140,31 @@ IPTRule* IPTChain::addRule( const QString& rule_name, KMFError *err ) {
 		QString tmp_name = m_ruleset.at( i ) ->name();
 		if ( tmp_name == name ) {
 			err->setErrType(KMFError::NORMAL);
-			err->setErrMsg( i18n( "<qt><p>Unable to add rule: <b>%1</b> into chain: <b>%2</b>.<br>There is already a rule defined with that name. Please try again with another name (must be unique in that chain).</p></qt>" ).arg( tmp_name ).arg( m_name ) );
+			err->setErrMsg( i18n( "<qt><p>Unable to add rule: <b>%1</b> into chain: <b>%2</b>.<br>There is already a rule defined with that name. Please try again with another name (must be unique in that chain).</p></qt>" ).arg( tmp_name ).arg( NetfilterObject::name() ) );
 			return 0;
 		}
 	}
 
-	IPTRule* new_rule = new IPTRule( this, new_name, target );
+	IPTRule* new_rule = new IPTRule( this, new_name.latin1(), new_name, target );
 	if ( ! new_rule ) {
 		err->setErrType(KMFError::FATAL);
 		err->setErrMsg( i18n("Sorry, a bad internal error occurred. This is a bug ;-(") );
 		return 0;
 	}
 	err->setErrType(KMFError::OK);
-	m_ruleset.append( new_rule );
-	regenerateRuleNumbers();
+	if ( index == -1 ) {
+		kdDebug() << "IPTChain::addRule: appending rule" << endl;
+		m_ruleset.append( new_rule );
+	} else {
+		if ( index < 0 || index > (int) chainRuleset().count() - -1 ) {
+			err->setErrType(KMFError::FATAL);
+			err->setErrMsg( i18n("Cannot insert rule at %1 This is a bug ;-(").arg( index ) );
+			return 0;
+		}
+		kdDebug() << "IPTChain::addRule: inserting rule at: " << index << endl;
+		m_ruleset.insert( index, new_rule  );
+	}
+// 	regenerateRuleNumbers();
 	changed();
 	return new_rule;
 }
@@ -141,16 +177,16 @@ KMFError* IPTChain::delRule( IPTRule* rule ) {
 
 	QString name = rule->name();
 	if ( m_ruleset.find( rule ) != -1 ) {
-		// 		kdDebug() << "Deleteing Rule: " << rule->name() << " ..." <<  endl;
 		m_err->setErrType(KMFError::OK );
 		if ( ! m_ruleset.remove( rule ) ) {
+			rule->deleteLater();
 			rule = 0;
 			m_err->setErrType( KMFError::FATAL );
 			const QString& msg = i18n( "Sorry, can't find Rule: %1 in the Ruleset of this chain." ).arg( rule->name() );
 			m_err->setErrMsg( msg );
 			return m_err;
 		}
-		regenerateRuleNumbers();
+		rule->deleteLater();
 		changed();
 		kdDebug() << "DONE." << endl;
 		return m_err;
@@ -161,11 +197,11 @@ KMFError* IPTChain::delRule( IPTRule* rule ) {
 		return m_err;
 	}
 	kdDebug() << "Sorry can't delete Rule: " << rule->name() << ".\nRule not found in Chain !!!" << endl;
+	return m_err;
 }
 
 bool IPTChain::moveRule( IPTRule* rule, int how_much ) {
 	// 	kdDebug() << "void IPTChain::moveRule(IPTRule* rule,int how_much)" << endl;
-
 	int num = m_ruleset.find( rule );
 	int new_pos = num + how_much;
 	if ( new_pos < 0 || new_pos > ( int ) m_ruleset.count() - 1 ) {
@@ -175,23 +211,12 @@ bool IPTChain::moveRule( IPTRule* rule, int how_much ) {
 		// 		kdDebug() << "Moving rule to position: " << new_pos << endl;
 		IPTRule *tmp_rule = m_ruleset.take( num );
 		m_ruleset.insert( new_pos, tmp_rule );
-		regenerateRuleNumbers();
+		// regenerateRuleNumbers();
 		changed();
 		return true;
 	}
 }
 
-void IPTChain::regenerateRuleNumbers() {
-	// 	kdDebug() << "void IPTChain::regenerateRuleNumbers()" << endl;
-	IPTRule* tmp_rule;
-	int i = 0;
-	for ( tmp_rule = m_ruleset.first(); tmp_rule; tmp_rule = m_ruleset.next() ) {
-		tmp_rule->setRuleNum( i );
-		i++;
-	}
-	// 	kdDebug() << "exit: void IPTChain::regenerateRuleNumbers()" << endl;
-}
-
 void IPTChain::setDropLogging( bool enabled, QString& limit, QString& burst, QString& prefix ) {
 	enable_log = enabled;
 
@@ -206,7 +231,7 @@ void IPTChain::setDropLogging( bool enabled, QString& limit, QString& burst, QSt
 	if ( !burst.isEmpty() )
 		m_log_burst = burst;
 	else
-		m_log_burst = "";
+		m_log_burst = XML::Undefined_Value;
 	changed();
 }
 
@@ -251,7 +276,7 @@ QPtrList<IPTRule>& IPTChain::chainFeeds() {
 				++it2;
 				QString target = tmp_rule->target();
 				//kdDebug() << "Found Rule: " << tmp_rule->name() << " with target: " << target << endl;
-				if ( target == m_name ) {
+				if ( target == name() ) {
 					feeds->append( tmp_rule );
 					//kdDebug() << "Found feed: " << tmp_rule->name() << " from chain: " << tmp_chain->name() << endl;
 				}
@@ -265,17 +290,19 @@ QPtrList<IPTRule>& IPTChain::chainFeeds() {
 IPTRule* IPTChain::ruleForName( const QString& name ) {
 	IPTRule * tmp_rule = 0;
 	for ( tmp_rule = m_ruleset.first();tmp_rule; tmp_rule = m_ruleset.next() ) {
-		if ( tmp_rule->name() == name )
+		if ( tmp_rule->name() == name ) {
 			return tmp_rule;
+		}
 	}
 	return 0;
 }
 
-IPTRule* IPTChain::ruleForID( int id ) {
+IPTRule* IPTChain::ruleForUuid( const QUuid& id ) {
 	IPTRule * tmp_rule = 0;
 	for ( tmp_rule = m_ruleset.first();tmp_rule; tmp_rule = m_ruleset.next() ) {
-		if ( tmp_rule->objectID() == id )
+		if ( tmp_rule->uuid() == id ) {
 			return tmp_rule;
+		}
 	}
 	return 0;
 
@@ -289,7 +316,7 @@ QString IPTChain::createIPTablesChainDefinition() {
 		m_cmd_chain_definition = "$IPT -t ";
 		m_cmd_chain_definition.append( m_table->name() );
 		m_cmd_chain_definition.append( " -N " );
-		m_cmd_chain_definition.append( m_name );
+		m_cmd_chain_definition.append( name() );
 		return m_cmd_chain_definition;
 	}
 }
@@ -300,11 +327,11 @@ QString IPTChain::createIPTablesChainDefaultTarget() {
 	m_cmd_default_target.append( m_table->name() );
 	if ( is_build_in_chain ) {
 		m_cmd_default_target.append( " -P " );
-		m_cmd_default_target.append( m_name );
+		m_cmd_default_target.append( name() );
 		m_cmd_default_target.append( " " );
 	} else {
 		m_cmd_default_target.append( " -A " );
-		m_cmd_default_target.append( m_name );
+		m_cmd_default_target.append( name() );
 		m_cmd_default_target.append( " -j " );
 	}
 	m_cmd_default_target.append( m_default_target );
@@ -315,27 +342,28 @@ QString IPTChain::createIPTablesChainDefaultTarget() {
 const QDomDocument& IPTChain::getDOMTree() {
 	// 	kdDebug() << "const QString& IPTChain::getIDOMree( )" << endl;
 	QDomDocument doc;
-	QDomElement root = doc.createElement( "chain" );
-	root.setAttribute( "id", m_object_id );
+	QDomElement root = doc.createElement( XML::Chain_Element );
+	NetfilterObject::saveUuid( root );
 
-	root.setAttribute( "name", m_name );
+
+	root.setAttribute( XML::Name_Attribute, name() );
 	if ( hasDefaultTarget() )
-		root.setAttribute( "default_target", m_default_target );
+		root.setAttribute( XML::DefaultTarget_Attribute, m_default_target );
 
-	root.setAttribute( "description", description() );
+	root.setAttribute( XML::Description_Attribute, description() );
 
 	if ( isBuildIn() ) {
-		root.setAttribute( "builtin", "yes" );
+		root.setAttribute( XML::BuiltIn_Attribute, XML::Yes_Value );
 	} else {
-		root.setAttribute( "builtin", "no" );
+		root.setAttribute( XML::BuiltIn_Attribute, XML::No_Value );
 	}
 
 	if ( logging() ) {
-		QDomElement logging = doc.createElement( "logging" );
+		QDomElement logging = doc.createElement( XML::Logging_Element );
 		root.appendChild( logging );
-		logging.setAttribute( "prefix", m_log_prefix );
-		logging.setAttribute( "limit", m_log_limit );
-		logging.setAttribute( "burst", m_log_burst );
+		logging.setAttribute( XML::Prefix_Attribute, m_log_prefix );
+		logging.setAttribute( XML::Limit_Attribute, m_log_limit );
+		logging.setAttribute( XML::Burst_Attribute, m_log_burst );
 	}
 
 	QPtrListIterator<IPTRule> it ( m_ruleset );
@@ -353,22 +381,30 @@ const QDomDocument& IPTChain::getDOMTree() {
 	return *( new QDomDocument( doc ) );
 }
 
-void IPTChain::loadXML( const QDomDocument& doc ) {
+void IPTChain::loadXML( const QDomDocument& doc, QStringList& errors ) {
+	kdDebug() << "void IPTChain::loadXML( const QDomDocument& )" << endl;
+ 	QDomElement root = doc.documentElement();
+ 	loadXML( root, errors );
+}
+void IPTChain::loadXML( QDomNode root, QStringList& errors ) {
+/*
+void IPTChain::loadXML( const QDomDocument& doc ) {*/
 	kdDebug() << "\nvoid IPTChain::loadXML( const QDomDocument& doc )" << endl;
-	QDomElement root = doc.documentElement();
-	QDomNode curr = root.toElement();
+	NetfilterObject::loadUuid( root, errors );
+	// QDomNode curr = root.toElement();
+	
 	QString target = "";
 	QString builtin = "";
 	QString desc = "";
-	kdDebug() << "Parsing node: " << curr.nodeName() << endl;
-	builtin = curr.toElement().attribute( "builtin" );
-	if ( !builtin.isEmpty() && builtin == "yes" ) {
+	kdDebug() << "Parsing node: " << root.nodeName() << endl;
+	builtin = root.toElement().attribute( XML::BuiltIn_Attribute );
+	if ( !builtin.isEmpty() && builtin == XML::Yes_Value ) {
 		setBuildIn( true );
-	} else if ( !builtin.isEmpty() && builtin == "no"  ) {
+	} else if ( !builtin.isEmpty() && builtin == XML::No_Value  ) {
 		setBuildIn( false );
 	}
 
-	target = curr.toElement().attribute( "default_target" );
+	target = root.toElement().attribute( XML::DefaultTarget_Attribute );
 	if ( !target.isEmpty() ) {
 		if ( ! isBuildIn() )
 			hasCustomDefaultTarget( true );
@@ -376,31 +412,28 @@ void IPTChain::loadXML( const QDomDocument& doc ) {
 	}
 
 
-	desc = curr.toElement().attribute( "description" );
-	// if ( !desc.isEmpty() ) {
-		setDescription( *(new QString( desc ) ) );
-		kdDebug() << "Loading chain description: " << desc << endl;
-	// }
+	desc = root.toElement().attribute( XML::Description_Attribute );
+	setDescription( *(new QString( desc ) ) );
+	kdDebug() << "Loading chain description: " << desc << endl;
+	
 	kdDebug() << "Loading Chain: " << name() << endl;
 	kdDebug() << "Is built in: " << builtin << endl;
 	kdDebug() << "Description chain description: " << desc << endl;
 	
-	curr = curr.firstChild();
+	QDomNode curr = root.firstChild();
 	QPtrList<IPTRule> used_rules;
 	while ( !curr.isNull() ) {
 		kdDebug() << "Parsing Node: " << curr.nodeName() << endl;
-		if ( curr.isElement() && curr.nodeName() == "rule" ) {
-			QString name = curr.toElement().attribute( "name" );
-			QString id = curr.toElement().attribute( "id" );
-			bool ok;
-			int rule_id = id.toInt(&ok);
-			if ( ! ok )
-				return;
+		if ( curr.isElement() && curr.nodeName() == XML::Rule_Element ) {
+			QString name = curr.toElement().attribute(  XML::Name_Attribute );
+			QString id = curr.toElement().attribute( XML::Uuid_Attribute );
+			QUuid rule_id( id );
+			
 			// 			kdDebug() << "IPTChain: Start Parsing Rule: " <<  name << endl;
 			QDomDocument rule_xml;
 			rule_xml.appendChild( curr.cloneNode(true) );
 			IPTRule* rule = 0;
-			rule = ruleForID( rule_id );
+			rule = ruleForUuid( rule_id );
 			if ( ! rule ) {
 				rule = ruleForName( name );
 				if ( ! rule ) {
@@ -413,30 +446,31 @@ void IPTChain::loadXML( const QDomDocument& doc ) {
 				}
 				// 				kdDebug() << "IPTChain: Created Rule: " << name << endl;
 			}
-			rule->loadXML( rule_xml );
+			rule->loadXML( rule_xml, errors );
 			used_rules.append( rule );
 			// 			kdDebug() << "IPTChain: Finished Parsing Rule: " <<  name  << endl;
-		} else if( curr.isElement() && curr.nodeName() == "logging"  ) {
+		} else if( curr.isElement() && curr.nodeName() == XML::Logging_Element  ) {
 			QString log_limit = "";
 			QString log_burst = "";
 			QString log_prefix = "";
 
 			enable_log  = true;
-			log_limit = curr.toElement().attribute( "limit" );
-			if( !log_limit.isEmpty() )
+			log_limit = curr.toElement().attribute( XML::Limit_Attribute );
+			if( ! log_limit.isEmpty() )
 				m_log_limit = log_limit;
 
-			log_burst = curr.toElement().attribute( "burst" );
-			if( !log_burst.isEmpty() )
+			log_burst = curr.toElement().attribute( XML::Burst_Attribute );
+			if( ! log_burst.isEmpty() )
 				m_log_burst = log_burst;
 
-			log_prefix = curr.toElement().attribute( "prefix" );
-			if( !log_prefix.isEmpty() )
-				m_log_prefix = log_burst;
+			log_prefix = curr.toElement().attribute( XML::Prefix_Attribute );
+			if( ! log_prefix.isEmpty() )
+				m_log_prefix = log_prefix;
 		}
 		curr = curr.nextSibling();
 	}
 
+
 	QPtrListIterator<IPTRule> it ( m_ruleset );
 	while ( it.current() ) {
 		// 		kdDebug() << "IPTChain::xloadXML() cleanup loop." << endl;
@@ -457,7 +491,6 @@ void IPTChain::loadXML( const QDomDocument& doc ) {
 			++it;
 		}
 	}
-	regenerateRuleNumbers();
 	changed();
 }
 
@@ -489,24 +522,24 @@ QPtrList<QStringList>& IPTChain::createIPTablesChainRules() {
 		chain_log.append( "$IPT -t " );
 		chain_log.append( m_table->name() );
 		chain_log.append( " -A " );
-		chain_log.append( m_name );
-		if ( !m_log_limit.isEmpty() ) {
+		chain_log.append( name() );
+		if ( ! m_log_limit.isEmpty() && m_log_limit != XML::Undefined_Value ) {
 			chain_log.append( " -m limit --limit " );
 			chain_log.append( m_log_limit );
-			if ( !m_log_burst.isEmpty() ) {
+			if ( ! m_log_burst.isEmpty()  && m_log_burst != XML::Undefined_Value ) {
 				chain_log.append( " --limit-burst " );
 				chain_log.append( m_log_burst );
 			}
 		}
 		chain_log.append( " -j LOG" );
-		if ( !m_log_prefix.isEmpty() ) {
+		if ( !m_log_prefix.isEmpty() && m_log_prefix != XML::Undefined_Value ) {
 			chain_log.append( " --log-prefix \"" );
 			chain_log.append( m_log_prefix );
 			chain_log.append( "\"" );
 		}
 		kdDebug() << chain_log << endl;
 		QStringList* chainDefs = new QStringList();
-		QString rule_name = i18n( "Chain: %1 Drop Logging" ).arg( m_name );
+		QString rule_name = i18n( "Chain: %1 Drop Logging" ).arg( name() );
 		chainDefs->append( rule_name );
 		chainDefs->append( chain_log );
 		all_rules->append( chainDefs );
@@ -515,7 +548,7 @@ QPtrList<QStringList>& IPTChain::createIPTablesChainRules() {
 	if ( has_default_target && !m_cmd_default_target.isEmpty() ) {
 		QString deftg = createIPTablesChainDefaultTarget();
 		QStringList* chainDefs = new QStringList();
-		QString rule_name = i18n( "Chain: %1 Default Target" ).arg( m_name );
+		QString rule_name = i18n( "Chain: %1 Default Target" ).arg( name() );
 		chainDefs->append( rule_name );
 		chainDefs->append( deftg );
 		all_rules->append( chainDefs );
@@ -524,4 +557,5 @@ QPtrList<QStringList>& IPTChain::createIPTablesChainRules() {
 	return *all_rules;
 }
 
+}
 
