@@ -1,7 +1,7 @@
 /***************************************************************************
 begin                : Mon Jan 28 2002
 copyright            : (C) 2002 by Christian Hubinger
-email                : chubinger@gmail.com
+email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -21,9 +21,11 @@ email                : chubinger@gmail.com
 #include "qptrlist.h"
 #include "qstringlist.h"
 #include "qdom.h"
+#include "quuid.h"
 
 #include <kdemacros.h>
 
+namespace KMF {
 class KMFError;
 class IPTRule;
 class IPTable;
@@ -38,9 +40,12 @@ public:
 
 	/** The Constructor that will set the "Name" and the "Table" of the chain. If buildin is set true the chain will be handeled as bild-in chain. Normaly this is only done at the initiallistaion of the whole KMFIPTDoc.
 	  */
-	IPTChain( IPTable* table, const QString& name , bool buildin = false );
+	IPTChain( IPTable* table,  const char* name, const QString& chainName, bool buildin = false );
 	~IPTChain();
-
+	
+	virtual int type();
+	virtual void clear();
+	
 	/** Sets the table to which this chain will be added */
 	void setTable( IPTable* table );
 
@@ -82,8 +87,11 @@ public:
 	bool moveRule( IPTRule* rule, int how_much );
 
 	/** Simply adds a Rule to the Chains Ruleset. */
-	IPTRule* addRule( const QString& name, KMFError* );
+	IPTRule* addRule( const QString& name, KMFError*, int index = -1 );
 
+	/** The the index of the rule in the chain **/
+	int indexOf( const IPTRule& );
+	
 	/** Deletes a given Rule in the Ruleset. */
 	KMFError* delRule( IPTRule* );
 
@@ -93,7 +101,7 @@ public:
 
 	/** Returns a pointer to a rule for the given object id if it exists in
 		this chain. If no rule is found 0 is returned */
-	IPTRule* ruleForID( int objectID );
+	IPTRule* ruleForUuid( const QUuid& );
 
 
 	/** Returns "true" if the logging is enabled, otherwhise "false". */
@@ -176,13 +184,12 @@ public:
 	const QDomDocument& getDOMTree();
 
 	/** Lad the XML sniplet representing the chain */
-	void loadXML( const QDomDocument& );
-
+	virtual void loadXML( const QDomDocument&, QStringList& errors );
+	
+	/** Load Table From DomNode */
+	virtual void loadXML( QDomNode, QStringList& errors );
 
 private:     // functions
-	/** Sets the rule numbers in the rules to the actual position
-	  in the Chain */
-	void regenerateRuleNumbers();
 
 private: //data	
 	// initialisized by the constructor
@@ -204,5 +211,5 @@ private: //data
 
 	QPtrList<IPTRule> m_ruleset;
 };
-
+}
 #endif
