@@ -1,7 +1,7 @@
 /***************************************************************************
 begin                : Mon Feb 4 2002
 copyright            : (C) 2002 by Christian Hubinger
-email                : chubinger@gmail.com
+email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -28,7 +28,11 @@ email                : chubinger@gmail.com
 /**
   *@author Christian Hubinger
   */
+
 class QString;
+
+namespace KMF {
+
 class IPTChain;
 class IPTable;
 class KMFError;
@@ -36,16 +40,20 @@ class KMFCheckInput;
 /** This class represents one "Rule" in the ruleset needed to settup
 		the firewall those rule look like: $IPT -A CHAINNAME [ OPTIONS ] -j TARGET [ OPTIONS ]
 */
+
 class KDE_EXPORT IPTRule : public NetfilterObject {
 public:
 	
 	/** The only constructor that sets the name,chain,target and table. */
-	IPTRule( IPTChain* chain , const QString& name , const QString& target );
+	IPTRule( IPTChain* chain, const char *objectname , const QString& name , const QString& target );
 	~IPTRule();
-
+	
+	virtual int type();
+	virtual void clear();
+	
 	/** Sets the name of the rule to name.
 		This name is displayed in the Document View */
-	KMFError* setName( const QString& name ); 
+	KMFError* setRuleName( const QString& name ); 
 
 	/** Sets the chain for which this Rule applies */
 	void setChain( IPTChain* chain ); 
@@ -73,10 +81,6 @@ public:
 		return m_log_rule;
 	};
 
-
-	/** Sets the rule number */
-	void setRuleNum( int num );
-
 	/** Addes a ruleoption defined the a string optionname
 		e.g. "ip_opt" "state" and the strings needed to define
 		all options is given by the list options values of the */
@@ -126,7 +130,10 @@ public:
 	const QDomDocument& getDOMTree();
 	
 	/** Load the rule from the QDomDocument */
-	void loadXML( const QDomDocument& );
+	virtual void loadXML( const QDomDocument&, QStringList& errors );
+
+	/** Load rule From DomNode */
+	virtual void loadXML( QDomNode, QStringList& errors );
 
 	/** Creates a deep copy of the rule */
 	void createRuleClone( IPTRule* );
@@ -149,9 +156,6 @@ private:
 	/** The target of the rule */
 	QString m_target;
 
-	/** The number of the rule in it's chain */
-	int m_rule_num;
-
 	KMFError *m_err;
 	KMFCheckInput *m_check_input;
 	QDict<IPTRuleOption> m_options;
@@ -168,5 +172,5 @@ private:
 	QString post; 
 	QString option_cmd;
 };
-
+}
 #endif
