@@ -1,7 +1,7 @@
 /***************************************************************************
 begin                : Mon Feb 4 2002
 copyright            : (C) 2002 by Christian Hubinger
-email                : chubinger@gmail.com
+email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -26,6 +26,9 @@ email                : chubinger@gmail.com
 #include "kmfiptdoc.h"
 #include "iptchain.h"
 #include "iptable.h"
+#include "xmlnames.h"
+
+namespace KMF {
 
 //############# Initialise static members
 QDict<QStringList>* IPTRuleOption::m_dict_option_strings = new QDict<QStringList>;
@@ -34,33 +37,45 @@ QDict<QString>* IPTRuleOption::m_dict_option_names = new QDict<QString>;
 QPtrList<QString>* IPTRuleOption::m_known_types = new QPtrList<QString>;
 bool IPTRuleOption::m_created_dict = false;
 
-IPTRuleOption::IPTRuleOption( IPTRule* rule ) : NetfilterObject( rule ) {
-	if ( rule != 0 ) {
-		// 		kdDebug() << "IPTRuleOption::IPTRuleOption(..): Creating new IPTRuleOption" << endl;
-		m_object_type = NetfilterObject::RULEOPTION;
-		m_rule = rule;
-		m_parent = rule;
-		m_option_type = "UNDEFINED";
-		m_target_option = false;
-		m_dict_option_strings->setAutoDelete( true );
-		m_known_types->setAutoDelete( true );
-		for ( int i = 0; i < MAXOPTNUM;i++ )
-			m_values[ i ] = "UNDEFINED";
-
-		if ( ! m_created_dict ) {
-			// Initialise Dict on first use
-			m_rule->chain()->table()->kmfDoc()->registerRuleOptions();
-			m_created_dict = true;
-		}
-
+IPTRuleOption::IPTRuleOption( IPTRule* rule, const char* name  ) : NetfilterObject( rule, name ) {
+	if ( rule == 0 ) {
+		kdDebug() << "ERROR: IPTRuleOption rule == 0" << endl;
+		return;
 	}
+	
+	// 		kdDebug() << "IPTRuleOption::IPTRuleOption(..): Creating new IPTRuleOption" << endl;
+	// m_object_type = NetfilterObject::RULEOPTION;
+	m_rule = rule;
+	m_option_type = XML::Undefined_Value;
+	m_target_option = false;
+	m_dict_option_strings->setAutoDelete( true );
+	m_known_types->setAutoDelete( true );
+	for ( int i = 0; i < MAXOPTNUM;i++ )
+		m_values[ i ] = XML::Undefined_Value;
+
+	if ( ! m_created_dict ) {
+		// Initialise Dict on first use
+		m_rule->chain()->table()->kmfDoc()->registerRuleOptions();
+		m_created_dict = true;
+	}
+
+	
 }
 
 IPTRuleOption::~IPTRuleOption() {}
 
+int IPTRuleOption::type() {
+// 	kdDebug() << "IPTRuleOption::type()" << endl;
+	return NetfilterObject::RULEOPTION;
+}
+
+void IPTRuleOption::clear() {
+}
+
 void IPTRuleOption::setOptionType( const QString& type ) {
-	if ( type.isNull() )
-		return ;
+	if ( type.isNull() ) {
+		return;
+	}
 	m_option_type = type;
 }
 
@@ -74,14 +89,14 @@ void IPTRuleOption::setTargetOption( bool is_tg_opt ) {
 
 void IPTRuleOption::reset() {
 	for ( int i = 0; i < MAXOPTNUM;i++ )
-		m_values[ i ] = "bool:off";
+		m_values[ i ] = XML::BoolOff_Value;
 	changed();
 }
 
 bool IPTRuleOption::isEmpty() {
 	for ( int i = 0; i < MAXOPTNUM ;i++ ) {
 		// 		kdDebug() << "Curr opt: " << m_values[ i ] << endl;
-		if ( ! m_values[ i ].isEmpty() && m_values[ i ] != "UNDEFINED" && m_values[ i ] != "bool:off" && m_values[ i ] != " " ) {
+		if ( ! m_values[ i ].isEmpty() && m_values[ i ] != XML::Undefined_Value && m_values[ i ] != XML::BoolOff_Value && m_values[ i ] != " " ) {
 			// 			kdDebug() << "Curr opt: " << getOptionType() << " is not empty found Value: "<< m_values[ i ]  << endl;
 			return false;
 		}
@@ -90,19 +105,23 @@ bool IPTRuleOption::isEmpty() {
 	return true;
 }
 
-void IPTRuleOption::loadXML( const QDomDocument& doc ) {
-	//	kdDebug() << "const QString& IPTRuleOption::restoreState( )\nXML INPUT: "<< xml << endl;
-	QDomElement root = doc.documentElement();
-	QString new_opt_type = root.toElement().attribute( "type" );
-	if ( m_option_type == "bool:off" ) {
-		// 		kdDebug() << "IPTRuleOption::restoreState: Setting Option Type: " << new_opt_type << endl;
+void IPTRuleOption::loadXML( const QDomDocument& doc, QStringList& errors ) {
+	kdDebug() << "void IPTRuleOption::loadXML( const QDomDocument& )" << endl;
+ 	QDomElement root = doc.documentElement();
+ 	loadXML( root, errors );
+}
+void IPTRuleOption::loadXML( QDomNode root, QStringList& errors ) {
+	NetfilterObject::loadUuid( root, errors );
+
+	QString new_opt_type = root.toElement().attribute( XML::Type_Attribute );
+	if ( m_option_type == XML::BoolOff_Value ) {
 		m_option_type = new_opt_type;
 	}
 	if ( m_option_type == new_opt_type )
 		kdDebug() << "Found Option Type: " << m_option_type << endl;
 
-	QString tmp = root.toElement().attribute( "targetoption" );
-	if ( tmp == "yes" )
+	QString tmp = root.toElement().attribute( XML::TargetOption_Attribute );
+	if ( tmp == XML::Yes_Value ) 
 		setTargetOption( true );
 	else
 		setTargetOption( false );
@@ -110,13 +129,13 @@ void IPTRuleOption::loadXML( const QDomDocument& doc ) {
 	QDomNode curr = root.firstChild();
 	while ( !curr.isNull() ) {
 		if ( curr.isElement() ) {
-			if ( curr.nodeName() == "ruleoptionvalue" ) {
+			if ( curr.nodeName() == XML::RuleOptionValue_Element ) {
 				for ( int i = 0; i < MAXOPTNUM;i++ ) {
 					QDomText textChild = curr.firstChild().toText();
 					QString attrib = QString( "value%1" ).arg( i );
 					QString val = curr.toElement().attribute( attrib );
 
-					if ( !val.isEmpty() && val != "UNDEFINED" ) {
+					if ( !val.isEmpty() && val != XML::Undefined_Value ) {
 						m_values[ i ] = val;
 						//            kdDebug() << "Found Option Value: " << m_values[i] << endl;
 					}
@@ -130,7 +149,7 @@ void IPTRuleOption::loadXML( const QDomDocument& doc ) {
 
 void IPTRuleOption::loadValues( QStringList args ) {
 	for ( uint i = 0; i < MAXOPTNUM; i++ )
-		m_values[ i ] = "bool:off";
+		m_values[ i ] = XML::BoolOff_Value;
 
 	int i = 0;
 	for ( QStringList::Iterator it = args.begin(); it != args.end(); ++it ) {
@@ -161,26 +180,24 @@ const QDomDocument& IPTRuleOption::getDOMTree() {
 
 	bool found = false;
 	for ( uint i = 0; i < MAXOPTNUM; i++ )
-		if ( !m_values[ i ].isEmpty() && m_values[ i ] != "UNDEFINED" )
+		if ( !m_values[ i ].isEmpty() && m_values[ i ] != XML::Undefined_Value )
 			found = true;
 	if ( true ) {
-		QDomElement root = doc.createElement( "ruleoption" );
-		root.setAttribute( "type", m_option_type );
-		if ( m_target_option )
-			root.setAttribute( "targetoption", "yes" );
-		else
-			root.setAttribute( "targetoption", "no" );
+		QDomElement root = doc.createElement( XML::RuleOption_Element );
+		NetfilterObject::saveUuid( root );
+		root.setAttribute( XML::Type_Attribute, m_option_type );
+		if ( m_target_option ) {
+			root.setAttribute( XML::TargetOption_Attribute, XML::Yes_Value );
+		} else {
+			root.setAttribute( XML::TargetOption_Attribute, XML::No_Value );
+		}
 
-		root.setAttribute( "id", m_object_id );
-		// 		kdDebug() << "Root Element Should be: ruleoption" << endl;
-		// 		kdDebug() << "Root Element: " << root.tagName() << endl;
 		doc.appendChild( root );
-		//  kdDebug() << "Storing: " << m_values->length() << " values" << endl;
 		for ( uint i = 0; i < MAXOPTNUM; i++ ) {
 			QString val = m_values[ i ];
-			if ( !val.isEmpty() && val != "UNDEFINED" ) {
-				QDomElement tag = doc.createElement( "ruleoptionvalue" );
-				tag.setTagName( "ruleoptionvalue" );
+			if ( !val.isEmpty() && val != XML::Undefined_Value ) {
+				QDomElement tag = doc.createElement( XML::RuleOptionValue_Element );
+				tag.setTagName( XML::RuleOptionValue_Element );
 				//			kdDebug() << "Writing Attribute: value" << i << " val: " << val << endl;
 				QString attrib = QString( "value%1" ).arg( i );
 				tag.setAttribute( attrib, val );
@@ -214,9 +231,9 @@ const QString& IPTRuleOption::toString() {
 
 			if (! val.isNull() &&
 			        ! val.isEmpty() &&
-			        val != "UNDEFINED" &&
-			        val != "bool:off" ) {
-				if ( val == "bool:on" ) {
+			        val != XML::Undefined_Value &&
+			        val != XML::BoolOff_Value ) {
+				if ( val == XML::BoolOn_Value ) {
 					val = "";
 				}
 //				kdDebug() << "Create option Name: " << guiName << endl;
@@ -257,9 +274,9 @@ void IPTRuleOption::readRuleOptionDefinition( const QDomDocument& doc ) {
 	while ( !curr.isNull() ) {
 		//		kdDebug() << "IPTRuleOption: Parsing Node: " << curr.nodeName() << endl;
 		if ( curr.isElement() && curr.nodeName() == "ruleoptiondefinition" ) {
-			name = curr.toElement().attribute( "name" );
-			guiName = curr.toElement().attribute( "guiName" );
-			//	version = root.toElement().attribute( "version" );
+			name = curr.toElement().attribute(  XML::Name_Attribute );
+			guiName = curr.toElement().attribute( XML::GUIName_Attribute );
+			//	version = root.toElement().attribute( XML::Version_Attribute );
 			vals = new QStringList();
 			gui = new QStringList();
 			kdDebug() << "+ Register Rule Option: " << name << " " <<  guiName << endl;
@@ -295,7 +312,7 @@ void IPTRuleOption::readRuleOptionDefinitionNode( const QDomNode& currNode, QStr
 		kdDebug() << "IPTRuleOption: Parsing Node: " << curr.nodeName() << endl;
 		if ( curr.isElement() && curr.nodeName() == "option" ) {
 			QString cmd = curr.toElement().attribute( "command" );
-			QString guiName = curr.toElement().attribute( "guiName" );
+			QString guiName = curr.toElement().attribute( XML::GUIName_Attribute );
 
 			cmd.simplifyWhiteSpace();
 			vals->append( cmd );
@@ -310,3 +327,4 @@ void IPTRuleOption::readRuleOptionDefinitionNode( const QDomNode& currNode, QStr
 	}
 }
 
+}
