@@ -1,7 +1,7 @@
 /***************************************************************************
 begin                : Mon Feb 4 2002
 copyright            : (C) 2002 by Christian Hubinger
-email                : chubinger@gmail.com
+email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -28,7 +28,7 @@ email                : chubinger@gmail.com
 #include <kdemacros.h>
 
 #define MAXOPTNUM 10
-
+namespace KMF {
 class KMFIPTDoc;
 class IPTRule;
 
@@ -36,6 +36,7 @@ class IPTRule;
   * class IPTRuleOption
   *
   */
+
 class KDE_EXPORT IPTRuleOption : public NetfilterObject {
 	//########## BEGIN STATIC ##############
 
@@ -65,11 +66,17 @@ private:  // DATA
 
 	/** Public methods: */
 public:
-	IPTRuleOption( IPTRule* = 0 );
+	IPTRuleOption( IPTRule* rule, const char* name );
 	~IPTRuleOption();
 
+	virtual int type();
+	virtual void clear();
+	
 	/** restore the opotion config from an xml sniplet */
-	void loadXML( const QDomDocument& );
+	virtual void loadXML( const QDomDocument&, QStringList& errors );
+
+	/** restore the opotion config from an xml sniplet */
+	virtual void loadXML( QDomNode, QStringList& errors );
 
 	/** Makes the option empty. */
 	void reset();
@@ -121,6 +128,6 @@ private:
 	bool m_target_option;
 	QString m_values[ MAXOPTNUM ];
 };
-
+}
 
 #endif // IPTRULEOPTION_H
