@@ -1,7 +1,7 @@
 /***************************************************************************
     begin                : Fri Feb 21 2003
     copyright            : (C) 2003 by Christian Hubinger
-    email                : chubinger@gmail.com
+    email                : chubinger@irrsinnig.org
  ***************************************************************************/
 
 /***************************************************************************
@@ -28,6 +28,9 @@
   */
 
 class QLabel;
+namespace KMF {
+
+
 class KDE_EXPORT KMFApp : public KApplication  {
 public: 
 	KMFApp();
@@ -37,4 +40,6 @@ public:
 KDE_EXPORT void showSplash();
 KDE_EXPORT void closeSplash();
 KDE_EXPORT void set_splash_status( const QString& );
+
+}
 #endif
