@@ -14,21 +14,24 @@
 
 #include <kdemacros.h>
 
+
+namespace KMF {
 /**
 @author Christian Hubinger
 */
+
 class KDE_EXPORT KMFAppState{
 public:
     KMFAppState();
     ~KMFAppState();
 	static void setUpAndRunning( bool on );
-	static void setHasOpenDoc( bool on );
+// 	static void setHasOpenDoc( bool on );
 	static bool upAndRunning();
-	static bool hasOpenDoc();
+// 	static bool hasOpenDoc();
 
 private:
 	static bool _upAndRunning;
 	static bool _hasOpenDoc;
 };
-
+}
 #endif
