@@ -1,7 +1,7 @@
 /***************************************************************************
  begin                : Tue Aug 6 2002
  copyright            : (C) 2002 by Christian Hubinger
- email                : chubinger@gmail.com
+ email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -24,9 +24,11 @@
 #include <qdict.h>
 
 #include <kdemacros.h>
-
 class QString;
+
+namespace KMF {
 class KMFError;
+
 class KDE_EXPORT KMFCheckInput {
 public:
 	KMFCheckInput();
@@ -34,7 +36,7 @@ public:
 
 	/** Check the given input string is it matches the criterias
 		for the given input type */
-	void checkInput( QString& inp, const QString& inp_type, KMFError* );
+	void checkInput( const QString& inp, const QString& inp_type, KMFError* );
 
 private:    // functions
 	void generateMsgDict();
@@ -71,5 +73,5 @@ private:    // functions
 private:     // data
 	QDict<QString> m_msg_dict;
 };
-
+}
 #endif
