@@ -1,7 +1,7 @@
 /***************************************************************************
  begin                : Tue Aug 6 2002
  copyright            : (C) 2002 by Christian Hubinger
- email                : chubinger@gmail.com
+ email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -18,17 +18,28 @@
 
 #include <qstring.h> 
 #include <kdemacros.h>
+#include <qstringlist.h>
 
 /**
   *@author Christian Hubinger
   */
-
+using namespace std;
+namespace KMF {
 class KDE_EXPORT KMFError {
+
+// using namespace  ;
+
+
+public:
+/** Contains the option commands for each option type. */
+	static KMFError* parseErrors( QStringList& errors );
+	static const QString& getAsString( int error_type, const QString& msg );
+
 public:
 	KMFError();
 	~KMFError();
 
-	enum {  OK = 0, HINT = 1, NORMAL = 2, FATAL = 3 };
+	enum {  OK = 0, HINT = 1, NORMAL = 2, FATAL = 3, WARNING = 4 };
 	
 	/** Set the error message */
 	void setErrMsg( const QString& msg );
@@ -52,9 +63,9 @@ public:
 	};
 
 private:
-	private: int m_err_type;
-	private: QString m_err_msg;
-	private: int m_err_num;
+	int m_err_type;
+	QString m_err_msg;
+	int m_err_num;
 };
-
+}
 #endif
