@@ -1,7 +1,7 @@
 /***************************************************************************
  begin                : Tue Aug 6 2002
  copyright            : (C) 2002 by Christian Hubinger
- email                : chubinger@gmail.com
+ email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -25,6 +25,8 @@
 // my includes
 #include "kmferror.h"
 
+namespace KMF {
+
 KMFErrorHandler::KMFErrorHandler( const QString& name ) {
 	m_caller_name = new QString( name );
 }
@@ -65,3 +67,7 @@ bool KMFErrorHandler::showError( KMFError* err ) {
 	}
 	return false;
 }
+
+}
+
+
