@@ -5,7 +5,7 @@
 // Description:
 //
 //
-// Author: Christian Hubinger <chubinger@gmail.com>, (C) 2003
+// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2003
 //
 // Copyright: See COPYING file that comes with this distribution
 //
@@ -22,6 +22,7 @@
 
 #include "kmfgenericdoc.h"
 
+
 // QT includes
 #include <qfile.h>
 #include <qdom.h>
@@ -29,6 +30,7 @@
 #include <qstringlist.h>
 
 // KDE includes
+#include <kapplication.h>
 #include <kdebug.h>
 #include <kstandarddirs.h>
 #include <klocale.h>
@@ -38,19 +40,24 @@
 #include <klibloader.h>
 
 // Project includes
-#include "../interfaces/kmfcompilerinterface.h"
-#include "../interfaces/kmfplugin.h"
+#include "../version.h"
+#include "kmfcompilerinterface.h"
+#include "kmfplugin.h"
 #include "kmfpluginfactory.h"
 #include "kmferror.h"
 #include "kmfprotocol.h"
 #include "kmfnetzone.h"
+#include "kmfnetwork.h"
 #include "kmfconfig.h"
 #include "ipaddress.h"
+#include "kmftarget.h"
+#include "xmlnames.h"
 
+namespace KMF {
 
-KMFGenericDoc::KMFGenericDoc( QObject* parent, const char* name ) : KMFDoc( parent, name ) {
+KMFGenericDoc::KMFGenericDoc( NetfilterObject* parent, const char* name, KMFTarget* target ) : KMFDoc( parent, name ), KMFRulesetDoc( target ) {
 	kdDebug() << "KMFGenericDoc::KMFGenericDoc( QObject *parent, const char *name ) : KMFDoc( parent, name )" << endl;
-	m_zones.setAutoDelete( true );
+	m_zones.setAutoDelete( false );
 	m_zone_incoming = 0;
 	m_zone_outgoing = 0;
 	m_zone_trusted= 0 ;
@@ -67,10 +74,15 @@ KMFGenericDoc::KMFGenericDoc( QObject* parent, const char* name ) : KMFDoc( pare
 	m_limitLog  = true;
 	m_logPrefix = "KMF: ";
 	m_natAddress = new IPAddress(0,0,0,0);
-	m_outgoingInterface = "bool:off";
+	m_outgoingInterface = XML::BoolOff_Value;
 	initDoc();
 }
 
+int KMFGenericDoc::type() {
+// 	kdDebug() << "IPTRule::type()" << endl;
+	return NetfilterObject::GENERIC_RULESET;
+}
+
 
 KMFGenericDoc::~KMFGenericDoc() {}
 
@@ -92,9 +104,9 @@ void KMFGenericDoc::clear() {
 	m_limitLog  = true;
 	m_logPrefix = "KMF: ";
 	m_natAddress->setAddress( 0, 0, 0, 0 );
-	m_outgoingInterface = "bool:off";
-	m_name = i18n("Unamed Ruleset");
-	m_description = i18n("No description available");
+	m_outgoingInterface = XML::BoolOff_Value;
+	resetUrl();
+	setDescription( i18n("No description available") );
 }
 
 void KMFGenericDoc::initDoc() {
@@ -102,21 +114,15 @@ void KMFGenericDoc::initDoc() {
 	m_err = new KMFError;
 	m_url.setFileName( i18n( "Untitled" ) );
 
-	if ( ! loadProtocolLibrary() )
-		kdDebug() << "ERROR: Couldn't load protocol Library" << endl;
-
-	if ( ! loadCustomProtocolLibrary() )
-		kdDebug() << "ERROR: Couldn't load custom protocol Library" << endl;
-
-	m_zone_incoming = new KMFNetZone( this,  "incoming_world" );
-	m_zone_outgoing = new KMFNetZone( this,  "outgoing_world" );
-	m_zone_trusted= new KMFNetZone( this,  "trusted_hosts" );
-	m_zone_malicious= new KMFNetZone( this,  "malicious_hosts" );
-	m_zone_badClients= new KMFNetZone( this,  "badClients_hosts" );
-	m_zone_badServers= new KMFNetZone( this,  "badServers_hosts" );
+	m_zone_incoming = new KMFNetZone( this,  "incoming_world",  "incoming_world" );
+	m_zone_outgoing = new KMFNetZone( this,  "outgoing_world",  "outgoing_world" );
+	m_zone_trusted= new KMFNetZone( this,  "trusted_hosts",  "trusted_hosts" );
+	m_zone_malicious= new KMFNetZone( this,  "malicious_hosts",  "malicious_hosts" );
+	m_zone_badClients= new KMFNetZone( this,  "badClients_hosts",  "badClients_hosts" );
+	m_zone_badServers= new KMFNetZone( this,  "badServers_hosts",  "badServers_hosts" );
 
 
-	m_zone_incoming->setGuiName(  i18n( "Incoming Connections" ) );
+	m_zone_incoming->setGuiName( i18n( "Incoming Connections" ) );
 	m_zone_outgoing->setGuiName( i18n( "Outgoing Connections" )  );
 	m_zone_trusted->setGuiName( i18n( "Trusted Hosts" )  );
 	m_zone_malicious->setGuiName( i18n( "Malicious Hosts" )  );
@@ -143,9 +149,13 @@ void KMFGenericDoc::initDoc() {
 																	"to use services your computer provides." ) );
 	m_zone_badServers->setDescription( i18n( "You will not be able to use the services\n"
 																	"of the hosts in that list." ) );
-
-
 }
+
+const QString& KMFGenericDoc::getFileDialogFilter() {
+	return *( new QString("*.kmfgrs|KMyFirewall Generic Ruleset(*.kmfgrs)") );
+}
+
+
 void KMFGenericDoc::setRestrictOutgoingConnections( bool onoff ) {
 	m_restrictOutgoingConnections = onoff;
 }
@@ -191,56 +201,10 @@ void KMFGenericDoc::setUseMasquerade( bool onoff ) {
 	m_useMasquerade = onoff;
 }
 
-// void KMFGenericDoc::setRestrictNat( bool onoff ) {
-// 	m_restrictNat = onoff;
-// }
-
 void KMFGenericDoc::setNatAddress( const QString& addr ) {
 	m_natAddress->setAddress( addr );
 }
 
-// void KMFGenericDoc::setAllowedIncomingInterfaces( const QString& str ) {
-// 	m_allowedIncomingInterfaces.clear();
-// 	m_allowedIncomingInterfaces = QStringList::split("|",str,false );
-// }
-
-// bool KMFGenericDoc::isAllowedIncomingInterface( const QString& interf ) {
-// 	for ( QStringList::Iterator it = m_allowedIncomingInterfaces.begin(); it != m_allowedIncomingInterfaces.end(); ++it ) {
-// 		if ( *it  == interf ) {
-// // 			kdDebug() << "KMFGenericDoc::isAllowedIncomingInterface( const QString& interf ): Found Interface: " << interf << endl;
-// 			return true;
-// 		}
-//     }
-// 	return false;
-// }
-
-// void KMFGenericDoc::addAllowedIncomingInterface( const QString& interf ) {
-// 	bool found = false;
-// 	for ( QStringList::Iterator it = m_allowedIncomingInterfaces.begin(); it != m_allowedIncomingInterfaces.end() && ! found; ++it ) {
-// 
-// 		if ( *it  == interf )
-// 			found = true;
-//     }
-// 
-// 	if ( !found ) {
-// // 			kdDebug() << "KMFGenericDoc::addAllowedIncomingInterface( const QString& interf ): Adding Interface: " << interf << endl;
-// 		m_allowedIncomingInterfaces << interf;
-// 	}
-// }
-
-
-// // void KMFGenericDoc::delAllowedIncomingInterface( const QString& interf ) {
-// // 	bool found = false;
-// // 	for ( QStringList::Iterator it = m_allowedIncomingInterfaces.begin(); it != m_allowedIncomingInterfaces.end() && ! found; ++it ) {
-// // 
-// // 		if ( *it  == interf ) {
-// // // 			kdDebug() << "KMFGenericDoc::delAllowedIncomingInterface( const QString& interf ): Removeing Interface: " << interf << endl;
-// // 			m_allowedIncomingInterfaces.remove( *it );
-// // 			found = true;
-// // 		}
-// //     }
-// // }
-
 void KMFGenericDoc::setOutgoingInterface( const QString& str ) {
 	m_outgoingInterface = str;
 }
@@ -270,10 +234,9 @@ KMFNetZone* KMFGenericDoc::addZone( const QString& name, KMFError* err ) {
 		return 0;
 	}
 
-	KMFNetZone * new_zone = new KMFNetZone( this, name );
+	KMFNetZone * new_zone = new KMFNetZone( this, name.latin1(), name );
 	m_zones.append( new_zone );
 	err->setErrType( KMFError::OK );
-	changed();
 	return new_zone;
 }
 
@@ -284,7 +247,7 @@ KMFError* KMFGenericDoc::delZone( KMFNetZone* zone ) {
 	while ( it.current() && ! found ) {
 		KMFNetZone * z = it.current();
 		++it;
-		if ( zone->name() == z->name() ) {
+		if ( zone == z ) {
 			found = true;
 			m_zones.remove( zone );
 			m_err->setErrType( KMFError::OK );
@@ -312,103 +275,23 @@ KMFNetZone* KMFGenericDoc::findZone( const QString& name ) const {
 	return 0;
 }
 
-bool KMFGenericDoc::loadProtocolLibrary() {
-// 	kdDebug() << "bool KMFGenericDoc::loadProtocolLibrary()" << endl;
-	KStandardDirs std_dir;
-	QString file = std_dir.findResource( "data", "kmyfirewall/protocols/kmfprotocollibrary.xml" );
-// 	kdDebug () << "Found Library at:" << file << endl;
-	QFile kmfrsFile( file );
-	QDomDocument domTree;
-	if ( !kmfrsFile.open( IO_ReadOnly ) ) {
-		kdDebug() << "Couldn't open file" << endl;
-		return false;
-	}
-	if ( !domTree.setContent( &kmfrsFile ) ) {
-		kdDebug() << "Couldn't set DomDocument content" << endl;
-		kmfrsFile.close();
-		return false;
-	}
-	kmfrsFile.close();
-
-	QDomElement root = domTree.documentElement();
-	QDomNode curr = root.firstChild();
-	while ( !curr.isNull() ) {
-// 		kdDebug() << "Parsing Node: " << curr.nodeName() << endl;
-		if ( curr.isElement() && curr.nodeName() == "protocol" ) {
-			KMFProtocol * prot = new KMFProtocol( this );
-			QDomDocument protocol;
-			protocol.appendChild( curr.cloneNode( true ) );
-			prot->loadXML( protocol );
-			m_protocol_library.append( prot );
-		} else if ( curr.isElement() && curr.nodeName() == "abstract" ) {
-			kdDebug() << "Parsing Abstract not implemented" << endl;
-		}
-		curr = curr.nextSibling();
-	}
-// 	kdDebug() << "Finished Parsing Protocol Library" << endl;
-	return true;
-}
-
-bool KMFGenericDoc::loadCustomProtocolLibrary() {
-	// FIXME: NOT IMPLEMENTED YET
-// 	kdDebug() << "bool KMFGenericDoc::loadCustomProtocolLibrary()" << endl;
-	KStandardDirs std_dir;
-	QString file = std_dir.findResource( "data", "kmyfirewall/protocols/kmfcustomprotocollibrary.xml" );
-	kdDebug () << "Found Library at:" << file << endl;
-	if ( file.isEmpty() ) {
-		kdDebug() << "INFORMATION: Creating file $KDEHOME/share/apps/kmyfirewall/protocols/kmfcustomprotocollibrary.xml" << endl;
-		QString path = std_dir.findResourceDir( "data", "$KDEHOME/share/apps" );
-		kdDebug() << "KDEHome dir: " << path << endl;
-	}
-
-// 	QFile kmfrsFile( file );
-// 	QDomDocument domTree;
-// 	if ( !kmfrsFile.open( IO_ReadOnly ) ) {
-// 		kdDebug() << "Couldn't open file" << endl;
-// 		return false;
-// 	}
-// 	if ( !domTree.setContent( &kmfrsFile ) ) {
-// 		kdDebug() << "Couldn't set DomDocument content" << endl;
-// 		kmfrsFile.close();
-// 		return false;
-// 	}
-// 	kmfrsFile.close();
-//
-// 	QDomElement root = domTree.documentElement();
-// 	QDomNode curr = root.firstChild();
-// 	while ( !curr.isNull() ) {
-// 		kdDebug() << "Parsing Node: " << curr.nodeName() << endl;
-// 		if ( curr.isElement() && curr.nodeName() == "protocol" ) {
-// 			KMFProtocol * prot = new KMFProtocol( this );
-// 			QDomDocument protocol;
-// 			protocol.appendChild( curr.cloneNode( true ) );
-// 			prot->loadXML( protocol );
-// 			m_protocol_library.append( prot );
-// 		} else if ( curr.isElement() && curr.nodeName() == "abstract" ) {
-// 			kdDebug() << "Parsing Abstract not implemented" << endl;
-// 		}
-// 		curr = curr.nextSibling();
-// 	}
-// 	kdDebug() << "Finished Parsing Protocol Library" << endl;
-	return true;
-}
-
 const QString& KMFGenericDoc::compile() {
 // 	kdDebug() << "const QString& KMFGenericDoc::compile()" << endl;
-	KMFCompilerInterface* compiler = KMFPluginFactory::KMFCompiler( KMFConfig::oS(), KMFConfig::backend()   /* "linux", "iptables" */, parent() );
+	KMFCompilerInterface* compiler = target()->compiler();
 	if ( ! compiler ) {
 		return *( new QString("ERROR") );
 	}
 	return compiler->compile( this );
 }
 
-KMFDoc* KMFGenericDoc::parseXMLRuleset( const KURL& url ) {
-// 	kdDebug() << "KMFDoc* KMFGenericDoc::parseXMLRuleset( const KURL& )" << endl;
+void KMFGenericDoc::parseDocument( const KURL& url, QStringList& errors ) {
+// 	kdDebug() << "KMFDoc* KMFGenericDoc::parseDocument( const KURL& )" << endl;
 	QString xmlfile;
-	if ( ! KIO::NetAccess::download( url, xmlfile, 0 ) ) {
+	if ( ! KIO::NetAccess::download( url, xmlfile, KApplication::kApplication()->mainWidget() ) ) {
 		clear();
 		m_url.setFileName( i18n( "Untitled" ) );
-		return this;
+		return;
+		// return this;
 	}
 
 	if ( !xmlfile.isEmpty() ) {
@@ -418,31 +301,39 @@ KMFDoc* KMFGenericDoc::parseXMLRuleset( const KURL& url ) {
 		QFile kmfrsFile( xmlfile );
 		QDomDocument domTree;
 		if ( !kmfrsFile.open( IO_ReadOnly ) ) {
-			return 0;
+			return;
+			// return 0;
 		}
 		if ( !domTree.setContent( &kmfrsFile ) ) {
 			kmfrsFile.close();
-			return 0;
+			return;
+			// return 0;
 		}
 		kmfrsFile.close();
 
 		kdDebug() << "############ Start Parsing ############" << endl;
-		loadXML( domTree );
+		loadXML( domTree, errors );
 		kdDebug() << "########## Finished Parsing ###########" << endl;
-
-		setUrl( url );
+			
+		m_url = url;
+		// setUrl( url );
 		emit documentChanged();
 		KIO::NetAccess::removeTempFile( xmlfile );
-		return this;
+		return;
+	//	return this;
 	}
 	KIO::NetAccess::removeTempFile( xmlfile );
-	return this;
+//	return this;
 }
 
 const QDomDocument& KMFGenericDoc::getDOMTree() {
-	kdDebug() << "const QDomDocument& KMFGenericDoc::getDOMTree()" << endl;
+	// kdDebug() << "const QDomDocument& KMFGenericDoc::getDOMTree()" << endl;
 	QDomDocument doc( "kmyfirewall-ruleset" );
-	QDomElement root = doc.createElement( "kmfgrs" );
+	QDomElement root = doc.createElement( XML::GenericDoc_DocumentElement );
+	NetfilterObject::saveUuid( root );
+	root.setAttribute( XML::Version_Attribute, KMYFIREWALL_VERSION );
+	root.setAttribute( XML::MinVersion_Attribute, "1.0.0" );
+	root.setAttribute( XML::MaxVersion_Attribute, "~" );
 
 	root.appendChild( m_zone_incoming->getDOMTree() );
 	root.appendChild( m_zone_outgoing->getDOMTree() );
@@ -450,232 +341,215 @@ const QDomDocument& KMFGenericDoc::getDOMTree() {
 	root.appendChild( m_zone_badServers->getDOMTree() );
 	root.appendChild( m_zone_malicious->getDOMTree() );
 	root.appendChild( m_zone_trusted->getDOMTree() );
-	QDomElement abstract = doc.createElement( "abstract" );
+	QDomElement abstract = doc.createElement( XML::Abstract_Element );
 	if ( restrictOutgoingConnections() )
-		abstract.setAttribute( "restrictOutgoingConnections" ,"bool:on" );
+		abstract.setAttribute( XML::RestrictOutgoingConnections_Attribute ,XML::BoolOn_Value );
 	else
-		abstract.setAttribute( "restrictOutgoingConnections" ,"bool:off" );
+		abstract.setAttribute( XML::RestrictOutgoingConnections_Attribute ,XML::BoolOff_Value );
 	if ( allowIncomingConnections() )
-		abstract.setAttribute( "allowIncomingConnections" ,"bool:on" );
+		abstract.setAttribute( XML::AllowIncomingConnections_Attribute ,XML::BoolOn_Value );
 	else
-		abstract.setAttribute( "allowIncomingConnections" ,"bool:offf" );
+		abstract.setAttribute( XML::AllowIncomingConnections_Attribute ,XML::BoolOff_Value );
 	
-	abstract.setAttribute( "description", description() );
-	abstract.setAttribute( "name", name() );
+	abstract.setAttribute( XML::Description_Attribute, description() );
+	abstract.setAttribute(  XML::Name_Attribute, name() );
 	
 	root.appendChild( abstract );
 
-	QDomElement logging = doc.createElement( "logging" );
+	QDomElement logging = doc.createElement( XML::Logging_Element );
 	if ( logDropped() )
-		logging.setAttribute( "logDropped", "bool:on" );
+		logging.setAttribute( XML::LogDropped_Attribute, XML::BoolOn_Value );
 	else
-		logging.setAttribute( "logDropped", "bool:off" );
+		logging.setAttribute( XML::LogDropped_Attribute, XML::BoolOff_Value );
 
 	if ( limitLog() )
-		logging.setAttribute( "limitLog", "bool:on" );
+		logging.setAttribute( XML::LimitLog_Attribute, XML::BoolOn_Value );
 	else
-		logging.setAttribute( "limitLog", "bool:off" );
+		logging.setAttribute( XML::LimitLog_Attribute, XML::BoolOff_Value );
 
-	logging.setAttribute( "logPrefix", logPrefix() );
+	logging.setAttribute( XML::LogPrefix_Attribute, logPrefix() );
 	root.appendChild( logging );
 
-	QDomElement icmp = doc.createElement( "icmp" );
-	if ( allowPingReply() )
-		icmp.setAttribute( "allowPingReply", "bool:on" );
-	else
-		icmp.setAttribute( "allowPingReply", "bool:off" );
-
+	QDomElement icmp = doc.createElement( XML::ICMP_Element );
+	if ( allowPingReply() ) {
+		icmp.setAttribute( XML::AllowPingReply_Attribute, XML::BoolOn_Value );
+	} else {
+		icmp.setAttribute( XML::AllowPingReply_Attribute, XML::BoolOff_Value );
+	}
+	
 	if ( limitPingReply() )
-		icmp.setAttribute( "limitPingReply", "bool:on" );
+		icmp.setAttribute( XML::LimitPingReply_Attribute, XML::BoolOn_Value );
 	else
-		icmp.setAttribute( "limitPingReply", "bool:off" );
+		icmp.setAttribute( XML::LimitPingReply_Attribute, XML::BoolOff_Value );
 	root.appendChild( icmp );
 
-	QDomElement nat = doc.createElement( "nat" );
+	QDomElement nat = doc.createElement( XML::NAT_Element );
 	if ( useNat() )
-		nat.setAttribute( "useNat", "bool:on" );
+		nat.setAttribute( XML::UseNat_Attribute, XML::BoolOn_Value );
 	else
-		nat.setAttribute( "useNat", "bool:off" );
+		nat.setAttribute( XML::UseNat_Attribute, XML::BoolOff_Value );
 
 	if ( useMasquerade() )
-		nat.setAttribute( "useMasquerade", "bool:on" );
+		nat.setAttribute( XML::UseMasquerade_Attribute, XML::BoolOn_Value );
 	else
-		nat.setAttribute( "useMasquerade", "bool:off" );
+		nat.setAttribute( XML::UseMasquerade_Attribute, XML::BoolOff_Value );
 
-// 	if ( restrictNat() )
-// 		nat.setAttribute( "restrictNat", "bool:on" );
-// 	else
-// 		nat.setAttribute( "restrictNat", "bool:off" );
 
-	nat.setAttribute( "natAddress", m_natAddress->toString() );
+	nat.setAttribute( XML::NatAddress_Attribute, m_natAddress->toString() );
 	QString allowedIn = "";
 
-/*	for ( QStringList::Iterator it = m_allowedIncomingInterfaces.begin(); it != m_allowedIncomingInterfaces.end(); ++it ) {
-        allowedIn += *it + "|";
-    }*/
-// 	allowedIn = allowedIn.left( allowedIn.length() - 1 );
-// 	nat.setAttribute( "allowedIncomingInterfaces", allowedIn );
-	nat.setAttribute( "outgoingInterface", m_outgoingInterface );
+	nat.setAttribute( XML::OutgoingInterface_Attribute, m_outgoingInterface );
 
 	root.appendChild( nat );
 
 	doc.appendChild( root );
 	return *( new QDomDocument( doc ) );
 }
-
-void KMFGenericDoc::loadXML( const QDomDocument& doc ) {
-// 	kdDebug() << "void KMFGenericDoc::loadXML( const QDomDocument& )" << endl;
+void KMFGenericDoc::loadXML( const QDomDocument& doc, QStringList& errors ) {
+	kdDebug() << "void KMFGenericDoc::loadXML( const QDomDocument& )" << endl;
 	QDomElement root = doc.documentElement();
+	if ( root.nodeName() != XML::GenericDoc_DocumentElement ) {
+		kdDebug() << "!!! ERROR: Wrong XML format " << root.nodeName() << " found !!!" << endl;
+		errors.append( KMFError::getAsString( KMFError::FATAL, i18n("Wrong XML format <b>%1</b> found! Expected kmfgrs").arg( root.nodeName() ) ) );
+		return;
+ 	}
+ 	loadXML( root, errors );
+}
+
+void KMFGenericDoc::loadXML( QDomNode root, QStringList& errors ) {
+// void KMFGenericDoc::loadXML( const QDomDocument& doc ) {
+// 	kdDebug() << "void KMFGenericDoc::loadXML( const QDomDocument& )" << endl;
+//	QDomElement root = doc.documentElement();
+	NetfilterObject::loadUuid ( root, errors );
 	QDomNode curr = root.firstChild();
 	while ( !curr.isNull() ) {
 		kdDebug() << "Parsing Node: " << curr.nodeName() << endl;
-		if ( curr.isElement() && curr.nodeName() == "netzone" ) {
-			kdDebug() << "" << endl;
+		if ( curr.isElement() && curr.nodeName() == XML::NetZone_Element ) {
 			QString name = "";
-			name = curr.toElement().attribute( "name" );
+			name = curr.toElement().attribute(  XML::Name_Attribute );
 			QDomDocument zone_doc;
 			zone_doc.appendChild( curr.cloneNode(true) );
 			if ( name == "incoming_world" ) {
 // 				kdDebug() << "\nKMFIPTDoc: Start Parsing INCOMMING ZONES" << endl;
-				m_zone_incoming->loadXML( zone_doc );
+				m_zone_incoming->loadXML( zone_doc, errors );
 // 				kdDebug() << "KMFIPTDoc: Finished Parsing INCOMMING ZONES" << endl;
 			}
 			if ( name == "outgoing_world" ) {
 // 				kdDebug() << "\nKMFIPTDoc: Start Parsing OUTGOING ZONES" << endl;
-				m_zone_outgoing->loadXML( zone_doc );
+				m_zone_outgoing->loadXML( zone_doc, errors );
 // 				kdDebug() << "KMFIPTDoc: Finished Parsing OUTGOING ZONES" << endl;
 			}
 			if ( name == "trusted_hosts" ) {
 // 				kdDebug() << "\nKMFIPTDoc: Start Parsing TRUSTED HOSTS" << endl;
-				m_zone_trusted->loadXML( zone_doc );
+				m_zone_trusted->loadXML( zone_doc, errors );
 // 				kdDebug() << "KMFIPTDoc: Finished Parsing TRUSTED HOSTS" << endl;
 			}
 			if ( name == "malicious_hosts" ) {
 // 				kdDebug() << "\nKMFIPTDoc: Start Parsing MALICIOUS HOSTS" << endl;
-				m_zone_malicious->loadXML( zone_doc );
+				m_zone_malicious->loadXML( zone_doc, errors );
 // 				kdDebug() << "KMFIPTDoc: Finished Parsing MALICIOUS HOSTS" << endl;
 			}
 			if ( name == "badClients_hosts" ) {
 // 				kdDebug() << "\nKMFIPTDoc: Start Parsing BADCLIENTS HOSTS" << endl;
-				m_zone_badClients->loadXML( zone_doc );
+				m_zone_badClients->loadXML( zone_doc, errors );
 // 				kdDebug() << "KMFIPTDoc: Finished Parsing BADCLIENTS HOSTS" << endl;
 			}
 			if ( name == "badServers_hosts" ) {
 // 				kdDebug() << "\nKMFIPTDoc: Start Parsing BADSERVERS HOSTS" << endl;
-				m_zone_badServers->loadXML( zone_doc );
+				m_zone_badServers->loadXML( zone_doc, errors );
 // 				kdDebug() << "KMFIPTDoc: Finished Parsing BADSERVERS HOSTS" << endl;
 			}
-		} else if ( curr.isElement() && curr.nodeName() == "logging" ) {
+		} else if ( curr.isElement() && curr.nodeName() == XML::Logging_Element ) {
 // 			kdDebug() << "\nKMFIPTDoc: Start Parsing Logging" << endl;
 			QString logDropped = "";
 			QString limitLog = "";
 			QString logPrefix = "";
 
-			logDropped = curr.toElement().attribute( "logDropped" );
-			limitLog = curr.toElement().attribute( "limitLog" );
-			logPrefix = curr.toElement().attribute( "logPrefix" );
+			logDropped = curr.toElement().attribute( XML::LogDropped_Attribute );
+			limitLog = curr.toElement().attribute( XML::LimitLog_Attribute );
+			logPrefix = curr.toElement().attribute( XML::LogPrefix_Attribute );
 
-			if ( logDropped == "bool:on")
+			if ( logDropped == XML::BoolOn_Value )
 				setLogDropped( true );
 			else
 				setLogDropped( false );
 
-			if ( limitLog == "bool:on")
+			if ( limitLog == XML::BoolOn_Value )
 				setLimitLog( true );
 			else
 				setLimitLog( false );
 
 			setLogPrefix( logPrefix );
 // 			kdDebug() << "KMFIPTDoc: Finished Parsing Logging" << endl;
-		} else if ( curr.isElement() && curr.nodeName() == "icmp" ) {
+		} else if ( curr.isElement() && curr.nodeName() == XML::ICMP_Element ) {
 // 			kdDebug() << "\nKMFIPTDoc: Start Parsing ICMP" << endl;
 			QString allowPing = "";
 			QString limitPing = "";
-			allowPing = curr.toElement().attribute( "allowPingReply" );
-			limitPing = curr.toElement().attribute( "limitPingReply" );
+			allowPing = curr.toElement().attribute( XML::AllowPingReply_Attribute );
+			limitPing = curr.toElement().attribute( XML::LimitPingReply_Attribute );
 
-			if ( allowPing == "bool:on")
+			if ( allowPing == XML::BoolOn_Value)
 				setAllowPingReply( true );
 			else
 				setAllowPingReply( false );
 
-			if ( limitPing == "bool:on")
+			if ( limitPing == XML::BoolOn_Value )
 				setLimitPingReply( true );
 			else
 				setLimitPingReply( false );
 
 // 			kdDebug() << "KMFIPTDoc: Finished Parsing ICMP" << endl;
-		} else if ( curr.isElement() && curr.nodeName() == "nat" ) {
+		} else if ( curr.isElement() && curr.nodeName() == XML::NAT_Element ) {
 // 			kdDebug() << "\nKMFIPTDoc: Start Parsing NAT" << endl;
 			QString useNat = "";
 			QString useMasquerade = "";
-// 			QString restrictNat = "";
 			QString natAddress = "";
-// 			QString allowedIncoming = "";
 			QString outgoingInterface = "";
-			useNat = curr.toElement().attribute( "useNat" );
-			useMasquerade = curr.toElement().attribute( "useMasquerade" );
-// 			restrictNat = curr.toElement().attribute( "restrictNat" );
-			natAddress = curr.toElement().attribute( "natAddress" );
-// 			allowedIncoming = curr.toElement().attribute( "allowedIncomingInterfaces" );
-			outgoingInterface = curr.toElement().attribute( "outgoingInterface" );
+			useNat = curr.toElement().attribute( XML::UseNat_Attribute );
+			useMasquerade = curr.toElement().attribute( XML::UseMasquerade_Attribute );
+			natAddress = curr.toElement().attribute( XML::NatAddress_Attribute );
+			outgoingInterface = curr.toElement().attribute( XML::OutgoingInterface_Attribute );
 
 
 
-			if ( useNat == "bool:on")
+			if ( useNat == XML::BoolOn_Value)
 				setUseNat( true );
 			else
 				setUseNat( false );
 
-			if ( useMasquerade == "bool:on")
+			if ( useMasquerade == XML::BoolOn_Value)
 				setUseMasquerade( true );
 			else
 				setUseMasquerade( false );
 
-// 			if ( restrictNat == "bool:on")
-// 				setRestrictNat( true );
-// 			else
-// 				setRestrictNat( false );
-
-// 			m_allowedIncomingInterfaces.clear();
-
-// 			kdDebug() << "Loading interfaces string: " <<  allowedIncoming << endl;
-/*			m_allowedIncomingInterfaces = QStringList::split("|", allowedIncoming,false);
-			QString s = "";
-			for ( QStringList::Iterator it = m_allowedIncomingInterfaces.begin(); it != m_allowedIncomingInterfaces.end(); ++it ) {
-				s += *it + " ";
- 			}*/
-// 			kdDebug() << "Stored Allowed NAT interfaces: "  << s << endl;
-
-
 			m_outgoingInterface = outgoingInterface;
 			m_natAddress->setAddress( natAddress );
 // 			kdDebug() << "Stored NAT address: " << m_natAddress->toString() << endl;
 // 			kdDebug() << "KMFIPTDoc: Finished Parsing NAT" << endl;
-		}else if ( curr.isElement() && curr.nodeName() == "abstract" ) {
+		}else if ( curr.isElement() && curr.nodeName() == XML::Abstract_Element ) {
 			kdDebug() << "KMFIPTDoc: Start Parsing Abstract" << endl;
 			QString allowIncomingConnections = "";
 			QString restrictOutgoingConnections = "";
 			QString description = "";
 			QString name = "";
-			allowIncomingConnections =curr.toElement().attribute( "allowIncomingConnections" );
-			restrictOutgoingConnections =curr.toElement().attribute( "restrictOutgoingConnections" );
+			allowIncomingConnections =curr.toElement().attribute( XML::AllowIncomingConnections_Attribute );
+			restrictOutgoingConnections =curr.toElement().attribute( XML::RestrictOutgoingConnections_Attribute );
 			
-			description += curr.toElement().attribute( "description" );
+			description += curr.toElement().attribute( XML::Description_Attribute );
 			if ( ! description.isNull() )
 				setDescription( *( new QString( description ) ) );
 			
-			name += curr.toElement().attribute( "name" );
+			name += curr.toElement().attribute(  XML::Name_Attribute );
 			if ( ! name.isNull() )
 				setName( *( new QString( name ) ) );
 			
 				
-			if ( allowIncomingConnections == "bool:on" )
+			if ( allowIncomingConnections == XML::BoolOn_Value )
 				setAllowIncomingConnections( true );
 			else
 				setAllowIncomingConnections( false );
 
-			if ( restrictOutgoingConnections == "bool:on" )
+			if ( restrictOutgoingConnections == XML::BoolOn_Value )
 				setRestrictOutgoingConnections( true );
 			else
 				setRestrictOutgoingConnections( false );
@@ -685,5 +559,5 @@ void KMFGenericDoc::loadXML( const QDomDocument& doc ) {
 	}
 }
 
-
+}
 
