@@ -5,7 +5,7 @@
 // Description:
 //
 //
-// Author: Christian Hubinger <chubinger@gmail.com>, (C) 2003
+// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2003
 //
 // Copyright: See COPYING file that comes with this distribution
 //
@@ -25,11 +25,16 @@
 
 #include <kmyfirewallgenericinterfaceicmp.h>
 
+// QT Includes
+#include <qguardedptr.h>
+
 /**
 @author Christian Hubinger
 */
-
+namespace KMF {
 class KMFGenericDoc;
+class KMFNetwork;
+class NetfilterObject;
 
 class KMFGenericInterfaceIcmp : public KMyFirewallGenericInterfaceIcmp {
 	Q_OBJECT
@@ -38,17 +43,18 @@ public:
 
 	~KMFGenericInterfaceIcmp();
 
-void loadDoc( KMFGenericDoc* doc );
+void loadDoc( KMFNetwork* doc );
 
 public slots:
 	void slotUpdateView();
+	void slotUpdateView( NetfilterObject* );
 
 private slots:
 	void slotEnablePingReplyToggled( bool );
 	void slotEnablePingReplyLimitToggled( bool );
 
 private:
-	KMFGenericDoc *m_doc;	
+	QGuardedPtr<KMFNetwork> m_doc;	
 };
-
+}
 #endif
