@@ -1,7 +1,7 @@
 /***************************************************************************
  begin                : Sat Mar 2 2002
  copyright            : (C) 2002 by Christian Hubinger
- email                : chubinger@gmail.com
+ email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -15,11 +15,7 @@
 #include "kmfchainedit.h"
 #include "kmfchainedit.moc"
 
-#include "../core/kmfiptdoc.h"
-#include "kmfnewchaindlg.h"
-//#include "kmfview.h"
-#include "../kmyfirewall.h" 
-#include "kmfnewchaindlg.h"
+
 //QT includes
 #include <qcheckbox.h>
 #include <qcombobox.h>
@@ -40,16 +36,25 @@
 #include <qinputdialog.h>
 #include <qtabwidget.h>
 
-#include <qtabwidget.h>
-
 // KDE includes
 #include <klocale.h>
 #include <kdebug.h>
 #include <kapplication.h> 
 #include <kmessagebox.h>
-// STD includes
-// #include <iostream>
 
+// Project Includes
+#include "../core/xmlnames.h"
+#include "../core/kmfiptdoc.h"
+#include "../core/kmfnetwork.h"
+#include "../core/kmfundoengine.h"
+
+// #include "../kmyfirewall.h" 
+
+
+
+
+
+namespace KMF {
 KMFChainEdit::KMFChainEdit( QWidget* parent, const char* name, WFlags fl )
 		: KMyFirewallChainEditor( parent, name, fl ) {}
 KMFChainEdit::~KMFChainEdit() {}
@@ -61,7 +66,7 @@ void KMFChainEdit::loadChain( IPTChain* chain ) {
 	c_log_limit->setChecked( false );
 	c_log_burst->setChecked( false );
 	c_log_prefix ->setChecked( false );
-	t_log_prefix->clear();
+	// t_log_prefix->clear();
 	m_tabWidget->setCurrentPage( 0 );
 
 	if ( ! chain ) {
@@ -130,15 +135,14 @@ void KMFChainEdit::loadChain( IPTChain* chain ) {
 		log_burst = m_chain->logBurst();
 		if ( logging ) {
 			c_enable_log->setChecked( true );
-			if ( !log_prefix.isEmpty() ) {
+			if ( ! log_prefix.isEmpty() && log_prefix != XML::Undefined_Value ) {
 				kdDebug() << "Found Log Prefix: " << log_prefix << endl;
 				c_log_prefix->setChecked( true );
 				t_log_prefix->setText( log_prefix );
 			} else {
 				c_log_prefix->setChecked( false );
-				t_log_prefix->setText( "" );
 			}
-			if ( !log_limit.isEmpty() ) {
+			if ( !log_limit.isEmpty() && log_limit != XML::Undefined_Value ) {
 				c_log_limit->setChecked( true );
 				int i = log_limit.find( "/" );
 				QString str_num = log_limit.left( i );
@@ -155,11 +159,12 @@ void KMFChainEdit::loadChain( IPTChain* chain ) {
 				}
 				sb_limit->setValue( str_num.toInt() );
 				kdDebug() << "Found Log Limit: " << str_num << "/" << interval << endl;
-				if ( !log_burst.isEmpty() ) {
+				if ( ! log_burst.isEmpty() && log_burst != XML::Undefined_Value ) {
 					c_log_burst->setChecked( true );
 					sb_burst->setValue( log_burst.toInt() );
 				} else {
 					c_log_burst->setChecked( false );
+					// sb_burst->setValue( 5 );
 				}
 			} else {
 				c_log_limit->setChecked( false );
@@ -177,8 +182,10 @@ void KMFChainEdit::loadChain( IPTChain* chain ) {
 
 void KMFChainEdit::accept() {
 	kdDebug() << "KMyFirewallChainEditor::slotSaveChainChanges()" << endl;
-	m_chain->table()->kmfDoc()->startTransaction();
-	m_chain->saveState();
+	KMFUndoEngine::instance()->startTransaction(
+		m_chain, 
+		i18n( "Edit Chain: %1 properties" ).arg( m_chain->name() ) 
+	);
 	bool log = c_enable_log->isChecked();
 	bool log_limit = c_log_limit->isChecked();
 	bool log_prefix = c_log_prefix->isChecked();
@@ -195,13 +202,13 @@ void KMFChainEdit::accept() {
 	} else
 		m_chain->hasCustomDefaultTarget( false );
 
-	QString str_log_limit = "";
-	QString str_log_prefix = "";
-	QString str_log_burst = "";
+	QString str_log_limit = XML::Undefined_Value;
+	QString str_log_prefix = XML::Undefined_Value;
+	QString str_log_burst = XML::Undefined_Value;
 	if ( log ) {
 		kdDebug() << "You wanna Loga a little bit" << endl;
 		// set log limit
-		if ( log_prefix && !t_log_prefix->text().isEmpty() ) {
+		if ( log_prefix && ! t_log_prefix->text().isEmpty() ) {
 			str_log_prefix = t_log_prefix->text();
 			kdDebug() << "You wanna have as log prefix: " << str_log_prefix << endl;
 		}
@@ -211,23 +218,25 @@ void KMFChainEdit::accept() {
 			str_log_limit = str_limit_num + "/" + str_interval;
 			kdDebug() << "You wanna have " << str_log_limit << " as Logging limit" << endl;
 
-			if ( log_burst )
+			if ( log_burst ) {
 				str_log_burst = sb_burst->text();
-			else
-				str_log_burst = "";
-		} else // unset log limit
-			str_log_limit = "";
+			} else {
+				str_log_burst = XML::Undefined_Value;
+			}
+		} else {
+			str_log_limit = XML::Undefined_Value;
+		}
 		m_chain->setDropLogging( true, str_log_limit, str_log_burst, str_log_prefix );
 	} else {  // disable chain_logging
-		str_log_limit = "";
-		str_log_prefix = "";
-		str_log_burst = "";
+		str_log_limit = XML::Undefined_Value;
+		str_log_prefix = XML::Undefined_Value;
+		str_log_burst = XML::Undefined_Value;
 		m_chain->setDropLogging( false, str_log_limit, str_log_burst, str_log_prefix );
 	}
 
-	m_chain->table() ->kmfDoc() ->changed();
+	m_chain->table()->changed();
 	emit sigDocumentChanged();
-	m_chain->table()->kmfDoc()->endTransaction();
+	KMFUndoEngine::instance()->endTransaction();
 	emit sigHideMe();
 }
 
@@ -240,4 +249,4 @@ void KMFChainEdit::reject() {
 	emit sigHideMe();
 }
 
-
+}
