@@ -1,7 +1,7 @@
 /***************************************************************************
  begin                : Sat Mar 2 2002
  copyright            : (C) 2002 by Christian Hubinger
- email                : chubinger@gmail.com
+ email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -24,6 +24,7 @@
   *@author Christian Hubinger
   */
 
+namespace KMF {
 class KMFChainEdit : public KMyFirewallChainEditor {
 	Q_OBJECT
 public:
@@ -43,5 +44,5 @@ signals:
 	void sigDocumentChanged();
 	void sigHideMe();
 };
-
+}
 #endif
