@@ -1,7 +1,7 @@
 /***************************************************************************
 begin                : Thu Feb 7  2002
 copyright            : (C) 2002 by Christian Hubinger
-email                : chubinger@gmail.com
+email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -51,8 +51,10 @@ email                : chubinger@gmail.com
 #include <qwidgetstack.h>
 
 // project includes
+#include "../core/xmlnames.h"
 #include "../core/kmfdoc.h"
 #include "../core/kmfiptdoc.h"
+#include "../core/kmfnetwork.h"
 #include "../core/iptchain.h"
 #include "../core/iptrule.h"
 #include "../core/kmfcheckinput.h"
@@ -61,20 +63,25 @@ email                : chubinger@gmail.com
 #include "../core/kmfconfig.h"
 #include "../core/kmfpluginfactory.h"
 #include "../core/kmfappstate.h"
+#include "../core/kmyfirewallinterface.h"
+#include "../core/kmfruleoptioneditinterface.h"
+#include "../core/kmfruletargetoptioneditinterface.h"
+
 
 #include "../kmfwidgets/kmflistview.h"
+#include "../kmfwidgets/kmfmynetworkwidget.h"
 #include "../kmfwidgets/kmflistviewitem.h"
 #include "../kmfwidgets/kmfobjectinfo.h"
-#include "../interfaces/kmfruleoptioneditinterface.h"
-#include "../interfaces/kmfruletargetoptioneditinterface.h"
+#include "../kmfwidgets/kmfmainwindow.h"
 
 #include "kmfchainedit.h"
 #include "kmfnewchaindlg.h"
-
+namespace KMF {
 KMFRuleEdit::KMFRuleEdit( QWidget* parent, const char* name, WFlags fl )
 		: KMyFirewallRuleEditor( parent, name, fl ), KMFRuleEditInterface() {
 	if ( !name )
 		setName( "KMFRuleEdit" );
+	m_lastDisplayDoc = 0;
 	loadIcons();
 	m_editPlugins.clear();
 	b_move_up->setPixmap( icon_up );
@@ -98,7 +105,10 @@ KMFRuleEdit::KMFRuleEdit( QWidget* parent, const char* name, WFlags fl )
 
 	connect( m_lv_table_filter, SIGNAL( clicked( QListViewItem* ) ),
 	         this, SLOT( slotNewItemSelected( QListViewItem* ) ) );
-
+	
+// 	connect( this, SIGNAL( sigUpdateView() ),
+// 	         m_lv_table_filter, SLOT( slotUpdateView() ) );
+	
 	m_widgetStack->addWidget( m_lv_table_filter );
 
 
@@ -127,7 +137,9 @@ KMFRuleEdit::KMFRuleEdit( QWidget* parent, const char* name, WFlags fl )
 
 	connect( m_lv_table_mangle, SIGNAL( clicked( QListViewItem* ) ),
 	         this, SLOT( slotNewItemSelected( QListViewItem* ) ) );
-
+	
+	
+	
 	m_widgetStack->addWidget( m_lv_table_mangle );
 	
 	m_splitter->setResizeMode( gb_options, QSplitter::KeepSize );
@@ -149,6 +161,14 @@ KMFRuleEdit::KMFRuleEdit( QWidget* parent, const char* name, WFlags fl )
 	connect( m_new_chain, SIGNAL( sigUpdateView() ),
 	         this, SIGNAL( sigUpdateView() ) );
 
+	m_myNetworkWidget = new KMFMyNetworkWidget( this, "KMFMyNetworkWidget" );
+	connect( this, SIGNAL( sigUpdateView() ),
+	         m_myNetworkWidget, SLOT( slotUpdateView() ) );
+	connect( this, SIGNAL( sigUpdateView( NetfilterObject* ) ),
+	         m_myNetworkWidget, SLOT( slotUpdateView( NetfilterObject* ) ) );
+	
+	connect( m_myNetworkWidget, SIGNAL( sigActiveTargetChanged() ), this, SLOT( slotUpdateView() ) );
+	m_myNetworkWidget->hide(); //close( false );
 
 	m_check_input = new KMFCheckInput();
 	m_contextMenu = new KPopupMenu( this );
@@ -164,25 +184,29 @@ KMFRuleEdit::KMFRuleEdit( QWidget* parent, const char* name, WFlags fl )
 	m_rule = 0;
 	m_chain = 0;
 	m_table = 0;
-	kmfdoc = 0;
+	m_network = 0;
 	m_widgetStack->show();
-	slotNewTableSelected();
-	slotShowOverview();
+	// slotNewTableSelected();
+	// slotShowOverview();
 	adjustSize();
 	show();
 }
 
 KMFRuleEdit::~KMFRuleEdit() {}
 
+void KMFRuleEdit::setKMFMainWindow( KMFMainWindow* win ) {
+	m_app = win;
+}
+
 void KMFRuleEdit::slotSelectionInvalid() {
 	m_rule = 0;
 	m_chain = 0;
 	m_table = 0;
 }
 
-void KMFRuleEdit::loadDoc( KMFIPTDoc* doc ) {
+void KMFRuleEdit::loadDoc( KMFNetwork* network ) {
 	kdDebug() << "void KMFRuleEdit::loadDoc(KMFIPTDoc* doc)" << endl;
-	if ( doc == 0 ) {
+	if ( network == 0 ) {
 		m_err->setErrType(KMFError::NORMAL );
 		const QString& msg = i18n( "KMFRuleEdit:::loadDoc(KMFIPTDoc* doc)\n"
 		                           "KMFIPTDoc* doc == 0. This is a bug." );
@@ -192,15 +216,16 @@ void KMFRuleEdit::loadDoc( KMFIPTDoc* doc ) {
 		return ;
 	}
 
-	if ( !isEnabled() )
+	if ( !isEnabled() ) {
 		setEnabled( true );
-	kmfdoc = doc;
-// 	m_output_viewer->loadDoc( kmfdoc );
-
-	rb_filter->setEnabled( kmfdoc->useFilter() );
-	rb_mangle->setEnabled( kmfdoc->useMangle() );
-	rb_nat->setEnabled( kmfdoc->useNat() );
-	if ( kmfdoc->useModules() ) {
+	}
+	m_network = network;
+	m_myNetworkWidget->setNetwork( m_network );
+	
+	rb_filter->setEnabled( m_network->currentDocAsIPTDoc()->useFilter() );
+	rb_mangle->setEnabled( m_network->currentDocAsIPTDoc()->useMangle() );
+	rb_nat->setEnabled( m_network->currentDocAsIPTDoc()->useNat() );
+	if ( m_network->currentDocAsIPTDoc()->useModules() ) {
 		m_led_modules->setColor( green );
 		m_led_modules->on();
 	} else {
@@ -208,7 +233,7 @@ void KMFRuleEdit::loadDoc( KMFIPTDoc* doc ) {
 		m_led_modules->off();
 	}
 
-	if ( kmfdoc->useIPFwd() ) {
+	if ( m_network->currentDocAsIPTDoc()->useIPFwd() ) {
 		m_led_fwd->setColor( green );
 		m_led_fwd->on();
 	} else {
@@ -216,7 +241,7 @@ void KMFRuleEdit::loadDoc( KMFIPTDoc* doc ) {
 		m_led_fwd->off();
 	}
 
-	if ( kmfdoc->useSynCookies() ) {
+	if ( m_network->currentDocAsIPTDoc()->useSynCookies() ) {
 		m_led_syn->setColor( green );
 		m_led_syn->on();
 	} else {
@@ -224,7 +249,7 @@ void KMFRuleEdit::loadDoc( KMFIPTDoc* doc ) {
 		m_led_syn->off();
 	}
 
-	if ( kmfdoc->useRPFilter() ) {
+	if ( m_network->currentDocAsIPTDoc()->useRPFilter() ) {
 		m_led_rp->setColor( green );
 		m_led_rp->on();
 	} else {
@@ -232,49 +257,55 @@ void KMFRuleEdit::loadDoc( KMFIPTDoc* doc ) {
 		m_led_rp->off();
 	}
 
-	if ( kmfdoc->useMartians() ) {
+	if ( m_network->currentDocAsIPTDoc()->useMartians() ) {
 		m_led_martians->setColor( green );
 		m_led_martians->on();
 	} else {
 		m_led_martians->setColor( red );
 		m_led_martians->off();
 	}
+	
 
+	
 	IPTable *filter = 0;
-	filter = kmfdoc->table( "filter" );
-	if ( !filter )
-		return ;
-	m_lv_table_filter->clearAllItems();
-	m_lv_table_filter->slotLoadNode( filter );
-	IPTable *nat = 0;
-	nat = kmfdoc->table( "nat" );
-	if ( !nat )
-		return ;
-	m_lv_table_nat->clearAllItems();
-	m_lv_table_nat->slotLoadNode( nat );
-	IPTable *mangle = 0;
-	mangle = kmfdoc->table( "mangle" );
-	if ( !mangle )
-		return ;
-	m_lv_table_mangle->clearAllItems();
-	m_lv_table_mangle->slotLoadNode( mangle );
+	filter = m_network->currentDocAsIPTDoc()->table( Constants::FilterTable_Name );
+	if ( filter ) {
+		m_lv_table_filter->clearAllItems();
+		m_lv_table_filter->slotLoadNode( filter );
+	}
 
+//	IPTable *def = 0;
+	IPTable *nat = 0;
+	nat = m_network->currentDocAsIPTDoc()->table( Constants::NatTable_Name );
+	if ( nat ) {
+		m_lv_table_nat->clearAllItems();
+		m_lv_table_nat->slotLoadNode( nat );
+	}
+	
+	IPTable *mangle = 0;
+	mangle = m_network->currentDocAsIPTDoc()->table( Constants::MangleTable_Name );
+	if ( mangle ) {
+		m_lv_table_mangle->clearAllItems();
+		m_lv_table_mangle->slotLoadNode( mangle );
+	}
+	
 	m_rule = 0;
 	m_chain = 0;
 	m_table = 0;
+	
 	enableRuleEdit( false );
 	slotShowOverview();
-	if ( KMFAppState::upAndRunning() ) {
-		emit sigUpdateView();
+	// if ( KMFAppState::upAndRunning() ) {
 		rb_filter->setChecked( true );
 		slotNewTableSelected();
-	}
+		emit sigUpdateView();
+	//}
 }
 
-void KMFRuleEdit::slotLoadDocument( KMFIPTDoc * doc ) {
-	kdDebug() << "KMFRuleEdit::slotLoadDocument( KMFIPTDoc* doc )" << endl;
+void KMFRuleEdit::slotLoadDocument( KMFNetwork* network ) {
+	kdDebug() << "KMFRuleEdit::slotLoadDocument( KMFNetwork* network )" << endl;
 	slotShowOverview();
-	loadDoc( doc );
+	loadDoc( network );
 }
 
 
@@ -327,11 +358,11 @@ KMFListView* KMFRuleEdit::currTableView() {
 
 void KMFRuleEdit::slotNewTableSelected() {
 	kdDebug() << "void KMFRuleEdit::slotNewTableSelected()" << endl;
-	if ( kmfdoc == 0 ) {
+	if ( ! m_network->currentDocAsIPTDoc() ) {
 		setEnabled( false );
 		return ;
 	}
-	QString table = "filter";
+	QString table = Constants::FilterTable_Name;
 	if ( !rb_filter->isEnabled() )
 		rb_filter->setChecked( false );
 	if ( !rb_nat->isEnabled() )
@@ -341,25 +372,25 @@ void KMFRuleEdit::slotNewTableSelected() {
 
 
 	if ( rb_filter->isChecked() && rb_filter->isEnabled() ) {
-		table = "filter";
+		table = Constants::FilterTable_Name;
 		kdDebug() << "Enabling Filter View" << endl;
 		setCurrTableView( m_lv_table_filter );
 	} else if ( rb_nat->isChecked() && rb_nat->isEnabled() ) {
 		setEnabled( true );
-		table = "nat";
+		table = Constants::NatTable_Name;
 		setCurrTableView( m_lv_table_nat );
 	} else if ( rb_mangle->isChecked() && rb_mangle->isEnabled() ) {
 		setEnabled( true );
-		table = "mangle";
+		table = Constants::MangleTable_Name;
 		setCurrTableView( m_lv_table_mangle );
 	} else {
 		setEnabled( true );
-		table = "filter";
+		table = Constants::FilterTable_Name;
 		setCurrTableView( m_lv_table_filter );
 		return;
 	}
 
-	IPTable *tab = kmfdoc->table( table );
+	IPTable *tab = m_network->currentDocAsIPTDoc()->table( table );
 	if ( tab == 0 ) {
 		setEnabled( false );
 		kdDebug() << "Table: " << table << " not found in document!!" << table << endl;
@@ -377,6 +408,7 @@ void KMFRuleEdit::setCurrTableView( KMFListView* lv ) {
 		m_lv_table_filter->setEnabled( true );
 		connect( this, SIGNAL( sigUpdateView() ),
 	         m_lv_table_filter, SLOT( slotUpdateView() ) );
+		
 		connect( this, SIGNAL( sigUpdateView( NetfilterObject* ) ),
 	         m_lv_table_filter, SLOT( slotUpdateView( NetfilterObject* ) ) );
 
@@ -469,25 +501,25 @@ void KMFRuleEdit::slotNewItemSelected( QListViewItem* item ) {
 	if ( KMFListViewItem * kmf_item = dynamic_cast<KMFListViewItem*>( item ) ) {
 		IPTRuleOption * opt = 0;
 		switch( kmf_item->type() ) {
-			case KMFListViewItem::TABLE:
+			case NetfilterObject::TABLE:
 				m_rule = 0;
 				m_chain = 0;
 				m_table = kmf_item->table();
 				slotEditRule();
 				break;
-			case KMFListViewItem::CHAIN:
+			case NetfilterObject::CHAIN:
 				m_rule = 0;
 				m_chain = kmf_item->chain();
 				m_table = m_chain->table();
 				slotEditRule();
 				break;
-			case KMFListViewItem::RULE:
+			case NetfilterObject::RULE:
 				m_rule = kmf_item->rule();
 				m_chain = m_rule->chain();
 				m_table = m_rule->chain() ->table();
 				slotEditRule();
 				break;
-			case KMFListViewItem::RULEOPTION:
+			case NetfilterObject::RULEOPTION:
 				opt = kmf_item->ruleOption();
 				m_rule = opt->rule();
 				m_chain = opt->rule() ->chain();
@@ -527,13 +559,23 @@ void KMFRuleEdit::slotAddRule() {
 		if ( !text.isEmpty() && !ch.isEmpty() && !tab.isEmpty() && !target.isEmpty() ) {
 			m_check_input->checkInput( text, "RULENAME", m_err );
 			if ( m_err_handler->showError( m_err ) ) {
-				kmfdoc->startTransaction();
-				m_chain->saveState();
-				m_chain->addRule( text, m_err );
+				KMFUndoEngine::instance()->startTransaction( 
+					m_chain,
+					i18n( "Add Rule: %1 to Chain: %2" ).arg( text ).arg( m_chain->name() ) 
+				);
+				
+				IPTRule *inserted = m_chain->addRule( text, m_err );
 				if ( m_err_handler->showError( m_err ) ) {
-					kmfdoc->changed();
+					if ( m_rule ) {
+						m_chain->moveRule( inserted, ( m_rule->ruleNum() ) -inserted->ruleNum() );
+					}
+//					m_network->changed();
+					KMFUndoEngine::instance()->endTransaction();
+				} else {
+					KMFUndoEngine::instance()->abortTransaction();
 				}
-				kmfdoc->endTransaction();
+				
+				m_rule = inserted;
 				emit sigUpdateView( m_chain );
 			}
 		} else {
@@ -573,22 +615,27 @@ void KMFRuleEdit::slotDelChain() {
 	kdDebug() << "Message Box returned: " << doit << endl;
 	if ( doit == 3 ) {
 		kdDebug() << "Try to delete Chain" << endl;
-		kmfdoc->startTransaction();
-		m_table->saveState();
-		m_err = kmfdoc->table( m_chain->table()->name() )->delChain( m_chain );
+		KMFUndoEngine::instance()->startTransaction( 
+			m_table,
+			i18n( "Delete Chain: %1 from Table: %2").arg( m_chain->name() ).arg( m_chain->table()->name() )  
+		);
+		m_err = m_network->currentDocAsIPTDoc()->table( m_chain->table()->name() )->delChain( m_chain );
 		if ( m_err_handler ->showError( m_err ) ) {
-			emit sigUpdateView();
+			emit sigUpdateView( m_chain->table() );
 			m_chain = 0;
 			m_rule = 0;
 			m_table = 0;
+			KMFUndoEngine::instance()->endTransaction();
+		} else {
+			KMFUndoEngine::instance()->abortTransaction();
 		}
-		kmfdoc->endTransaction();
+		
 	}
 }
 
 void KMFRuleEdit::slotEditRule() {
 	m_ws_target_opt->setEnabled(false);	
-	if ( ! KMFAppState::upAndRunning() || ! KMFAppState::hasOpenDoc() ) {
+	if ( ! KMFAppState::upAndRunning() /* || ! KMFAppState::hasOpenDoc() */ ) {
 		slotSelectionInvalid();
 		return;
 	}
@@ -617,10 +664,10 @@ void KMFRuleEdit::slotEditRule() {
 		IPTRuleOption *opt = m_rule->getOptionForName( "frag_opt" );
 		if ( !opt->isEmpty() ) {
 			QStringList vals = opt->getValues();
-			if ( *vals.at( 0 ) == "bool:on" && *vals.at( 0 ) == "bool:off" ) {
+			if ( *vals.at( 0 ) == XML::BoolOn_Value && *vals.at( 0 ) == XML::BoolOff_Value ) {
 				cb_frag->setChecked( true );
 				cb_frag_inv->setChecked( false );
-			} else if ( *vals.at( 0 ) == "bool:off" && *vals.at( 1 ) == "bool:on" ) {
+			} else if ( *vals.at( 0 ) == XML::BoolOff_Value && *vals.at( 1 ) == XML::BoolOn_Value ) {
 				cb_frag->setChecked( true );
 				cb_frag_inv->setChecked( true );
 			} else {
@@ -681,35 +728,70 @@ void KMFRuleEdit::enableRuleEdit( bool on ) {
 
 void KMFRuleEdit::slotMoveRuleDown() {
 	kdDebug() << "KMFRuleEdit::slotMoveRuleDown()" << endl;
-	if ( m_rule  && kb_optSelect->isEnabled() ) {
-		kmfdoc->startTransaction();
-		m_chain->saveState();
-		if ( m_chain->moveRule( m_rule, 1 ) ) {
-			kmfdoc->changed();
-			KMFListViewItem* item = currTableView() ->findKMFItem( m_rule->name(),2, m_rule->objectID() );
-			if ( item )
-				currTableView()->setSelected( item, true);
-			item->loadNetfilterObject( m_rule );
-			emit sigUpdateView( m_rule->chain() );
-		}
-		kmfdoc->endTransaction();
-	}
+	moveRuleInChain( 1 );
+// 	if ( m_rule  && kb_optSelect->isEnabled() ) {
+// 		KMFUndoEngine::instance()->startTransaction( 
+// 			m_chain,
+// 			i18n("Move Rule: %1 down").arg( m_rule->name() ) 
+// 		);
+// 		m_chain->saveState();
+// 		if ( m_chain->moveRule( m_rule, 1 ) ) {
+// 			m_rule->chain()->changed();
+// 			KMFListViewItem* item = currTableView()->findKMFItem( m_rule->name(), 2, m_rule->uuid() );
+// 			if ( item ) {
+// 				currTableView()->setSelected( item, true );
+// 			}
+// 			item->loadNetfilterObject( m_rule );
+// 			emit sigUpdateView( m_rule->chain() );
+// 			KMFUndoEngine::instance()->endTransaction();
+// 		} else {
+// 			KMFUndoEngine::instance()->abortTransaction();
+// 		}
+// 		
+// 	}
 }
 
 void KMFRuleEdit::slotMoveRuleUp() {
 	kdDebug() << "KMFRuleEdit::slotMoveRuleUp()" << endl;
+	moveRuleInChain( -1 );
+// 	if ( m_rule && kb_optSelect->isEnabled() ) {
+// 		KMFListViewItem* item = currTableView() ->findKMFItem( m_rule->name(),2, m_rule->uuid() );
+// 		if ( item ) {
+// 			KMFUndoEngine::instance()->startTransaction( 
+// 				m_chain,
+// 				i18n("Move Rule: %1 up").arg( m_rule->name() ) 
+// 			);
+// 			m_chain->saveState();
+// 			if ( m_chain->moveRule( m_rule, -1 ) ) {
+// 				m_rule->chain()->changed();
+// 				currTableView()->setSelected( item, true );
+// 				item->loadNetfilterObject( m_rule );
+// 				emit sigUpdateView( m_rule->chain()  );
+// 			}
+// 			KMFUndoEngine::instance()->endTransaction();
+// 		}
+// 	}
+}
+
+void KMFRuleEdit::moveRuleInChain( int amount ) {
 	if ( m_rule && kb_optSelect->isEnabled() ) {
-		KMFListViewItem* item = currTableView() ->findKMFItem( m_rule->name(),2, m_rule->objectID() );
+		const QString& transMsg = i18n("Move Rule: %1 %2").arg( m_rule->name() ).arg( amount > 0 ? "down" : "up" );
+		KMFListViewItem* item = currTableView()->findKMFItem( m_rule->name(),2, m_rule->uuid() );
 		if ( item ) {
-			kmfdoc->startTransaction();
-			m_chain->saveState();
+			KMFUndoEngine::instance()->startTransaction( 
+				m_chain,
+				transMsg.arg( m_rule->name() ) 
+			);
 			if ( m_chain->moveRule( m_rule, -1 ) ) {
-				kmfdoc->changed();
+				m_rule->chain()->changed();
 				currTableView()->setSelected( item, true );
 				item->loadNetfilterObject( m_rule );
+				KMFUndoEngine::instance()->endTransaction();
 				emit sigUpdateView( m_rule->chain()  );
+			} else {
+				KMFUndoEngine::instance()->abortTransaction();
 			}
-			kmfdoc->endTransaction();
+			
 		}
 	}
 }
@@ -728,19 +810,24 @@ void KMFRuleEdit::slotDelRule() {
 		kdDebug() << "Message Box returned: " << doit << endl;
 		if ( doit == 3 ) { // OK clicked
 			kdDebug() << "clicked ok" << endl;
-			kmfdoc->startTransaction();
-			m_chain->saveState();
+			KMFUndoEngine::instance()->startTransaction( 
+				m_chain,
+				i18n("Delete Rule: %1 from Chain: %2").arg( m_rule->name() ).arg( m_chain->name() ) 
+			);
 			m_err = m_chain->delRule( m_rule );
-			m_err_handler->showError( m_err );
-			kmfdoc->changed();
-			kdDebug() << "KMFRuleEdit::slotDelRule()\nCalling: kmfdoc->updateView()" << endl;
-			m_rule = 0;
-			slotEditRule();
-			kmfdoc->endTransaction();
-			if ( m_chain )
-				emit sigUpdateView( m_chain );
-			else
-				emit sigUpdateView();
+			if ( m_err_handler->showError( m_err ) ) {
+				kdDebug() << "KMFRuleEdit::slotDelRule()\nCalling: m_network->currentDocAsIPTDoc()->updateView()" << endl;
+				m_rule = 0;
+				slotEditRule();
+				KMFUndoEngine::instance()->endTransaction();
+				if ( m_chain ) {
+					emit sigUpdateView( m_chain );
+				} else {
+					emit sigUpdateView();
+				}
+			} else {
+				KMFUndoEngine::instance()->abortTransaction();
+			}
 		}
 	}
 }
@@ -749,7 +836,7 @@ void KMFRuleEdit::showOverview() {
 }
 
 void KMFRuleEdit::slotShowOverview() {
-	kdDebug() << "\nKMFRuleEdit::slotShowOverview()\n" << endl;
+	kdDebug() << "KMFRuleEdit::slotShowOverview()" << endl;
 	if ( rb_filter->isChecked() ) {
 		m_widgetStack->raiseWidget( m_lv_table_filter );
 	} else if ( rb_nat->isChecked() ) {
@@ -757,10 +844,13 @@ void KMFRuleEdit::slotShowOverview() {
 	} else if ( rb_mangle->isChecked() ) {
 		m_widgetStack->raiseWidget( m_lv_table_mangle );
 	}
-	if ( kmfdoc )
-		kmfdoc->endTransaction();
-	if ( m_table )
+/*	if ( m_network->currentDocAsIPTDoc() ) {
+		KMFUndoEngine::instance()->endTransaction();
+	}*/
+	if ( m_table ) {
+		kdDebug() << "emit KMFRuleEdit::sigUpdateView()" << endl;
 		emit sigUpdateView();
+	}
 	slotEditRule();
 }
 
@@ -871,6 +961,18 @@ void KMFRuleEdit::slotEditChain() {
 	m_widgetStack->raiseWidget( m_editchain );
 }
 
+void KMFRuleEdit::slotEditNetwork() {
+	kdDebug() << "void KMFRuleEdit::slotEditNewtwork()" << endl;
+	// kdDebug() << "Parent is: " << parent()->className() << endl;
+// 	KMFMyNetworkWidget *wid = new KMFMyNetworkWidget( this, "name" );
+// 	wid->setNetwork( m_network );
+// 	wid->slotUpdateView();
+/*	connect( wid, SIGNAL( sigActiveTargetChanged() ), this, SLOT( slotUpdateView() ) );*/
+	m_myNetworkWidget->slotUpdateView();
+	m_app->setOutputWidget( m_myNetworkWidget );
+	m_app->showOutput();
+}
+
 // Slots For Rule Edit Changes
 void KMFRuleEdit::slotFragChanged() {
 	kdDebug() << "KMFRuleEdit::slotFragChanged(bool frag)" << endl;
@@ -883,50 +985,56 @@ void KMFRuleEdit::slotFragChanged() {
 	QString* cmd = new QString( "frag_opt" );
 	if ( frag ) {
 		if ( frag_inv ) {
-			vals->append( new QString( "bool:off" ) );
-			vals->append( new QString( "bool:on" ) );
+			vals->append( new QString( XML::BoolOff_Value ) );
+			vals->append( new QString( XML::BoolOn_Value ) );
 		} else {
-			vals->append( new QString( "bool:on" ) );
-			vals->append( new QString( "bool:off" ) );
+			vals->append( new QString( XML::BoolOn_Value ) );
+			vals->append( new QString( XML::BoolOff_Value ) );
 		}
 	} else {
-		vals->append( new QString( "bool:off" ) );
-		vals->append( new QString( "bool:off" ) );
+		vals->append( new QString( XML::BoolOff_Value ) );
+		vals->append( new QString( XML::BoolOff_Value ) );
 	}
-	kmfdoc->startTransaction();
-	m_rule->saveState();
+	KMFUndoEngine::instance()->startTransaction( 
+		m_rule,
+		i18n( "Edit Rule: %1 Fragment option" ).arg( m_rule->name() ) 
+	);
 	m_rule->addRuleOption( *cmd, *vals );
-	kmfdoc->changed();
-	kmfdoc->endTransaction();
+	KMFUndoEngine::instance()->endTransaction();
 	emit sigUpdateView( m_rule );
 }
 
 void KMFRuleEdit::slotLogRuleChanged() {
 	if ( !m_rule )
 		return ;
-	kmfdoc->startTransaction();
-	m_rule->saveState();
-	if ( cb_log_rule->isChecked() )
+	KMFUndoEngine::instance()->startTransaction( 
+		m_rule,
+		i18n( "Edit Rule: %1 Log option" ).arg( m_rule->name() ) 
+	);
+	if ( cb_log_rule->isChecked() ) {
 		m_rule->setLogging( true );
-	else
+	} else {
 		m_rule->setLogging( false );
-	kmfdoc->changed();
-	kmfdoc->endTransaction();
+	}
+//	m_network->changed();
+	KMFUndoEngine::instance()->endTransaction();
 	emit sigUpdateView( m_rule );
 }
 
 void KMFRuleEdit::slotEnableRuleChanged() {
 	if ( !m_rule )
 		return ;
-	kmfdoc->startTransaction();
-	m_rule->saveState();
+	KMFUndoEngine::instance()->startTransaction( 
+		m_rule,
+		i18n( "Edit Rule: %1 enabled state" ).arg( m_rule->name() ) 
+	);
 	if ( cb_disable_rule->isChecked() ) {
 		m_rule->setEnabled( false );
 	} else {
 		m_rule->setEnabled( true );
 	}
-	kmfdoc->changed();
-	kmfdoc->endTransaction();
+//	m_network->changed();
+	KMFUndoEngine::instance()->endTransaction();
 	emit sigUpdateView( m_rule );
 	slotEditRule();
 }
@@ -949,18 +1057,19 @@ void KMFRuleEdit::slotTargetChanged( const QString & tg ) {
 			                                      " defined <b>Target Options</b> will be deleted.<br>"
 			                                      "Click <b>Continue</b> to change the Target.<br>"
 			                                      "Click <b>Cancel</b> keep your current Target. </qt>" ), i18n( "Warning" ), KStdGuiItem::cont(), KStdGuiItem::cancel(), "change_target_option_warning" );
-			kdDebug() << "\n\n\nKMessageBox:: Returned" << ans << "\n\n\n" << endl;
+			kdDebug() << "KMessageBox:: Returned" << ans << endl;
 			switch ( ans ) {
 
 			case 3:
 				kdDebug() << "Clicked Continue" << endl;
 				if ( m_rule->target() != tg ) {
-					kmfdoc->startTransaction();
-					m_rule->saveState();
+					KMFUndoEngine::instance()->startTransaction( 
+						m_rule,
+						i18n( "Change Rule: %1 target from: %2 to: %3" ).arg( m_rule->name() ).arg( m_rule->target() ).arg( tg ) 
+					);
 					m_rule->setTarget( tg );
-					kmfdoc->changed();
-					emit sigUpdateView();
-					kmfdoc->endTransaction();
+					emit sigUpdateView( m_rule->chain()->table() );
+					KMFUndoEngine::instance()->endTransaction();
 				}
 				break;
 			case 2:
@@ -972,12 +1081,13 @@ void KMFRuleEdit::slotTargetChanged( const QString & tg ) {
 			}
 		} else {
 			if ( m_rule->target() != tg ) {
-				kmfdoc->startTransaction();
-				m_rule->saveState();
+				KMFUndoEngine::instance()->startTransaction( 
+					m_rule,
+					i18n( "Change Rule: %1 target from: %2 to: %3" ).arg( m_rule->name() ).arg( m_rule->target() ).arg( tg ) 
+				);
 				m_rule->setTarget( tg );
-				kmfdoc->changed();
-				emit sigUpdateView();
-				kmfdoc->endTransaction();
+				emit sigUpdateView( m_rule->chain()->table() );
+				KMFUndoEngine::instance()->endTransaction();
 			}
 		}
 		slotEditTargetOption();
@@ -991,7 +1101,6 @@ void KMFRuleEdit::addRuleOption(QString* name, QPtrList< QString >* values){
 void KMFRuleEdit::slotAddRuleOption( QString * name, QPtrList<QString>* values ) {
 	kdDebug() << "KMFRuleEdit::slotAddRuleOption(QString* name, QPtrList<QString>* values)" << endl;
 	m_rule->addRuleOption( *name, *values );
-	kmfdoc->changed();
 	emit sigUpdateView( m_rule );
 }
 
@@ -1017,13 +1126,13 @@ void KMFRuleEdit::slotRuleRBM( QListViewItem * item, const QPoint & point, int )
 	}
 	if ( KMFListViewItem * tmp_item = dynamic_cast<KMFListViewItem*>( item ) ) {
 		slotNewItemSelected( item );
-		if ( ( tmp_item->type() == KMFListViewItem::RULE || tmp_item->type() == KMFListViewItem::RULEOPTION )&& m_rule ) {
+		if ( ( tmp_item->type() == NetfilterObject::RULE || tmp_item->type() == NetfilterObject::RULEOPTION )&& m_rule ) {
 			createRBM( m_rule );
 			m_contextMenu->popup( point );
-		} else if ( tmp_item->type() == KMFListViewItem::CHAIN && m_chain ) {
+		} else if ( tmp_item->type() == NetfilterObject::CHAIN && m_chain ) {
 			createRBM( m_chain );
 			m_contextMenu->popup( point );
-		} else if ( tmp_item->type() == KMFListViewItem::TABLE && m_table ) {
+		} else if ( tmp_item->type() == NetfilterObject::TABLE && m_table ) {
 			createRBM( m_table );
 			m_contextMenu->popup( point );
 		} else {
@@ -1081,7 +1190,9 @@ void KMFRuleEdit::createRBM( NetfilterObject* obj ) {
 			connect( sub_edit,SIGNAL(activated( int ) ), 
 				this,SLOT(slotNewOptionType( int ) ) );			
 			m_contextMenu->insertItem( i18n( "Edit Rule Option" ), sub_edit );
-			m_contextMenu->insertItem( icon_new, i18n( "Add Rule..." ), this, SLOT( slotAddRule() ) );
+			// m_contextMenu->insertItem( icon_new, i18n( "Add Rule..." ), this, SLOT( slotAddRule() ) );
+			m_contextMenu->insertItem( icon_new, i18n( "Insert Rule..." ), this, SLOT( slotAddRule() ) );
+//			m_contextMenu->insertSeparator();
 			m_contextMenu->insertItem( icon_del, i18n( "Delete Rule" ), this, SLOT( slotDelRule() ) );
 			m_contextMenu->insertSeparator();
 			m_contextMenu->insertItem( icon_rename, i18n( "Rename Rule" ), this, SLOT( slotRenameRule() ) );
@@ -1122,16 +1233,19 @@ void KMFRuleEdit::slotMoveRule( int index ) {
 	kdDebug() << "void KMFRuleEdit::slotCopyRule( int )" << endl;
 	kdDebug() << "Move Rule  to Chain Nr: " << index << endl;
 	IPTChain *chain_target = 0;
-	chain_target = kmfdoc->table( m_table->name() ) ->chains().at( index );
+	chain_target = m_network->currentDocAsIPTDoc()->table( m_table->name() ) ->chains().at( index );
 	if ( chain_target ) {
-		kmfdoc->startTransaction();
-		m_rule->chain()->table()->saveState();
-		chain_target->saveState();
-		m_err = m_rule->chain() ->table() ->moveRuleToChain( m_rule, chain_target );
-		m_err_handler->showError( m_err );
-		kmfdoc->changed();
-		kmfdoc->endTransaction();
-		emit sigUpdateView();
+		KMFUndoEngine::instance()->startTransaction( 
+			m_rule->chain()->table(),
+			i18n( "Move Rule: %1 from Chain: %2 to Chain: %3" ).arg( m_rule->name() ).arg( m_rule->chain()->name() ).arg( chain_target->name() ) 
+		);
+		m_err = m_rule->chain()->table()->moveRuleToChain( m_rule, chain_target );
+		if ( m_err_handler->showError( m_err ) ) {
+			KMFUndoEngine::instance()->endTransaction();
+			emit sigUpdateView( chain_target->table() );
+		} else {
+			KMFUndoEngine::instance()->abortTransaction();
+		}
 	}
 }
 
@@ -1139,23 +1253,26 @@ void KMFRuleEdit::slotCopyRule( int index ) {
 	kdDebug() << "void KMFRuleEdit::slotCopyRule( int )" << endl;
 	kdDebug() << "Move Rule  to Chain Nr: " << index << endl;
 	IPTChain *chain_target = 0;
-	chain_target = kmfdoc->table( m_table->name() ) ->chains().at( index );
+	chain_target = m_network->currentDocAsIPTDoc()->table( m_table->name() ) ->chains().at( index );
 	if ( chain_target ) {
-		kmfdoc->startTransaction();
-		m_rule->chain()->table()->saveState();
-		chain_target->saveState();
+		KMFUndoEngine::instance()->startTransaction( 
+			m_rule->chain() ->table(),
+			i18n( "Copy Rule: %1 from Chain: %2 to Chain: %3" ).arg( m_rule->name() ).arg( m_rule->chain()->name() ).arg( chain_target->name() ) 
+		);
 		m_err = m_rule->chain() ->table() ->copyRuleToChain( m_rule, chain_target );
-		m_err_handler->showError( m_err );
-		kmfdoc->changed();
-		kmfdoc->endTransaction();
-		emit sigUpdateView();
+		if ( m_err_handler->showError( m_err ) ) {
+			KMFUndoEngine::instance()->endTransaction();
+			emit sigUpdateView( chain_target->table() );
+		} else {
+			KMFUndoEngine::instance()->abortTransaction();
+		}
 	}
 }
 
 void KMFRuleEdit::slotRenameRule() {
 	if ( ! m_rule )
 		return;
-	if ( KMFListViewItem* rule_item = currTableView()->findKMFItem( m_rule->name(),2,m_rule->objectID() ) ) {
+	if ( KMFListViewItem* rule_item = currTableView()->findKMFItem( m_rule->name(),2,m_rule->uuid() ) ) {
 
 		rule_item->startRename(2);
 	}
@@ -1168,56 +1285,94 @@ void KMFRuleEdit::slotRenameRule( QListViewItem* , int, const QString& name ) {
 		return;
 	}
 
-	 if ( ! m_rule ) {
-		kdDebug() << "Rename rejected: m_rule == 0:" << endl;
-		return;
-	}
+// 	 if ( ! m_rule ) {
+// 		kdDebug() << "Rename rejected: m_rule == 0:" << endl;
+// 		return;
+// 	}
 
-	if ( KMFListViewItem* rule_item = currTableView()->findKMFItem( name ,2,m_rule->objectID() ) ) {
-		kdDebug() << "Found Rule Item: " << name <<  " ID " << m_rule->objectID() << endl;
+	if ( KMFListViewItem* rule_item = currTableView()->findKMFItem( name ,2,m_rule->uuid() ) ) {
+		kdDebug() << "Found Rule Item: " << name <<  " ID " << m_rule->uuid() << endl;
 		QPtrList<IPTRule>& rules = rule_item->rule()->chain()->chainRuleset();
 		IPTRule *rule = 0;
 		for ( rule = rules.first(); rule; rule = rules.next() ) {
 			if ( rule->name() == name ) {
 				KMessageBox::sorry( this, i18n( "<qt><p>Sorry, there is already a rule named: <b>%1</b> in the chain.<br>"
 												"Please make sure that the new rule name is unique in its chain.</qt>" ).arg( name ) );
+				rule_item->setText( 2, rule_item->rule()->name() );
+				emit sigUpdateView( rule_item->rule()->chain() );
 				return ;
 			}
 		}
-		kmfdoc->startTransaction();
-		rule_item->rule()->saveState();
-		m_err = rule_item->rule()->setName( name );
+		KMFUndoEngine::instance()->startTransaction( 
+			rule_item->rule(),
+			i18n( "Rename Rule: %1 to %2").arg( rule_item->rule()->name() ).arg( name ) 
+		);
+		
+		QString origName = rule_item->rule()->name();
+		m_err = rule_item->rule()->setRuleName( name );
 		if ( m_err_handler->showError( m_err ) ) {
-			kmfdoc->changed();
-			emit sigUpdateView( rule_item->rule()->chain() );
+			KMFUndoEngine::instance()->endTransaction();
+		} else {
+			rule_item->setText( 2, origName );
+			KMFUndoEngine::instance()->abortTransaction();
 		}
-		kmfdoc->endTransaction();
+		
+		emit sigUpdateView( rule_item->rule()->chain()->table() );
 	}
 }
 
 void KMFRuleEdit::slotUpdateView() {
 	kdDebug() << "void KMFRuleEdit::slotUpdateView()" << endl;
-	kdDebug() << "upAndRunning(): " << KMFAppState::upAndRunning() << endl;
-	kdDebug() << "hasOpenDoc()" << KMFAppState::hasOpenDoc() <<  endl;
-	if ( ! KMFAppState::upAndRunning() || ! KMFAppState::hasOpenDoc() ) {
-		m_lv_table_filter->clearAllItems();
-		m_lv_table_nat->clearAllItems();
-		m_lv_table_mangle->clearAllItems();
-		m_lv_table_filter->setEnabled( false );
-		m_lv_table_nat->setEnabled( false );
-		m_lv_table_mangle->setEnabled( false );
-		slotSelectionInvalid();
-		setEnabled( false );
+	if ( ! m_network ) {
+		kdDebug() << "ERROR: m_network is null" << endl;
 		return;
 	}
+	
+	kdDebug() << "upAndRunning(): " << KMFAppState::upAndRunning() << endl;
+		slotSelectionInvalid();
+	
+	bool reload = true;
+	if ( ! m_lastDisplayDoc ) {
+		m_lastDisplayDoc =  m_network->currentDocAsIPTDoc();
+	}
+	if ( m_lastDisplayDoc ==  m_network->currentDocAsIPTDoc() ) {
+		if ( ! m_network->newSavePathNeeded() ) {
+			kdDebug() << "IPTDoc still the same." << endl;
+			reload = false;
+		}
+	} else {
+		kdDebug() << "IPTDoc changed." << endl;
+		m_lastDisplayDoc = m_network->currentDocAsIPTDoc();
+		// reload = true;
+	}
+	
+	IPTable *filter = 0;
+	filter = m_network->currentDocAsIPTDoc()->table( Constants::FilterTable_Name );
+	if ( filter ) {
+		if ( reload ) m_lv_table_filter->clearAllItems();
+		m_lv_table_filter->slotLoadNode( filter );
+	}
+
+	IPTable *nat = 0;
+	nat = m_network->currentDocAsIPTDoc()->table( Constants::NatTable_Name );
+	if ( nat ) {
+		if ( reload ) m_lv_table_nat->clearAllItems();
+		m_lv_table_nat->slotLoadNode( nat );
+	}
+	
+	IPTable *mangle = 0;
+	mangle = m_network->currentDocAsIPTDoc()->table( Constants::MangleTable_Name );
+	if ( mangle ) {
+		if ( reload ) m_lv_table_mangle->clearAllItems();
+		m_lv_table_mangle->slotLoadNode( mangle );
+	}
+
+	rb_filter->setEnabled( m_network->currentDocAsIPTDoc()->useFilter() );
+	rb_mangle->setEnabled( m_network->currentDocAsIPTDoc()->useMangle() );
+	rb_nat->setEnabled( m_network->currentDocAsIPTDoc()->useNat() );
 
 
-	rb_filter->setEnabled( kmfdoc->useFilter() );
-	rb_mangle->setEnabled( kmfdoc->useMangle() );
-	rb_nat->setEnabled( kmfdoc->useNat() );
-
-
-	if ( kmfdoc->useModules() ) {
+	if ( m_network->currentDocAsIPTDoc()->useModules() ) {
 		m_led_modules->setColor( green );
 		m_led_modules->on();
 	} else {
@@ -1225,7 +1380,7 @@ void KMFRuleEdit::slotUpdateView() {
 		m_led_modules->off();
 	}
 
-	if ( kmfdoc->useIPFwd() ) {
+	if ( m_network->currentDocAsIPTDoc()->useIPFwd() ) {
 		m_led_fwd->setColor( green );
 		m_led_fwd->on();
 	} else {
@@ -1233,7 +1388,7 @@ void KMFRuleEdit::slotUpdateView() {
 		m_led_fwd->off();
 	}
 
-	if ( kmfdoc->useSynCookies() ) {
+	if ( m_network->currentDocAsIPTDoc()->useSynCookies() ) {
 		m_led_syn->setColor( green );
 		m_led_syn->on();
 	} else {
@@ -1241,7 +1396,7 @@ void KMFRuleEdit::slotUpdateView() {
 		m_led_syn->off();
 	}
 
-	if ( kmfdoc->useRPFilter() ) {
+	if ( m_network->currentDocAsIPTDoc()->useRPFilter() ) {
 		m_led_rp->setColor( green );
 		m_led_rp->on();
 	} else {
@@ -1249,7 +1404,7 @@ void KMFRuleEdit::slotUpdateView() {
 		m_led_rp->off();
 	}
 
-	if ( kmfdoc->useMartians() ) {
+	if ( m_network->currentDocAsIPTDoc()->useMartians() ) {
 		m_led_martians->setColor( green );
 		m_led_martians->on();
 	} else {
@@ -1271,6 +1426,7 @@ void KMFRuleEdit::slotUpdateView() {
 	}
 
 	emit sigUpdateView();
+	m_app->updateCaption();
 // 	slotEditRule();
 }
 
@@ -1355,4 +1511,7 @@ void KMFRuleEdit::loadIcons() {
 	icon_move = loader->loadIcon( icon_name, KIcon::Small );
 
 }
+
+}
+
 #include "kmfruleedit.moc"
