@@ -4,7 +4,7 @@
 // Description:
 //
 //
-// Author: Christian Hubinger <chubinger@gmail.com>, (C) 2004
+// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2004
 //
 // Copyright: See COPYING file that comes with this distribution
 // License: GPL
@@ -21,7 +21,7 @@
 #include <kdemacros.h>
 
 // project includes
-
+namespace KMF {
 class KMFDoc;
 
 class KDE_EXPORT KMFDocumentInfo : public KMyFirewallDocumentInfo {
@@ -53,6 +53,6 @@ signals:
    void sigSaveAsTemplate();
 
 };
-
+}
 #endif
 
