@@ -7,7 +7,7 @@
  *                                                                         *
  ***************************************************************************/
 /*
-Author: Christian Hubinger <chubinger@gmail.com>, (C) 2001-2004
+Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2001-2004
 */
 
 
@@ -35,9 +35,10 @@ Author: Christian Hubinger <chubinger@gmail.com>, (C) 2001-2004
 #include "../core/iptruleoption.h"
 #include "../core/kmfconfig.h"
 #include "../core/kmfappstate.h"
+#include "../core/xmlnames.h"
 #include "../kmyfirewall.h"
 #include "kmflistviewitem.h"
-
+namespace KMF {
 KMFListView::KMFListView( QWidget *parent, const char *name ) : KListView( parent, name ) {
 	loadIcons();
 	connect( this, SIGNAL( sigLoadNode( NetfilterObject* ) ),
@@ -71,12 +72,12 @@ KMFListView::~KMFListView() {}
 
 void KMFListView::slotLoadNode( NetfilterObject* node ) {
 	kdDebug() << "void KMFListView::loadNode( NetfilterObject* node )" << endl;
-	if ( ! node )
+	if ( ! node ) {
+		kdDebug() << "node == 0" << endl;
 		return;
+	}
 	m_NetfilterObject = node;
 
-
-
 	if ( m_NetfilterObject->type() == NetfilterObject::TABLE ) {
 		if ( IPTable * table = dynamic_cast<IPTable*> ( node ) ) {
 			bool changed = false;
@@ -119,19 +120,22 @@ void KMFListView::slotLoadNode( NetfilterObject* node ) {
 }
 
 void KMFListView::slotUpdateView() {
-	if ( ! m_NetfilterObject )
+	kdDebug() << "KMFListView::slotUpdateView()." << endl;
+	if ( ! m_NetfilterObject ) {
+		kdDebug() << "m_NetfilterObject = 0" << endl;
 		return ;
+	}
 	if ( ! KMFAppState::upAndRunning() ) {
 		kdDebug() << "Doing Nothing during startup." << endl;
 		this->clearAllItems();
 		return;
 	}
-	if ( ! KMFAppState::hasOpenDoc() ) {
+/*	if ( ! KMFAppState::hasOpenDoc() ) {
 		kdDebug() << "No Open doc. disabling view!" << endl;
 		this->clearAllItems();
 		setEnabled( false );
 		return;
-	}
+	}*/
 	setEnabled( true );
 	emit sigLoadNode( m_NetfilterObject );
 }
@@ -146,32 +150,32 @@ void KMFListView::slotUpdateView( NetfilterObject* obj ) {
 	}
 	if ( obj->type() == NetfilterObject::TABLE ) {
 		if ( ! m_NetfilterObject )
-		return ;
+			return;
 		emit sigLoadNode( m_NetfilterObject );
 	} else if ( obj->type() == NetfilterObject::CHAIN ) {
 		if ( IPTChain* chain = dynamic_cast<IPTChain*>( obj ) ) {
-			if ( KMFListViewItem* item = findKMFItem( chain->name(), 2, chain->objectID()) ) {
+			if ( KMFListViewItem* item = findKMFItem( chain->name(), 2, chain->uuid() ) ) {
 				if ( KMFListViewItem* parent = dynamic_cast<KMFListViewItem*>( item->parent() ) ) {
 					emit sigLoadNode( m_NetfilterObject );
-// 					kdDebug() << "\nStarting update for Chain Item..." << endl;
+// 					kdDebug() << "Starting update for Chain Item..." << endl;
 					setUpdatesEnabled( false );
 					setupChainView( chain, parent );
 					setUpdatesEnabled( true );
 					triggerUpdate();
-// 					kdDebug() << "Finished update for Chain Item.\n" << endl;
+// 					kdDebug() << "Finished update for Chain Item." << endl;
 				}
 			}
 		}
 	} else if ( obj->type() == NetfilterObject::RULE  ) {
 		if ( IPTRule* rule = dynamic_cast<IPTRule*>( obj ) ) {
-			if ( KMFListViewItem* item = findKMFItem( rule->name(), 2, rule->objectID()) ) {
+			if ( KMFListViewItem* item = findKMFItem( rule->name(), 2, rule->uuid()) ) {
 				if ( KMFListViewItem* parent = dynamic_cast<KMFListViewItem*>( item->parent() ) ) {
-// 					kdDebug() << "\nStarting update for Rule Item..." << endl;
+// 					kdDebug() << "Starting update for Rule Item..." << endl;
 					setUpdatesEnabled( false );
 					setupRuleView( rule, parent );
 					setUpdatesEnabled( true );
 					triggerUpdate();
-// 					kdDebug() << "Finished update for Rule Item.\n" << endl;
+// 					kdDebug() << "Finished update for Rule Item." << endl;
 				}
 			}
 		}
@@ -214,11 +218,12 @@ void KMFListView::setupTableView( IPTable* table ) {
 	if ( ! table )
 		return ;
 	KMFListViewItem* item = 0;
-	item = findKMFItem( i18n( "Table:" ), 0, table->objectID() );
+	item = findKMFItem( table->name(), 2, table->uuid() );
 	if ( ! item ) {
 		item = new KMFListViewItem( this, 0, table );
 		item->setOpen( true );
 	}
+		
 	item->sortChildItems( 0, true );
 	item->setTopItem( true );
 	item->setInUse( true );
@@ -235,13 +240,13 @@ void KMFListView::setupTableView( IPTable* table ) {
 		if ( chain ) {
 			used_chains.append ( chain );
 			QString str_obj_id = "";
-			str_obj_id =  str_obj_id.setNum( chain->objectID() );
+			str_obj_id =  chain->uuid().toString();
 
 			if ( m_existed_chains.findIndex ( chain->name() + "|" + str_obj_id ) == -1 )
 				m_existed_chains.append ( chain->name() + "|" + str_obj_id );
 
-			if ( ! m_dict_existed_rules.find( chain->objectID() ) ) {
-				m_dict_existed_rules.insert( chain->objectID(), new QStringList );
+			if ( ! m_dict_existed_rules.find( chain->uuid() ) ) {
+				m_dict_existed_rules.insert( chain->uuid().toString(), new QStringList );
 			}
 			setupChainView( chain, item );
 		}
@@ -252,9 +257,10 @@ void KMFListView::setupTableView( IPTable* table ) {
 			int delimiter = existed_chain_name_id.find( "|" );
 			QString existed_chain_name = existed_chain_name_id.left( delimiter );
 			QString existed_chain_id_str = existed_chain_name_id.right( existed_chain_name_id.length() - delimiter - 1 );
-			int existed_obj_id = -1;
+/*			QUuid existed_obj_id = -1;
 			bool ok;
-			existed_obj_id = existed_chain_id_str.toInt( &ok );
+			existed_obj_id = existed_chain_id_str.toInt( &ok );*/
+			QUuid existed_obj_id( existed_chain_id_str );
 // 			kdDebug() << "Found Existed Name: " << existed_chain_name << " ID: " << existed_obj_id << endl;
 			bool have = false;
 			QPtrListIterator<IPTChain> it2( used_chains );
@@ -284,14 +290,14 @@ void KMFListView::setupTableView( IPTable* table ) {
 	while ( it3.current() ) {
 		IPTChain * chain = it3.current();
 		++it3;
-		QString str_obj_id = "";
-		str_obj_id =  str_obj_id.setNum( chain->objectID() );
+		QString str_obj_id = chain->uuid().toString();
+		// str_obj_id =  str_obj_id.setNum( chain->uuid() );
 
 		m_existed_chains.append( chain->name() + "|" +  str_obj_id );
 	}
 
 	KMFListViewItem * item_desc = 0;
-	item_desc = findKMFItem( i18n( "Documentation:" ), 0, table->objectID() );
+	item_desc = findKMFItem( i18n( "Documentation:" ), 0, table->uuid() );
 
 	if ( !table->description().isEmpty() && m_show_desc ) {
 		if ( ! item_desc ) {
@@ -310,7 +316,7 @@ void KMFListView::setupTableView( IPTable* table ) {
 
 	
 	KMFListViewItem* item_obj_id = 0;
-	item_obj_id = findKMFItem( i18n( "Object ID:" ), 0, table->objectID(),true,item );
+	item_obj_id = findKMFItem( i18n( "Object ID:" ), 0, table->uuid(),true,item );
 	
 	if  ( KMFConfig::showObjectID() ) {
 		if ( ! item_obj_id ) {
@@ -318,8 +324,7 @@ void KMFListView::setupTableView( IPTable* table ) {
 		}
 		item_obj_id->setInUse( true );
 		item_obj_id->setText( 0, i18n( "Object ID:" ) );
-		QString sid = "";
-		item_obj_id->setText( 2, sid.setNum( table->objectID() ) );
+		item_obj_id->setText( 2, table->uuid().toString() );
 	} else if ( item_obj_id ) {
 		delete item_obj_id;
 	}
@@ -333,7 +338,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 		return ;
 
 	KMFListViewItem * item = 0;
-	item = findKMFItem( chain->name(), 2, chain->objectID(), true, parent );
+	item = findKMFItem( chain->name(), 2, chain->uuid(), true, parent );
 	if ( ! item ) {
 // 		kdDebug() << "No Item for Chain: " << chain->name() << " found.\nNeed to create a new one." << endl;
 		item = new KMFListViewItem( parent, 0, chain );
@@ -359,7 +364,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 
 
 	KMFListViewItem * i_chain_cmd = 0;
-	i_chain_cmd = findKMFItem( i18n( "Cmd:" ), 0, chain->objectID(), true, item );
+	i_chain_cmd = findKMFItem( i18n( "Cmd:" ), 0, chain->uuid(), true, item );
 
 	if ( ! chain->isBuildIn() && m_show_cmds ) {
 		if ( ! i_chain_cmd ) {
@@ -379,7 +384,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 	//	kdDebug() << "Setting up item: i_chain_log" << endl;
 	bool logging = chain->logging();
 	KMFListViewItem * i_chain_log = 0;
-	i_chain_log = findKMFItem( i18n( "Chain logging:" ), 0, chain->objectID(), true, item );
+	i_chain_log = findKMFItem( i18n( "Chain logging:" ), 0, chain->uuid(), true, item );
 
 	if ( ! logging && i_chain_log ) {
 		delete i_chain_log;
@@ -399,13 +404,13 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 		QString prefix = chain->logPrefix();
 		QString burst = chain->logBurst();
 		KMFListViewItem * i_limit = 0;
-		i_limit = findKMFItem( i18n( "Limit rate:" ), 0, chain->objectID(), true, i_chain_log );
+		i_limit = findKMFItem( i18n( "Limit rate:" ), 0, chain->uuid(), true, i_chain_log );
 
 
-		if ( limit.isEmpty() && i_limit ) {
+		if ( ( limit.isEmpty() || limit == XML::Undefined_Value ) && i_limit ) {
 			delete i_limit;
 		} else
-			if ( ! limit.isEmpty() ) {
+			if ( ! limit.isEmpty() && limit != XML::Undefined_Value ) {
 				if ( ! i_limit ) {
 					i_limit = new KMFListViewItem( i_chain_log, 0, chain );
 					i_limit->setOpen( true );
@@ -416,13 +421,13 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 				i_limit->setText( 2, limit );
 
 				KMFListViewItem * i_burst = 0;
-				i_burst = findKMFItem( i18n( "Limit burst:" ), 0, chain->objectID(), true, i_chain_log );
+				i_burst = findKMFItem( i18n( "Limit burst:" ), 0, chain->uuid(), true, i_chain_log );
 
-				if ( burst.isEmpty() && i_burst ) {
+				if ( ( burst.isEmpty() || burst == XML::Undefined_Value ) && i_burst ) {
 					delete i_burst;
 				}
 
-				if ( ! burst.isEmpty() ) {
+				if ( ! burst.isEmpty() && burst != XML::Undefined_Value ) {
 					//			 kdDebug() << "Setting up item: i_burst" << endl;
 					if ( ! i_burst ) {
 						i_burst = new KMFListViewItem( i_chain_log, i_limit, chain );
@@ -435,14 +440,13 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 			}
 
 		KMFListViewItem* i_prefix = 0;
-		i_prefix = findKMFItem( i18n( "Log prefix:" ), 0, chain->objectID(), true, i_chain_log );
+		i_prefix = findKMFItem( i18n( "Log prefix:" ), 0, chain->uuid(), true, i_chain_log );
 
-		if ( prefix.isEmpty() && i_prefix ) {
-// 			i_prefix->setInUse( false );
+		if ( ( prefix.isEmpty() || prefix == XML::Undefined_Value ) && i_prefix ) {
 			delete i_prefix;
 		}
 
-		if ( ! prefix.isEmpty() ) {
+		if ( ! prefix.isEmpty()  && prefix != XML::Undefined_Value ) {
 			// 			kdDebug() << "Setting up item: i_prefix" << endl;
 			if ( ! i_prefix ) {
 				i_prefix = new KMFListViewItem( i_chain_log, 0, chain );
@@ -461,7 +465,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 	bool buildin = chain->isBuildIn();
 	//	kdDebug() << "Setting up item: i_chain_feeds" << endl;
 	KMFListViewItem * i_chain_feeds = 0;
-	i_chain_feeds = findKMFItem( i18n( "Chain Feeds:" ), 0, chain->objectID(), true, item );
+	i_chain_feeds = findKMFItem( i18n( "Chain Feeds:" ), 0, chain->uuid(), true, item );
 	if ( ! i_chain_feeds ) {
 		if ( logging )
 			i_chain_feeds = new KMFListViewItem( item, i_chain_log, chain );
@@ -500,7 +504,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 
 	//	kdDebug() << "Setting up item: i_chain_fwds" << endl;
 	KMFListViewItem * i_chain_fwds = 0;
-	i_chain_fwds = findKMFItem( i18n( "Chain forwards:" ), 0, chain->objectID(), true, item );
+	i_chain_fwds = findKMFItem( i18n( "Chain forwards:" ), 0, chain->uuid(), true, item );
 	if ( ! i_chain_fwds ) {
 		i_chain_fwds = new KMFListViewItem( item, i_chain_feeds, chain );
 		i_chain_fwds->setOpen( true );
@@ -536,7 +540,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 
 	//	kdDebug() << "Setting up item: i_chain_allrules" << endl;
 	KMFListViewItem* i_chain_allrules = 0;
-	i_chain_allrules = findKMFItem( i18n( "Rule(s):" ), 0, chain->objectID(), true, item );
+	i_chain_allrules = findKMFItem( i18n( "Rule(s):" ), 0, chain->uuid(), true, item );
 	if ( ! i_chain_allrules ) {
 		i_chain_allrules = new KMFListViewItem( item, i_chain_fwds, chain );
 		i_chain_allrules->setOpen( TRUE );
@@ -551,7 +555,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 	i_chain_allrules->setText( 2, num_rules );
 
 	QPtrList<IPTRule> used_rules;
-	QStringList *existed_rules = m_dict_existed_rules.find( chain->objectID() );
+	QStringList *existed_rules = m_dict_existed_rules.find( chain->uuid() );
 
 	if ( existed_rules ) {
 		if ( !tmp_ruleset.isEmpty() ) {
@@ -559,10 +563,8 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 			for ( curr_rule = tmp_ruleset.last(); curr_rule; curr_rule = tmp_ruleset.prev() ) {
 // 				kdDebug() << "Entering: for ( curr_rule = tmp_ruleset.last(); curr_rule; curr_rule = tmp_ruleset.prev() )" << endl;
 				used_rules.append( curr_rule );
-				QString str_obj_id = "";
-				str_obj_id =  str_obj_id.setNum( curr_rule->objectID() );
-				if ( existed_rules->findIndex ( curr_rule->name() + "|" + str_obj_id ) == -1 ) {
-					existed_rules->append( curr_rule->name() + "|" + str_obj_id );
+				if ( existed_rules->findIndex ( curr_rule->name() + "|" + curr_rule->uuid().toString() ) == -1 ) {
+					existed_rules->append( curr_rule->name() + "|" + curr_rule->uuid().toString() );
 				}
 				setupRuleView( curr_rule , i_chain_allrules );
 			}
@@ -573,9 +575,9 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 				int delimiter = existed_rule_name_id.find( "|" );
 				QString existed_rule_name = existed_rule_name_id.left( delimiter );
 				QString existed_rule_id_str = existed_rule_name_id.right( existed_rule_name_id.length() - delimiter - 1 );
-				int existed_obj_id = -1;
-				bool ok;
-				existed_obj_id = existed_rule_id_str.toInt( &ok );
+				QUuid existed_obj_id( existed_rule_id_str );
+/*				bool ok;
+				existed_obj_id = existed_rule_id_str.toInt( &ok );*/
 // 				kdDebug() << "Found Existed Name: " << existed_rule_name << " ID: " << existed_obj_id << endl;
 				bool have = false;
 				QPtrListIterator<IPTRule> it2( used_rules );
@@ -584,8 +586,8 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 					used_rule = it2.current();
 // 					kdDebug() << "Entering: while ( used_rule )" << endl;
 					++it2;
-					if ( existed_rule_name == used_rule->name() ) {
-// 						kdDebug() << "Rule: " << used_rule->name() << " already existed." << endl;
+					if ( /*existed_rule_name == used_rule->name() &&*/ existed_obj_id == used_rule->uuid() ) {
+ 						kdDebug() << "Rule: " << used_rule->name() << " already existed." << endl;
 						have = true;
 					}
 				}
@@ -611,8 +613,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 		while ( it3.current() ) {
 			IPTRule * rule = it3.current();
 			++it3;
-			QString str_obj_id = "";
-			existed_rules->append( rule->name() + "|" + str_obj_id.setNum( rule->objectID() ) );
+			existed_rules->append( rule->name() + "|" + rule->uuid().toString() );
 		}
 	}
 	//########### End Rules ####################
@@ -620,7 +621,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 	// default target ++++++++++++++++++++++++++++++++++++
 
 	KMFListViewItem * i_deftg = 0;
-	i_deftg = findKMFItem( i18n( "Target:" ), 0, chain->objectID(), true, item );
+	i_deftg = findKMFItem( i18n( "Target:" ), 0, chain->uuid(), true, item );
 	if ( ! i_deftg ) {
 		i_deftg = new KMFListViewItem( item, i_chain_allrules, chain );
 	}
@@ -645,7 +646,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 
 		//		kdDebug() << "Setting up item: i_deftg_rule" << endl;
 		KMFListViewItem * i_deftg_rule = 0;
-		i_deftg_rule = findKMFItem( i18n( "Policy Cmd:" ), 0, chain->objectID(), true, i_deftg );
+		i_deftg_rule = findKMFItem( i18n( "Policy Cmd:" ), 0, chain->uuid(), true, i_deftg );
 
 		if ( m_show_cmds ) {
 			if ( ! i_deftg_rule ) {
@@ -663,7 +664,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 	}
 
 	KMFListViewItem * item_desc = 0;
-	item_desc = findKMFItem( i18n( "Documentation:" ), 0, chain->objectID(), true, item  );
+	item_desc = findKMFItem( i18n( "Documentation:" ), 0, chain->uuid(), true, item  );
 
 	if ( !chain->description().isEmpty() && ! chain->isBuildIn() ) {
 		if ( ! item_desc ) {
@@ -689,7 +690,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 	}
 	
 	KMFListViewItem* item_obj_id = 0;
-	item_obj_id = findKMFItem( i18n( "Object ID:" ), 0, chain->objectID(),true,item );
+	item_obj_id = findKMFItem( i18n( "Object ID:" ), 0, chain->uuid(),true,item );
 	
 	if  ( KMFConfig::showObjectID() ) {
 		if ( ! item_obj_id ) {
@@ -698,7 +699,7 @@ void KMFListView::setupChainView( IPTChain* chain, KMFListViewItem* parent ) {
 		item_obj_id->setInUse( true );
 		item_obj_id->setText( 0, i18n( "Object ID:" ) );
 		QString sid = "";
-		item_obj_id->setText( 2, sid.setNum( chain->objectID() ) );
+		item_obj_id->setText( 2, chain->uuid().toString() );
 	} else if ( item_obj_id ) {
 		delete item_obj_id;
 	}	
@@ -712,9 +713,9 @@ void KMFListView::setupRuleView( IPTRule* rule, KMFListViewItem* parent ) {
 		return ;
 
 	KMFListViewItem * item = 0;
-	item = findKMFItem( rule->name(), 2, rule->objectID(), true, parent );
+	item = findKMFItem( rule->name(), 2, rule->uuid(), true, parent );
 	if ( ! item ) {
-// 		kdDebug() << "No Item for Rule: " << rule->name() << " found.\nNeed to create a new one." << endl;
+//		kdDebug() << "No Item for Rule: " << rule->name() << " found.\nNeed to create a new one." << endl;
 		item = new KMFListViewItem( parent, parent, rule );
 	}
 	item->sortChildItems( 0, true );
@@ -752,7 +753,7 @@ void KMFListView::setupRuleView( IPTRule* rule, KMFListViewItem* parent ) {
 
 	//	kdDebug() << "There are options defined for the rule " << endl;
 	KMFListViewItem* item_target = 0;
-	item_target = findKMFItem( i18n( "Target:" ), 0, rule->objectID(), true, item );
+	item_target = findKMFItem( i18n( "Target:" ), 0, rule->uuid(), true, item );
 	if ( !item_target ) {
 		item_target = new KMFListViewItem( item, 0, rule );
 	}
@@ -775,7 +776,7 @@ void KMFListView::setupRuleView( IPTRule* rule, KMFListViewItem* parent ) {
 	}
 
 	KMFListViewItem* item_cmd = 0;
-	item_cmd = findKMFItem( i18n( "Cmd:" ), 0, rule->objectID(), true, item );
+	item_cmd = findKMFItem( i18n( "Cmd:" ), 0, rule->uuid(), true, item );
 
 	if ( m_show_cmds ) {
 		if ( ! item_cmd ) {
@@ -815,11 +816,11 @@ void KMFListView::setupRuleView( IPTRule* rule, KMFListViewItem* parent ) {
 		} else {
 
 			KMFListViewItem * kmfitem = 0;
-			kmfitem = findKMFItem( opt->guiName(), 0, opt->objectID(), true, item );
+			kmfitem = findKMFItem( opt->guiName(), 0, opt->uuid(), true, item );
 			if ( kmfitem ) {
 				delete kmfitem;
 			} else {
-				kmfitem = findKMFItem( opt->guiName(), 0, opt->objectID(), true, item_target );
+				kmfitem = findKMFItem( opt->guiName(), 0, opt->uuid(), true, item_target );
 				if ( kmfitem ) {
 					delete kmfitem;
 				}
@@ -831,7 +832,7 @@ void KMFListView::setupRuleView( IPTRule* rule, KMFListViewItem* parent ) {
 	//############## End Rule Options ############
 
 	KMFListViewItem * item_log = 0;
-	item_log = findKMFItem( i18n( "Logging:" ), 0, rule->objectID(),true,item );
+	item_log = findKMFItem( i18n( "Logging:" ), 0, rule->uuid(),true,item );
 	if ( rule->logging() && ! rule->enabled() ) {
 		if ( ! item_log ) {
 			item_log = new KMFListViewItem( item, 0, rule );
@@ -846,7 +847,7 @@ void KMFListView::setupRuleView( IPTRule* rule, KMFListViewItem* parent ) {
 
 
 	KMFListViewItem * item_desc = 0;
-	item_desc = findKMFItem( i18n( "Documentation:" ), 0, rule->objectID(),true,item );
+	item_desc = findKMFItem( i18n( "Documentation:" ), 0, rule->uuid(),true,item );
 
 	if ( !rule->description().isEmpty() ) {
 		if ( ! item_desc ) {
@@ -864,7 +865,7 @@ void KMFListView::setupRuleView( IPTRule* rule, KMFListViewItem* parent ) {
 	}
 	
 	KMFListViewItem* item_obj_id = 0;
-	item_obj_id = findKMFItem( i18n( "Object ID:" ), 0, rule->objectID(),true,item );
+	item_obj_id = findKMFItem( i18n( "Object ID:" ), 0, rule->uuid(),true,item );
 
 	if  ( KMFConfig::showObjectID() ) {
 		if ( ! item_obj_id ) {
@@ -873,7 +874,7 @@ void KMFListView::setupRuleView( IPTRule* rule, KMFListViewItem* parent ) {
 		item_obj_id->setInUse( true );
 		item_obj_id->setText( 0, i18n( "Object ID:" ) );
 		QString sid = "";
-		item_obj_id->setText( 2, sid.setNum( rule->objectID() ) );
+		item_obj_id->setText( 2, rule->uuid().toString() );
 	} else if ( item_obj_id ) {
 		delete item_obj_id;
 	}
@@ -892,7 +893,7 @@ void KMFListView::setupRuleOptionView( IPTRuleOption* option, KMFListViewItem* p
 	QString name = "";
 	
 	KMFListViewItem * item = 0;
-	item = findKMFItem( option->guiName(), 0, option->objectID(),true,parent );
+	item = findKMFItem( option->guiName(), 0, option->uuid(),true,parent );
 	
 	if ( values.count() < 1 || option->isEmpty() ) {
 		if ( item ) {
@@ -921,15 +922,15 @@ void KMFListView::setupRuleOptionView( IPTRuleOption* option, KMFListViewItem* p
 
 
 		if ( gui_strings->count() > i /* + 1 */ ) {
-			// 			if (val == "bool:on")
+			// 			if (val == XML::BoolOn_Value)
 			// 				val = "";
 
 			QString gui_name = "";
 			gui_name = *gui_strings->at( i  /* + 1 */ );
 			KMFListViewItem * item_opt = 0;
-			item_opt = findKMFItem( gui_name, 0, option->objectID(),true,item );
+			item_opt = findKMFItem( gui_name, 0, option->uuid(),true,item );
 
-			if ( gui_name.isEmpty() || val == "UNDEFINED" || val.simplifyWhiteSpace().isEmpty() || val == "bool:off" ) {
+			if ( gui_name.isEmpty() || val == XML::Undefined_Value || val.simplifyWhiteSpace().isEmpty() || val == XML::BoolOff_Value ) {
 				if ( item_opt ) {
 					delete item_opt;
 				}
@@ -939,7 +940,7 @@ void KMFListView::setupRuleOptionView( IPTRuleOption* option, KMFListViewItem* p
 					item_opt = new KMFListViewItem( item, 0, option );
 					item_opt->setOpen( true );
 				}
-				if ( val != "bool:on" ) {
+				if ( val != XML::BoolOn_Value ) {
 					item_opt->setInUse( true );
 					item_opt->setText( 0, gui_name );
 					item_opt->setText( 2, val );
@@ -952,15 +953,14 @@ void KMFListView::setupRuleOptionView( IPTRuleOption* option, KMFListViewItem* p
 	}
 	
 	KMFListViewItem *item_obj_id = 0;
-	item_obj_id = findKMFItem( i18n( "Object ID:" ), 0, option->objectID(),true,item );
+	item_obj_id = findKMFItem( i18n( "Object ID:" ), 0, option->uuid(),true,item );
 	if  ( KMFConfig::showObjectID() ) {
 		if ( ! item_obj_id ) {
 			item_obj_id = new KMFListViewItem( item, 0, option );
 		}
 		item_obj_id->setInUse( true );
 		item_obj_id->setText( 0, i18n( "Object ID:" ) );
-		QString sid = "";
-		item_obj_id->setText( 2, sid.setNum( option->objectID() ) );
+		item_obj_id->setText( 2, option->uuid().toString() );
 	} else if ( item_obj_id ) {
 		delete item_obj_id;
 	}
@@ -973,9 +973,9 @@ void KMFListView::clearAllItems() {
 	clear();
 }
 
-KMFListViewItem* KMFListView::findKMFItem( const QString & text, int column, int obj_id, bool exact /*= true*/, KMFListViewItem* from_item /*= 0*/  ) {
+KMFListViewItem* KMFListView::findKMFItem( const QString & text, int column, const QUuid& obj_id, bool exact /*= true*/, KMFListViewItem* from_item /*= 0*/  ) {
 	// 	kdDebug() << "KMFListViewItem* KMFListView::findKMFItem(const QString& text: " << text << ", int column:" << column << ", int obj_id: " << obj_id << ")" << endl;
-	if ( text.isNull() || column < 0 || obj_id < -1 )
+	if ( text.isNull() || column < 0 )
 		return 0;
 	if ( from_item ) {
 // 		from_item->setText(1,"Serching this item");
@@ -988,12 +988,18 @@ KMFListViewItem* KMFListView::findKMFItem( const QString & text, int column, int
 			item = dynamic_cast<KMFListViewItem*> ( qitem );
 			if ( item ) {
 // 				item->setPixmap(1,icon_accept);
-				if ( exact ) {
-					if ( item->text( column ) == text && item->objectID() == obj_id )
+				if ( obj_id.isNull() ) {
+					if ( item->text( column ) == text ) {
 						return item;
+					}
 				} else {
-					if ( item->text( column ).contains( text ) && item->objectID() == obj_id )
-						return item;
+					if ( exact ) {
+						if ( item->text( column ) == text && item->uuid() == obj_id )
+							return item;
+					} else {
+						if ( item->text( column ).contains( text ) && item->uuid() == obj_id )
+							return item;
+					}
 				}
 			} else {
 				kdDebug() << "CAST ERROR: Item not of type KMFListViewItem" << endl;
@@ -1009,12 +1015,18 @@ KMFListViewItem* KMFListView::findKMFItem( const QString & text, int column, int
 			KMFListViewItem * item = 0;
 			item = dynamic_cast<KMFListViewItem*> ( qitem );
 			if ( item ) {
-				if ( exact ) {
-					if ( item->text( column ) == text && item->objectID() == obj_id )
+				if ( obj_id.isNull() ) {
+					if ( item->text( column ) == text ) {
 						return item;
+					}
 				} else {
-					if ( item->text( column ).contains( text ) && item->objectID() == obj_id )
-						return item;
+					if ( exact ) {
+						if ( item->text( column ) == text && item->uuid() == obj_id )
+							return item;
+					} else {
+						if ( item->text( column ).contains( text ) && item->uuid() == obj_id )
+							return item;
+					} 
 				}
 			} else {
 				kdDebug() << "CAST ERROR: Item not of type KMFListViewItem" << endl;
@@ -1025,9 +1037,9 @@ KMFListViewItem* KMFListView::findKMFItem( const QString & text, int column, int
 	return 0;
 }
 
-KMFListViewItem* KMFListView::findKMFItem( const QString & text1, int column1,const QString & text2, int column2, int obj_id ) {
+KMFListViewItem* KMFListView::findKMFItem( const QString & text1, int column1,const QString & text2, int column2, const QUuid& obj_id ) {
 	// 	kdDebug() << "KMFListViewItem* KMFListView::findKMFItem(const QString& text: " << text << ", int column:" << column << ", int obj_id: " << obj_id << ")" << endl;
-	if ( text1.isNull() || column1 < 0 || obj_id < -1 || text2.isNull() || column2 < 0 )
+	if ( text1.isNull() || column1 < 0 || obj_id.isNull() || text2.isNull() || column2 < 0 )
 		return 0;
 
 	QListViewItemIterator it( this );
@@ -1037,7 +1049,7 @@ KMFListViewItem* KMFListView::findKMFItem( const QString & text1, int column1,co
 		KMFListViewItem * item = 0;
 		item = dynamic_cast<KMFListViewItem*> ( qitem );
 		if ( item ) {
-			if ( item->text( column1 ) == text1 &&   item->text( column2 ) == text2 && item->objectID() == obj_id )
+			if ( item->text( column1 ) == text1 &&   item->text( column2 ) == text2 && item->uuid() == obj_id )
 				return item;
 		} else {
 			kdDebug() << "CAST ERROR: Item not of type KMFListViewItem" << endl;
@@ -1131,6 +1143,6 @@ void KMFListView::loadIcons() {
 }
 
 
-
+}
 
 #include "kmflistview.moc"
