@@ -4,7 +4,7 @@
 // Description:
 //
 //
-// Author: Christian Hubinger <chubinger@gmail.com>, (C) 2003
+// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2003
 //
 // Copyright: See COPYING file that comes with this distribution
 //
@@ -20,10 +20,11 @@
 
 
 #include "kmfmainwindow.h"
-
+namespace KMF {
 KMFMainWindow::KMFMainWindow( QWidget* parent, const char *name ) : KMyFirewallInterface(),  KParts::MainWindow( parent, name ) {}
 
 KMFMainWindow::~KMFMainWindow() {}
 
+}
 
 #include "kmfmainwindow.moc"
