@@ -4,7 +4,7 @@
 // Description:
 //
 //
-// Author: Christian Hubinger <chubinger@gmail.com>, (C) 2003
+// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2003
 //
 // Copyright: See COPYING file that comes with this distribution
 //
@@ -23,11 +23,14 @@
 #define KMFMAINWINDOW_H
 
 #include <kparts/mainwindow.h>
-#include "../interfaces/kmyfirewallinterface.h"
+#include "../core/kmyfirewallinterface.h"
+#include "../core/netfilterobject.h"
 
 /**
 @author Christian Hubinger
 */
+
+namespace KMF {
 class KDE_EXPORT KMFMainWindow : public KParts::MainWindow, virtual public KMyFirewallInterface {
 	Q_OBJECT
 public:
@@ -35,9 +38,9 @@ public:
 	KMFMainWindow( QWidget* parent, const char *name );
 	virtual ~KMFMainWindow();
 signals:
-	void sigHasDoc( bool );
 	void sigUpdateView();
+	void sigUpdateView( NetfilterObject*);
 	void sigEnableActions( bool );
 };
-
+}
 #endif
