@@ -4,7 +4,7 @@
 // Description:
 //
 //
-// Author: Christian Hubinger <chubinger@gmail.com>, (C) 2004
+// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2004
 //
 // Copyright: See COPYING file that comes with this distribution
 //
@@ -16,6 +16,7 @@
 
 // QT includes
 #include <qwidget.h>
+#include <qguardedptr.h>
 
 // KDE includes
 #include <kdemacros.h>
@@ -24,9 +25,13 @@
 #include "../core/kmfnetzone.h"
 #include "../core/ipaddress.h"
 
+class QFocusEvent;
+
 /**
 @author Christian Hubinger
 */
+
+namespace KMF {
 class KDE_EXPORT KMFNetworkWidget : public KMyFirewallNetworkWidget {
 	Q_OBJECT
 public:
@@ -37,18 +42,24 @@ public:
 	void allowEdit( bool );
 private slots:
 	void slotAddressChanged( const QString& );
-	void slotLongMaskChanged( const QString& );
+	void slotDescriptionChanged();
 	void slotShortMaskChanged( int );
+	void slotZoneDeleted();
+	
+protected:
+	void focusOutEvent( QFocusEvent * );
 	
 private:
 	void blockAllSignals( bool );	
 
 private:
-	KMFNetZone *m_zone;
+	QGuardedPtr<KMFNetZone> m_zone;
 	bool m_allowEdit;
 
+
+
 signals:
 	void sigZoneChanged( KMFNetZone* );
 };
-
+}
 #endif
