@@ -1,7 +1,7 @@
 /***************************************************************************
   begin                : Fri Nov 1 2002
   copyright            : (C) 2002 by Christian Hubinger
-  email                : chubinger@gmail.com
+  email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -23,6 +23,8 @@
 /**Widget for input of portnumbers, portranges or names
   *@author Christian Hubinger
   */
+  
+namespace KMF {
 class KMFError;
 
 class KDE_EXPORT KMFPortWidget : public KMyFirewallPortWidget {
@@ -39,5 +41,5 @@ private:
 signals:
 	void sigMultiPortChanged( bool );
 };
-
+}
 #endif
