@@ -24,7 +24,7 @@
 // Project Includes
 
 
-
+namespace KMF {
 class KDE_EXPORT KMFSelectInterface : public KMyFirewallSelectInterface {
 	Q_OBJECT
 
@@ -48,6 +48,6 @@ private: // DATA
 	 QStringList m_descs;
 	
 };
-
+}
 #endif
 
