@@ -4,7 +4,7 @@
 // Description:
 //
 //
-// Author: Christian Hubinger <chubinger@gmail.com>, (C) 2004
+// Author: Christian Hubinger <chubinger@irrsinnig.org>, (C) 2004
 //
 // Copyright: See COPYING file that comes with this distribution
 // License: GPL
@@ -27,7 +27,7 @@
 
 
 class QListBoxItem;
-
+namespace KMF {
 class KDE_EXPORT KMFTemplateChooser : public KMyFirewallTemplateChooser {
 	Q_OBJECT
 
@@ -41,7 +41,8 @@ public:
 	
 public slots:
 	/*$PUBLIC_SLOTS$*/
-
+	void parseTemplates();
+	
 protected:
 	/*$PROTECTED_FUNCTIONS$*/
 
@@ -54,7 +55,7 @@ protected slots:
 	void slotNewTemplateSelected( QListBoxItem* );
 	
 private:
-	void parseTemplates();
+	
 	void parseFile( const QString& );
 	
 private: // data
@@ -63,8 +64,9 @@ private: // data
 	
 signals:
     void sigLoadTemplate( const QString& );
+    void sigLoadEmptyDocument();
  		
 };
-
+}
 #endif
 
