@@ -1,7 +1,7 @@
 /***************************************************************************
  begin                : Sun Jan 27 21:17:58 CET 2002
  copyright            : (C) 2002 by Christian Hubinger
- email                : chubinger@gmail.com
+ email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -38,15 +38,15 @@ static const char description[] = I18N_NOOP( "KMyFirewall - IPTables configurati
 static KCmdLineOptions options[] =
     {
         { "+[URL]", I18N_NOOP( "Files/URLs to Open." ), 0 },
-		{ "--test ", I18N_NOOP("A short option which takes an argument."), 0 }, 
-        { 0, 0, 0}
+	{ "i <interfacename>", I18N_NOOP("Select the interface to use at startup. Use 'iptables' for the iptables/netfilter interface or 'generic' for the generic GUI. By default the interface selection dialog is shown if the user did not specify to allways use one of the interfaces by default."), 0 },
+	KCmdLineLastOption
     };
 
 int main( int argc, char *argv[] ) {
 	KAboutData aboutData( "kmyfirewall", I18N_NOOP( "KMyFirewall" ),
 	                      KMYFIREWALL_VERSION, description, KAboutData::License_GPL,
-	                      COPYRIGHT_STRING, 0, 0, "chubinger@gmail.com" );
-	aboutData.addAuthor( "Christian Hubinger", 0, "chubinger@gmail.com" );
+	                      COPYRIGHT_STRING, 0, 0, "chubinger@irrsinnig.org" );
+	aboutData.addAuthor( "Christian Hubinger", 0, "chubinger@irrsinnig.org" );
 	aboutData.addCredit( "Benjamin Meyer", I18N_NOOP("A handful of fixes and code cleanup."),"ben+kmyfirewall@meyerhome.net");
 	aboutData.addCredit( "Casper Planken", I18N_NOOP("Gave me some really good hints about how to do certain things"), "cplanken@chello.nl" );
 	aboutData.addCredit( "Malcolm Hunter", I18N_NOOP("Spell Checking and making messages real English"), "malcolm.hunter@gmx.co.uk" );
@@ -57,19 +57,19 @@ int main( int argc, char *argv[] ) {
 	KCmdLineArgs::init( argc, argv, &aboutData );
 	KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.
 
-	KMFApp a;
+	KMF::KMFApp a;
 	
 	// Splash screen
-	KMFConfig::self()->readConfig();
-	if ( KMFConfig::showSplash() ) {
-		showSplash();
+	KMF::KMFConfig::self()->readConfig();
+	if ( KMF::KMFConfig::showSplash() ) {
+		KMF::showSplash();
 	}
 	
-	KMyFirewall * kmyfirewall = new KMyFirewall;
+	KMF::KMyFirewall * kmyfirewall = new KMF::KMyFirewall;
 	a.setMainWidget( kmyfirewall );
 	kmyfirewall->show();
 	
-	closeSplash();
+	KMF::closeSplash();
 
 
 
