@@ -21,9 +21,11 @@
   *@author Christian Hubinger
   */
 
-class IPTRule;
+
 class QString;
 
+namespace KMF {
+class IPTRule;
     
 class KMFRuleEditCustomOpt : public KMyFirewallRuleEditorCustomRule  {
    Q_OBJECT
@@ -41,11 +43,11 @@ private:
 	IPTRule* m_rule;
 
 signals:
-	void sigAddRuleOpt( QString*, QPtrList<QString>* );
-	void sigAddTargetOpt( QString*, QPtrList<QString>* );
+/*	void sigAddRuleOpt( QString*, QPtrList<QString>* );
+	void sigAddTargetOpt( QString*, QPtrList<QString>* );*/
 	void sigHideMe();
 
 
 };
-
+}
 #endif
