@@ -1,7 +1,7 @@
 /***************************************************************************
   begin                : Fri Nov 1 2002
   copyright            : (C) 2002 by Christian Hubinger
-  email                : chubinger@gmail.com
+  email                : chubinger@irrsinnig.org
 ***************************************************************************/
 
 /***************************************************************************
@@ -31,16 +31,19 @@
 #include <qgroupbox.h>
 
 // project includes
+#include "../../core/xmlnames.h"
 #include "../../core/iptruleoption.h"
 #include "../../core/iptrule.h"
 #include "../../core/iptchain.h"
 #include "../../core/iptable.h"
 #include "../../core/kmfdoc.h"
 #include "../../core/kmfiptdoc.h"
-
+#include "../../core/kmfnetwork.h"
+#include "../../core/kmfundoengine.h"
+#include "../../core/xmlnames.h"
 #include "../../kmfwidgets/kmfportwidget.h"
 #include "../../kmfwidgets/kmfmultiportwidget.h"
-
+namespace KMF {
 KMFRuleEditorProtocol::KMFRuleEditorProtocol( QWidget *parent, const char *name, WFlags fl ) : KMyFirewallRuleEditorProtocol( parent, name, fl ) {
 	use_multiport = false;
 	cb_icmp_type->insertItem( "echo-request" );
@@ -232,7 +235,7 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 	vals = option->getValues();
 	enable = "";
 	enable = *vals.at(0);
-	if ( enable == "bool:on" ) {
+	if ( enable == XML::BoolOn_Value ) {
 		c_tcp->setChecked( true );
 		c_udp->setEnabled( false );
 		finished =true;
@@ -246,7 +249,7 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 		vals = option->getValues();
 		enable = "";
 		enable = *vals.at(0);
-		if ( enable == "bool:on" ) {
+		if ( enable == XML::BoolOn_Value ) {
 			c_udp->setChecked( true );
 			c_tcp->setEnabled( false );
 			finished =true;
@@ -265,17 +268,17 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 	dest_port = *vals.at(2);
 	equ_port = *vals.at(3);
 
-	if ( !src_port.isEmpty() &&  src_port != "UNDEFINED" && src_port != "bool:off")  {
+	if ( !src_port.isEmpty() &&  src_port != XML::Undefined_Value && src_port != XML::BoolOff_Value)  {
 		mpwid->setType( "src" );
 		mpwid->loadPortString( src_port );
 		kdDebug() << "Option Value -sport: " << src_port << endl;
 	}
-	if ( !dest_port.isEmpty() &&  dest_port != "UNDEFINED"  && dest_port != "bool:off" ) {
+	if ( !dest_port.isEmpty() &&  dest_port != XML::Undefined_Value  && dest_port != XML::BoolOff_Value ) {
 		mpwid->setType( "dest" );
 		mpwid->loadPortString( dest_port );
 		kdDebug() << "Option Value -sport: " << dest_port << endl;
 	}
-	if ( !equ_port.isEmpty() &&  equ_port != "UNDEFINED"  && equ_port != "bool:off" ) {
+	if ( !equ_port.isEmpty() &&  equ_port != XML::Undefined_Value  && equ_port != XML::BoolOff_Value ) {
 		mpwid->setType( "equ" );
 		mpwid->loadPortString( equ_port );
 		kdDebug() << "Option Value -sport: " << equ_port << endl;
@@ -294,7 +297,7 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 	dest_port = *vals.at(2);
 	flags = *vals.at(3);
 	tcp_option = *vals.at(4);
-	if ( enable  == "bool:on" ) {
+	if ( enable  == XML::BoolOn_Value ) {
 		toggleMultiPort( false );
 		toggleTcpRule( true );
 		c_tcp->setChecked( true );
@@ -305,17 +308,17 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 		finished = true;
 		tabWidget->showPage( tab_ports );
 	}
-	if ( !src_port.isEmpty() &&  src_port != "UNDEFINED" && src_port != "bool:off")  {
+	if ( !src_port.isEmpty() &&  src_port != XML::Undefined_Value && src_port != XML::BoolOff_Value)  {
 		kdDebug() << "Option Value -sport: " << src_port << endl;
 		c_src_port->setChecked( true );
 		pw_src_port->loadPortString( src_port );
 	}
-	if ( !dest_port.isEmpty() &&  dest_port != "UNDEFINED"  && dest_port != "bool:off" ) {
+	if ( !dest_port.isEmpty() &&  dest_port != XML::Undefined_Value  && dest_port != XML::BoolOff_Value ) {
 		kdDebug() << "Option Value -dport: " << dest_port << endl;
 		c_dest_port->setChecked( true );
 		pw_dest_port->loadPortString( dest_port );
 	}
-	if ( !flags.isEmpty() &&  flags != "UNDEFINED" && flags != "bool:off"  ) {
+	if ( !flags.isEmpty() &&  flags != XML::Undefined_Value && flags != XML::BoolOff_Value  ) {
 		kdDebug() << "Option Value --tcp-flags: " << flags << endl;
 		gb_mask->setEnabled( true );
 		gb_comp->setEnabled( true );
@@ -366,7 +369,7 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 		if ( comp_lst.contains( "ALL" ) > 0 )
 			c_all_comp->setChecked( true );
 	}
-	if ( !tcp_option.isEmpty() &&  tcp_option != "UNDEFINED" && tcp_option != "bool:off"  ) {
+	if ( !tcp_option.isEmpty() &&  tcp_option != XML::Undefined_Value && tcp_option != XML::BoolOff_Value  ) {
 		kdDebug() << "Option Value --tcp-options: " << tcp_option << endl;
 		c_tcp_option->setChecked( true );
 		sb_tcp_option_num->setEnabled( true );
@@ -392,7 +395,7 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 	src_port = *vals.at(1);
 	dest_port = *vals.at(2);
 
-	if ( enable == "bool:on" ) {
+	if ( enable == XML::BoolOn_Value ) {
 		finished =true;
 		toggleMultiPort( false );
 		c_udp->setChecked( true );
@@ -404,12 +407,12 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 		tabWidget->showPage( tab_ports );
 	}
 
-	if ( !src_port.isEmpty() &&  src_port != "UNDEFINED" && src_port != "bool:off")  {
+	if ( !src_port.isEmpty() &&  src_port != XML::Undefined_Value && src_port != XML::BoolOff_Value)  {
 		kdDebug() << "Option Value -sport: " << src_port << endl;
 		c_src_port->setChecked( true );
 		pw_src_port->loadPortString( src_port );
 	}
-	if ( !dest_port.isEmpty() &&  dest_port != "UNDEFINED"  && dest_port != "bool:off" ) {
+	if ( !dest_port.isEmpty() &&  dest_port != XML::Undefined_Value  && dest_port != XML::BoolOff_Value ) {
 		kdDebug() << "Option Value -dport: " << dest_port << endl;
 		c_dest_port->setChecked( true );
 		pw_dest_port->loadPortString( dest_port );
@@ -424,9 +427,9 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 	enable = *vals.at(0);
 	type = *vals.at(1);
 
-	if ( enable == "bool:on" ) {
+	if ( enable == XML::BoolOn_Value ) {
 		c_icmp->animateClick();
-		if ( type != "UNDEFINED" && !type.isEmpty() ) {
+		if ( type != XML::Undefined_Value && !type.isEmpty() ) {
 			if ( type.startsWith( "! " ) ) {
 				c_inv_icmp->setChecked( true );
 				type = type.right( type.length() - 2 );
@@ -463,8 +466,10 @@ void KMFRuleEditorProtocol::loadRule( IPTRule* rule ) {
 
 void KMFRuleEditorProtocol::accept() {
 	kdDebug() << "void KMFRuleEditorProtocol::accept()" << endl;
-	m_rule->chain()->table()->kmfDoc()->startTransaction();
-	m_rule->saveState();
+	KMFUndoEngine::instance()->startTransaction( 
+		m_rule,
+		i18n("Edit Rule: %1 Protocol Option").arg( m_rule->name() ) 
+	);
 
 	///////////////////////////////////////////////////////
 	// set options ++++++++++++++++++++++++++++++++++++++++
@@ -475,14 +480,14 @@ void KMFRuleEditorProtocol::accept() {
 	QString* icmp_name = new QString("icmp_opt");
 	QString* all_name = new QString("all_prot_opt");
 	QPtrList<QString>* empty_opt = new QPtrList<QString>;
-	empty_opt->append( new QString("bool:off") ); 
-	emit sigAddRuleOpt( tcp_multiport_name, empty_opt );
-	emit sigAddRuleOpt( udp_multiport_name, empty_opt );
-	emit sigAddRuleOpt( tcp_name, empty_opt );
-	emit sigAddRuleOpt( udp_name, empty_opt );
-	emit sigAddRuleOpt( icmp_name, empty_opt );
-	emit sigAddRuleOpt( all_name, empty_opt );
-
+	empty_opt->append( new QString(XML::BoolOff_Value) ); 
+	m_rule->addRuleOption( *tcp_multiport_name, *empty_opt );
+	m_rule->addRuleOption( *udp_multiport_name, *empty_opt );
+	m_rule->addRuleOption( *tcp_name, *empty_opt );
+	m_rule->addRuleOption( *udp_name, *empty_opt );
+	m_rule->addRuleOption( *icmp_name, *empty_opt );
+	m_rule->addRuleOption( *all_name, *empty_opt );
+	
 	if ( multiportEnabled() ) {
 		kdDebug() << "You like a Multiport Rule" << endl;
 		QPtrList<QString>* options = new QPtrList<QString>;
@@ -494,38 +499,39 @@ void KMFRuleEditorProtocol::accept() {
 			QString type = "";
 			type = mpwid->type();
 			QString* opt = new QString( src );
-			options->append( new QString("bool:on") );
+			options->append( new QString(XML::BoolOn_Value) );
 			if ( type == "src" ) {
 				options->append( new QString( src ) );
 			} else if ( type == "dest" ) {
-				options->append( new QString("bool:off") );
+				options->append( new QString(XML::BoolOff_Value) );
 				options->append( new QString( src ) );
 			} else if ( type == "equ" ) {
-				options->append( new QString("bool:off") );
-				options->append( new QString("bool:off") );
+				options->append( new QString(XML::BoolOff_Value) );
+				options->append( new QString(XML::BoolOff_Value) );
 				options->append( new QString( src ) );
 			}
 			kdDebug() << "Found Option Type: " << *opt << endl;
 		}
 		if ( c_tcp->isChecked() ) {
-			emit sigAddRuleOpt( tcp_multiport_name, options );
+			m_rule->addRuleOption( *tcp_multiport_name, *options );
 			options->clear();
-			m_rule->chain()->table()->kmfDoc()->endTransaction();
+			KMFUndoEngine::instance()->endTransaction();
 			emit sigHideMe();
 
 		} else if ( c_udp->isChecked() ) {
-			emit sigAddRuleOpt( udp_multiport_name, options );
+			m_rule->addRuleOption( *udp_multiport_name, *options );
 			options->clear();
-			m_rule->chain()->table()->kmfDoc()->endTransaction();
+			KMFUndoEngine::instance()->endTransaction();
 			emit sigHideMe();
 		}
+		return;
 	}
 
 
 	if ( c_tcp->isChecked() ) {
 		kdDebug() << "You like a TCP Rule" << endl;
 		QPtrList<QString>* options = new QPtrList<QString>;
-		options->append( new QString( "bool:on" ) );
+		options->append( new QString( XML::BoolOn_Value ) );
 		QString src = "";
 		QString dest = "";
 		QString flags = "";
@@ -537,7 +543,7 @@ void KMFRuleEditorProtocol::accept() {
 			if ( !s->isEmpty() )
 				src = *s;
 			if ( src.isEmpty() ) {
-				m_rule->chain()->table()->kmfDoc()->endTransaction();
+				KMFUndoEngine::instance()->abortTransaction();
 				return ;
 			}
 		}
@@ -547,7 +553,7 @@ void KMFRuleEditorProtocol::accept() {
 			if ( !s->isEmpty() )
 				dest = *s;
 			if ( dest.isEmpty() ) {
-				m_rule->chain()->table()->kmfDoc()->endTransaction();
+				KMFUndoEngine::instance()->abortTransaction();
 				return ;
 			}
 		}
@@ -607,7 +613,7 @@ void KMFRuleEditorProtocol::accept() {
 				KMessageBox::sorry( this, i18n( "<qt><p>You need to select the TCP flags you like to "
 												"check and those your rule should match.</qt>" ) );
 
-				m_rule->chain()->table()->kmfDoc()->endTransaction();
+				KMFUndoEngine::instance()->abortTransaction();
 				return ;
 			}
 		}
@@ -618,22 +624,20 @@ void KMFRuleEditorProtocol::accept() {
 			}
 		}
 
-//		if ( !src.isEmpty() || ! dest.isEmpty() || ! flags.isEmpty() || ! tcp_option.isEmpty() ) {
-//			options->append( new QString("bool:on") );
+
 			if ( src.isEmpty() )
-				src = "bool:off";
+				src = XML::BoolOff_Value;
 			options->append( new QString( src )  );
 			if ( dest.isEmpty()  )
-				dest = "bool:off";
+				dest = XML::BoolOff_Value;
 			options->append( new QString( dest )  );
 			if ( flags.isEmpty() )
-				flags = "bool:off";
+				flags = XML::BoolOff_Value;
 			options->append( new QString( flags )  );
 			if ( tcp_option.isEmpty() )
-				tcp_option = "bool:off";
+				tcp_option = XML::BoolOff_Value;
 			options->append( new QString( tcp_option )  );
-//		}
-		emit sigAddRuleOpt( tcp_name, options );
+		m_rule->addRuleOption( *tcp_name, *options );
 	}
 	//////////////////////////////////////////////////////////////////
 	// start udp options
@@ -642,7 +646,7 @@ void KMFRuleEditorProtocol::accept() {
 		QPtrList<QString>* options = new QPtrList<QString>;
 		QString src = "";
 		QString dest = "";
-		options->append( new QString( "bool:on" ) );
+		options->append( new QString( XML::BoolOn_Value ) );
 
 		if ( c_src_port->isChecked() ) {
 			QString * s = new QString( "" );
@@ -650,7 +654,7 @@ void KMFRuleEditorProtocol::accept() {
 			if ( !s->isEmpty() )
 				src = *s;
 			if ( src.isEmpty() ) {
-				m_rule->chain()->table()->kmfDoc()->endTransaction();
+				KMFUndoEngine::instance()->abortTransaction();
 				return ;
 			}
 		}
@@ -660,21 +664,22 @@ void KMFRuleEditorProtocol::accept() {
 			if ( !s->isEmpty() )
 				dest = *s;
 			if ( dest.isEmpty() ) {
-				m_rule->chain()->table()->kmfDoc()->endTransaction();
+				KMFUndoEngine::instance()->abortTransaction();
 				return ;
 			}
 		}
 		if ( !src.isEmpty() || !dest.isEmpty() ) {
-//			options->append( new QString("bool:on") );
+//			options->append( new QString(XML::BoolOn_Value) );
 			if ( src.isEmpty() )
-				src = "bool:off";
+				src = XML::BoolOff_Value;
 			options->append( new QString( src ) );
 			if ( dest.isEmpty() )
-				dest = "bool:off";
+				dest = XML::BoolOff_Value;
 			options->append( new QString( dest ) );
 		}
 
-		emit sigAddRuleOpt( udp_name, options );
+		// emit sigAddRuleOpt( udp_name, options );
+		m_rule->addRuleOption( *udp_name, *options );
 		options->clear();
 	}
 
@@ -688,21 +693,23 @@ void KMFRuleEditorProtocol::accept() {
 				opt->prepend( "! " );
 			}
 		}
-		options->append( new QString("bool:on") );
+		options->append( new QString(XML::BoolOn_Value) );
 		options->append( opt );
-		emit sigAddRuleOpt( icmp_name, options );
+		// emit sigAddRuleOpt( icmp_name, options );
+		m_rule->addRuleOption( *icmp_name, *options );
 		options->clear();
 	}
 
 	if ( c_all->isChecked() ) {
 		kdDebug() << "You like an ALL Rule" << endl;
 		QPtrList<QString>* options = new QPtrList<QString>;
-		options->append( new QString("bool:on") );
-		emit sigAddRuleOpt( all_name, options );
+		options->append( new QString(XML::BoolOn_Value) );
+		// emit sigAddRuleOpt( all_name, options );
+		m_rule->addRuleOption( *all_name, *options );
 		options->clear();
 	}
 
-	m_rule->chain()->table()->kmfDoc()->endTransaction();
+	KMFUndoEngine::instance()->endTransaction();
 	emit sigHideMe();
 }
 
@@ -713,4 +720,6 @@ void KMFRuleEditorProtocol::reject() {
 	emit sigHideMe();
 }
 
+}
+
 #include "kmfruleeditorprotocol.moc"
